@@ -2,7 +2,7 @@
  * Routines for h245 packet dissection
  * Copyright 2004, Anders Broman <anders.broman@ericsson.com>
  *
- * $Id: packet-h245-template.c 25164 2008-04-24 06:36:42Z kukosa $
+ * $Id: packet-h245-template.c 27390 2009-02-07 14:33:47Z wmeier $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -38,7 +38,6 @@
 
 #include <glib.h>
 #include <epan/packet.h>
-#include <epan/conversation.h>
 #include <epan/strutil.h>
 
 #include <stdio.h>
@@ -71,7 +70,6 @@ static dissector_table_t gef_name_dissector_table;
 static dissector_table_t gef_content_dissector_table;
 static dissector_handle_t nsp_handle;
 static dissector_handle_t data_handle;
-static dissector_handle_t h245_handle;
 static dissector_handle_t MultimediaSystemControlMessage_handle;
 static dissector_handle_t h263_handle = NULL;
 static dissector_handle_t amr_handle = NULL;
@@ -252,6 +250,7 @@ static const value_string h245_RFC_number_vals[] = {
 	{  2429,	"RFC 2429 - 1998 Version of ITU-T Rec. H.263 Video (H.263+)" },
 	{  3016,	"RFC 3016 - RTP Payload Format for MPEG-4 Audio/Visual Streams" },
 	{  3267,	"RFC 3267 - Adaptive Multi-Rate (AMR) and Adaptive Multi-Rate Wideband (AMR-WB)" },
+	{  3984,	"RFC 3984 - RTP Payload Format for H.264 Video" },
 	{  0, NULL }
 };
 
@@ -554,6 +553,8 @@ void proto_register_h245(void) {
 
 /*--- proto_reg_handoff_h245 ---------------------------------------*/
 void proto_reg_handoff_h245(void) {
+	dissector_handle_t h245_handle;
+
 	rtp_handle = find_dissector("rtp");
 	rtcp_handle = find_dissector("rtcp");
 	t38_handle = find_dissector("t38");
@@ -562,9 +563,9 @@ void proto_reg_handoff_h245(void) {
 	amr_handle = find_dissector("amr_if2_nb");
 
 
-	h245_handle=create_dissector_handle(dissect_h245, proto_h245);
+	h245_handle = find_dissector("h245");
 	dissector_add_handle("tcp.port", h245_handle);
-	MultimediaSystemControlMessage_handle=create_dissector_handle(dissect_h245_h245, proto_h245);
+	MultimediaSystemControlMessage_handle = find_dissector("h245dg");
 	dissector_add_handle("udp.port", MultimediaSystemControlMessage_handle);
 }
 
