@@ -2,7 +2,7 @@
  * Written by Shaun Jackman <sjackman@gmail.com>.
  * Copyright 2007 Shaun Jackman
  *
- * $Id: packet-mpeg-pes-template.c 28405 2009-05-18 23:55:55Z gerald $
+ * $Id: packet-mpeg-pes-template.c 27322 2009-01-28 20:52:52Z wmeier $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -32,7 +32,6 @@
 #include <glib.h>
 
 #include <epan/packet.h>
-#include <epan/prefs.h>
 #include <epan/asn1.h>
 
 #include "packet-per.h"
@@ -408,15 +407,7 @@ dissect_mpeg_pes(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
 			offset = dissect_mpeg_pes_Stream(tvb, offset, &asn1_ctx,
 					tree, hf_mpeg_pes_extension);
-			/* https://bugs.wireshark.org/bugzilla/show_bug.cgi?id=2229 
-			 * A value of 0 indicates that the PES packet length is neither specified nor 
-			 * bounded and is allowed only in PES packets whose payload is a video elementary 
-			 * stream contained in Transport Stream packets. 
-			 * XXX Some one with access to the spec should check this 
-			 */ 
-			 if(length !=0 && stream != STREAM_VIDEO){
-				 length -= 5 * 8;
-			 }
+			length -= 5 * 8;
 
 			header_length = tvb_get_guint8(tvb, 8);
 			if (header_length > 0) {
@@ -425,16 +416,7 @@ dissect_mpeg_pes(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 						header_length, header_length);
 				dissect_mpeg_pes_header_data(header_data, pinfo, tree, flags);
 				offset += header_length * 8;
-				 /* lenght may be zero for Video stream */
-				if(length !=0 && stream != STREAM_VIDEO){
-					length -= header_length * 8;
-				}
-			}
-
-			/* lenght may be zero for Video stream */ 
-			if(length==0){ 
-				proto_tree_add_item(tree, hf_mpeg_pes_data, tvb, (offset>>3),-1, FALSE);
-				return TRUE;
+				length -= header_length * 8;
 			}
 
 			es = tvb_new_subset(tvb, offset / 8, -1, length / 8);
@@ -585,9 +567,8 @@ proto_register_mpeg_pes(void)
 void
 proto_reg_handoff_mpeg_pes(void)
 {
-	dissector_handle_t mpeg_handle = create_dissector_handle(
-			dissect_mpeg, proto_mpeg);
-	dissector_add("wtap_encap", WTAP_ENCAP_MPEG, mpeg_handle);
+	dissector_handle_t mpeg_handle = find_dissector("mpeg");
 
+	dissector_add("wtap_encap", WTAP_ENCAP_MPEG, mpeg_handle);
 	heur_dissector_add("mpeg", dissect_mpeg_pes, proto_mpeg_pes);
 }
