@@ -1,8 +1,8 @@
-/* packet-nbap.c
+/* packet-nbap-template.c
  * Routines for UMTS Node B Application Part(NBAP) packet dissection
- * Copyright 2005, Anders Broman <anders.broman@ericsson.com>
+ * Copyright 2005, 2009 Anders Broman <anders.broman@ericsson.com>
  *
- * $Id: packet-nbap-template.c 26703 2008-11-05 21:43:05Z wmeier $
+ * $Id: packet-nbap-template.c 27390 2009-02-07 14:33:47Z wmeier $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -31,7 +31,6 @@
 
 #include <glib.h>
 #include <epan/packet.h>
-#include <epan/conversation.h>
 
 #include <stdio.h>
 #include <string.h>
