@@ -17,7 +17,7 @@
  * See RFC 2578 for Structure of Management Information Version 2 (SMIv2)
  * Copyright (C) 2007 Luis E. Garcia Ontanon <luis@ontanon.org>
  *
- * $Id: packet-snmp-template.c 27037 2008-12-17 17:23:37Z wmeier $
+ * $Id: packet-snmp-template.c 27719 2009-03-13 22:21:22Z stig $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -1812,12 +1812,12 @@ static void* snmp_users_copy_cb(void* dest, const void* orig, unsigned len _U_)
 
 static void snmp_users_free_cb(void* p) {
 	snmp_ue_assoc_t* ue = p;
-	if (ue->user.userName.data) g_free(ue->user.userName.data);
-	if (ue->user.authPassword.data) g_free(ue->user.authPassword.data);
-	if (ue->user.privPassword.data) g_free(ue->user.privPassword.data);
-	if (ue->user.authKey.data) g_free(ue->user.authKey.data);
-	if (ue->user.privKey.data) g_free(ue->user.privKey.data);
-	if (ue->engine.data) g_free(ue->engine.data);
+	g_free(ue->user.userName.data);
+	g_free(ue->user.authPassword.data);
+	g_free(ue->user.privPassword.data);
+	g_free(ue->user.authKey.data);
+	g_free(ue->user.privKey.data);
+	g_free(ue->engine.data);
 }
 
 static void snmp_users_update_cb(void* p _U_, const char** err) {
@@ -1935,12 +1935,12 @@ void proto_register_snmp(void) {
   };
   module_t *snmp_module;
   static uat_field_t users_fields[] = {
-	  UAT_FLD_BUFFER(snmp_users,engine_id,"Engine-id for this entry (empty = any)"),
-	  UAT_FLD_LSTRING(snmp_users,userName,"The username"),
-	  UAT_FLD_VS(snmp_users,auth_model,auth_types,"Algorithm to be used for authentication."),
-	  UAT_FLD_LSTRING(snmp_users,authPassword,"The password used for authenticating packets for this entry"),
-	  UAT_FLD_VS(snmp_users,priv_proto,priv_types,"Algorithm to be used for privacy."),
-	  UAT_FLD_LSTRING(snmp_users,privPassword,"The password used for encrypting packets for this entry"),
+	  UAT_FLD_BUFFER(snmp_users,engine_id,"Engine ID","Engine-id for this entry (empty = any)"),
+	  UAT_FLD_LSTRING(snmp_users,userName,"Username","The username"),
+	  UAT_FLD_VS(snmp_users,auth_model,"Authentication model",auth_types,"Algorithm to be used for authentication."),
+	  UAT_FLD_LSTRING(snmp_users,authPassword,"Password","The password used for authenticating packets for this entry"),
+	  UAT_FLD_VS(snmp_users,priv_proto,"Privacy protocol",priv_types,"Algorithm to be used for privacy."),
+	  UAT_FLD_LSTRING(snmp_users,privPassword,"Privacy password","The password used for encrypting packets for this entry"),
 	  UAT_END_FIELDS
   };
 
