@@ -3,7 +3,7 @@
  *
  * Copyright 2004 Ian Schorr
  *
- * $Id: capinfos.c 25640 2008-06-30 17:16:29Z morriss $
+ * $Id: capinfos.c 27456 2009-02-15 21:47:57Z guy $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -267,6 +267,7 @@ usage(gboolean is_error)
   fprintf(output, "If no options are given, default is to display all infos\n");
 }
 
+#ifdef HAVE_PLUGINS
 /*
  *  Don't report failures to load plugins because most (non-wiretap) plugins
  *  *should* fail to load (because we're not linked against libwireshark and
@@ -277,6 +278,7 @@ failure_message(const char *msg_format _U_, va_list ap _U_)
 {
 	return;
 }
+#endif
 
 
 int
@@ -285,7 +287,6 @@ main(int argc, char *argv[])
   wtap *wth;
   int err;
   gchar *err_info;
-  extern char *optarg;
   extern int optind;
   int opt;
   int status = 0;
@@ -305,7 +306,7 @@ main(int argc, char *argv[])
 		g_warning("capinfos: init_progfile_dir(): %s", init_progfile_dir_error);
 		g_free(init_progfile_dir_error);
     } else {
-		init_report_err(failure_message,NULL,NULL);
+		init_report_err(failure_message,NULL,NULL,NULL);
 		init_plugins();
     }
 #endif
