@@ -1,6 +1,6 @@
 /* dumpcap.c
  *
- * $Id: dumpcap.c 26037 2008-08-19 05:10:16Z guy $
+ * $Id: dumpcap.c 27212 2009-01-11 18:16:50Z jake $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -411,6 +411,9 @@ print_statistics_loop(gboolean machine_readable)
     pcap_t      *pch;
     char        errbuf[PCAP_ERRBUF_SIZE];
     struct pcap_stat ps;
+#ifndef _WIN32
+    struct sigaction act;
+#endif	
 
     if_list = get_interface_list(&err, &err_str);
     if (if_list == NULL) {
@@ -453,6 +456,14 @@ print_statistics_loop(gboolean machine_readable)
             "Dropped");
     }
 
+#ifndef _WIN32
+    /* handle SIGPIPE signal to default action */
+    act.sa_handler = SIG_DFL;
+    sigemptyset(&act.sa_mask);
+    act.sa_flags = SA_RESTART;
+    sigaction(SIGPIPE,&act,NULL);
+#endif	
+
     global_ld.go = TRUE;
     while (global_ld.go) {
         for (stat_entry = g_list_first(stat_list); stat_entry != NULL; stat_entry = g_list_next(stat_entry)) {
@@ -1218,7 +1229,8 @@ capture_loop_open_input(capture_options *capture_opts, loop_data *ld,
     /* we've opened "iface" as a network device */
 #ifdef _WIN32
     /* try to set the capture buffer size */
-    if (pcap_setbuff(ld->pcap_h, capture_opts->buffer_size * 1024 * 1024) != 0) {
+    if (capture_opts->buffer_size > 1 &&
+	pcap_setbuff(ld->pcap_h, capture_opts->buffer_size * 1024 * 1024) != 0) {
         sync_secondary_msg_str = g_strdup_printf(
           "The capture buffer size of %luMB seems to be too high for your machine,\n"
           "the default of 1MB will be used.\n"
@@ -1683,7 +1695,9 @@ capture_loop_open_output(capture_options *capture_opts, int *save_file_fd,
   char tmpname[128+1];
   gchar *capfile_name;
   gboolean is_tempfile;
-
+#ifndef _WIN32
+  int ret;
+#endif
 
   g_log(LOG_DOMAIN_CAPTURE_CHILD, G_LOG_LEVEL_DEBUG, "capture_loop_open_output: %s",
       (capture_opts->save_file) ? capture_opts->save_file : "");
@@ -1765,7 +1779,7 @@ capture_loop_open_output(capture_options *capture_opts, int *save_file_fd,
   /* capture_opts.save_file is "g_free"ed later, which is equivalent to
      "g_free(capfile_name)". */
 #ifndef _WIN32
-  fchown(*save_file_fd, capture_opts->owner, capture_opts->group);
+  ret = fchown(*save_file_fd, capture_opts->owner, capture_opts->group);
 #endif
 
   return TRUE;
@@ -2283,13 +2297,6 @@ capture_loop_packet_cb(u_char *user, const struct pcap_pkthdr *phdr,
   loop_data *ld = (void *) user;
   int err;
 
-  /* if the user told us to stop after x packets, do we already have enough? */
-  if ((ld->packet_max > 0) && (ld->packet_count >= ld->packet_max))
-  {
-     ld->go = FALSE;
-     return;
-  }
-
   /* We may be called multiple times from pcap_dispatch(); if we've set
      the "stop capturing" flag, ignore this packet, as we're not
      supposed to be saving any more packets. */
@@ -2303,8 +2310,14 @@ capture_loop_packet_cb(u_char *user, const struct pcap_pkthdr *phdr,
     if (!libpcap_write_packet(ld->pdh, phdr, pd, &ld->bytes_written, &err)) {
       ld->go = FALSE;
       ld->err = err;
-    } else
+    } else {
       ld->packet_count++;
+      /* if the user told us to stop after x packets, do we already have enough? */
+      if ((ld->packet_max > 0) && (ld->packet_count >= ld->packet_max))
+      {
+        ld->go = FALSE;
+      }
+    }
   }
 }
 
@@ -2319,8 +2332,7 @@ main(int argc, char *argv[])
 
 #ifdef _WIN32
   WSADATA              wsaData;
-#endif  /* _WIN32 */
-#ifndef _WIN32
+#else
   struct sigaction action, oldaction;
 #endif
 
