@@ -1,7 +1,7 @@
 /* Edit capture files.  We can delete packets, adjust timestamps, or
  * simply convert from one format to another format.
  *
- * $Id: editcap.c 27051 2008-12-18 19:24:34Z morriss $
+ * $Id: editcap.c 27456 2009-02-15 21:47:57Z guy $
  *
  * Originally written by Richard Sharpe.
  * Improved by Guy Harris.
@@ -428,7 +428,7 @@ main(int argc, char *argv[])
     g_warning("capinfos: init_progfile_dir(): %s", init_progfile_dir_error);
     g_free(init_progfile_dir_error);
   } else {
-    init_report_err(failure_message,NULL,NULL);
+    init_report_err(failure_message,NULL,NULL,NULL);
     init_plugins();
   }
 #endif
