@@ -1,7 +1,7 @@
 /* addr_resolv.c
  * Routines for network object lookup
  *
- * $Id: addr_resolv.c 26240 2008-09-20 14:24:29Z stig $
+ * $Id: addr_resolv.c 27718 2009-03-13 22:06:48Z stig $
  *
  * Laurent Deniel <laurent.deniel@free.fr>
  *
@@ -110,6 +110,7 @@
 #  define socklen_t unsigned int
 # endif
 # include <ares.h>
+# include <ares_version.h>
 #else
 # ifdef HAVE_GNU_ADNS
 #  include <errno.h>
@@ -306,7 +307,12 @@ typedef struct _c_ares_queue_msg
   int                 family;
 } c_ares_queue_msg_t;
 
+#if ( ( ARES_VERSION_MAJOR < 1 )                                     \
+ || ( 1 == ARES_VERSION_MAJOR && ARES_VERSION_MINOR < 5 ) )
+static void c_ares_ghba_cb(void *arg, int status, struct hostent *hostent);
+#else
 static void c_ares_ghba_cb(void *arg, int status, int timeouts _U_, struct hostent *hostent);
+#endif
 
 #else
 /* GNU ADNS */
@@ -668,23 +674,26 @@ static void fill_dummy_ip4(guint addr, hashipv4_t* volatile tp)
 #ifdef HAVE_C_ARES
 
 static void
+#if ( ( ARES_VERSION_MAJOR < 1 )                                     \
+ || ( 1 == ARES_VERSION_MAJOR && ARES_VERSION_MINOR < 5 ) )
+c_ares_ghba_cb(void *arg, int status, struct hostent *he) {
+#else
 c_ares_ghba_cb(void *arg, int status, int timeouts _U_, struct hostent *he) {
+#endif
   c_ares_queue_msg_t *caqm = arg;
   char **p;
-  char name[MAXNAMELEN];
 
   if (!caqm) return;
   c_ares_in_flight--;
 
   if (status == ARES_SUCCESS) {
     for (p = he->h_addr_list; *p != NULL; p++) {
-      inet_ntop(he->h_addrtype, *p, name, sizeof(name));
       switch(caqm->family) {
         case AF_INET:
-          add_ipv4_name(caqm->addr.ip4, name);
+          add_ipv4_name(caqm->addr.ip4, he->h_name);
           break;
         case AF_INET6:
-          add_ipv6_name(&caqm->addr.ip6, name);
+          add_ipv6_name(&caqm->addr.ip6, he->h_name);
           break;
         default:
           /* Throw an exception? */
@@ -1952,8 +1961,7 @@ read_hosts_file (const char *hostspath)
         add_ipv4_name(host_addr[0], cp);
     }
   }
-  if (line != NULL)
-    g_free(line);
+  g_free(line);
 
   fclose(hf);
   return TRUE;
@@ -2019,8 +2027,7 @@ read_subnets_file (const char *subnetspath)
 
     subnet_entry_set(host_addr, (guint32)mask_length, cp);
   }
-  if (line != NULL)
-    g_free(line);
+  g_free(line);
 
   fclose(hf);
   return TRUE;
@@ -2174,7 +2181,7 @@ static guint32 get_subnet_mask(guint32 mask_length) {
     }
 }
 
-static void subnet_name_lookup_init()
+static void subnet_name_lookup_init(void)
 {
     gchar* subnetspath;
 
