@@ -1,7 +1,7 @@
 /* base64.c
  * Base-64 conversion
  *
- * $Id: base64.c 24654 2008-03-16 09:54:00Z jake $
+ * $Id: base64.c 25912 2008-08-04 02:31:35Z wmeier $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -66,3 +66,21 @@ size_t epan_base64_decode(char *s)
 
 	return i*3/4;
 }
+
+/* Return a tvb that contains the binary representation of a base64
+   string */
+
+tvbuff_t *
+base64_to_tvb(const char *base64)
+{
+  tvbuff_t *tvb;
+  char *data = g_strdup(base64);
+  size_t len;
+
+  len = epan_base64_decode(data);
+  tvb = tvb_new_real_data((const guint8 *)data, len, len);
+
+  tvb_set_free_cb(tvb, g_free);
+
+  return tvb;
+}
