@@ -1,7 +1,7 @@
 /* packet-atm.c
  * Routines for ATM packet disassembly
  *
- * $Id: packet-atm.c 26043 2008-08-20 02:58:41Z guy $
+ * $Id: packet-atm.c 27540 2009-02-25 09:26:27Z guy $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -140,7 +140,7 @@ static const value_string le_control_status_vals[] = {
 	{ 8,  "Invalid REQUESTOR-ID" },
 	{ 9,  "Invalid LAN destination" },
 	{ 10, "Invalid ATM address" },
-	{ 20, "No configuraton" },
+	{ 20, "No configuration" },
 	{ 21, "LE_CONFIGURE error" },
 	{ 22, "Insufficient information" },
 	{ 24, "TLV not found" },
@@ -227,10 +227,10 @@ dissect_lan_destination(tvbuff_t *tvb, int offset, const char *type, proto_tree
     trd = proto_tree_add_text(dest_tree, tvb, offset, 2, "Route descriptor: 0x%02X",
     			route_descriptor);
     rd_tree = proto_item_add_subtree(td, ett_atm_lane_lc_lan_dest_rd);
-    proto_tree_add_text(rd_tree, tvb, offset, 2,
+    proto_tree_add_text(rd_tree, tvb, offset, 2, "%s",
 	    decode_numeric_bitfield(route_descriptor, 0xFFF0, 2*8,
 			"LAN ID = %u"));
-    proto_tree_add_text(rd_tree, tvb, offset, 2,
+    proto_tree_add_text(rd_tree, tvb, offset, 2, "%s",
 	    decode_numeric_bitfield(route_descriptor, 0x000F, 2*8,
 			"Bridge number = %u"));
     break;
@@ -801,7 +801,7 @@ capture_atm(const union wtap_pseudo_header *pseudo_header, const guchar *pd,
 /*
  * Charles Michael Heard's CRC-32 code, from
  *
- *	http://cell-relay.indiana.edu/cell-relay/publications/software/CRC/32bitCRC.c.html
+ *	http://www.cell-relay.com/cell-relay/publications/software/CRC/32bitCRC.c.html
  *
  * with the CRC table initialized with values computed by
  * his "gen_crc_table()" routine, rather than by calling that routine
@@ -1208,7 +1208,7 @@ dissect_reassembled_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 /*
  * Charles Michael Heard's HEC code, from
  *
- *	http://cell-relay.indiana.edu/cell-relay/publications/software/CRC/32bitCRC.tutorial.html
+ *	http://www.cell-relay.com/cell-relay/publications/software/CRC/32bitCRC.tutorial.html
  *
  * with the syndrome and error position tables initialized with values
  * computed by his "gen_syndrome_table()" and "gen_err_posn_table()" routines,
@@ -1257,31 +1257,115 @@ static const guint8 syndrome_table[256] = {
 #define UNCORRECTIBLE_ERROR  128
 
 static const int err_posn_table[256] = {
-	NO_ERROR_DETECTED,	37,
+	NO_ERROR_DETECTED,	39,
+	38,			UNCORRECTIBLE_ERROR,
+	37,			UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	31,
 	36,			UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	8,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	30,			UNCORRECTIBLE_ERROR,
 	35,			UNCORRECTIBLE_ERROR,
-	UNCORRECTIBLE_ERROR,	29,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	23,
+	7,			UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	29,			UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
 	34,			UNCORRECTIBLE_ERROR,
-	UNCORRECTIBLE_ERROR,	6,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	22,			UNCORRECTIBLE_ERROR,
+	6,			UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	0,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
 	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
 	28,			UNCORRECTIBLE_ERROR,
-	33,			UNCORRECTIBLE_ERROR,
 	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
-	UNCORRECTIBLE_ERROR,	21,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	33,			UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	10,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	12,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	21,			UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	19,
 	5,			UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	17,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	3,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	15,
 	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
 	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
 	27,			UNCORRECTIBLE_ERROR,
 	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
 	32,			UNCORRECTIBLE_ERROR,
 	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
 	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	9,			UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	24,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	1,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	11,			UNCORRECTIBLE_ERROR,
 	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
 	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
 	20,			UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	13,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	18,			UNCORRECTIBLE_ERROR,
 	4,			UNCORRECTIBLE_ERROR,
 	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
 	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	16,			UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	25,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	2,			UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	14,			UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
 	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
 	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
 	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
@@ -1289,6 +1373,18 @@ static const int err_posn_table[256] = {
 	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
 	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
 	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
+	UNCORRECTIBLE_ERROR,	UNCORRECTIBLE_ERROR,
 };
 
 /*
@@ -1329,7 +1425,7 @@ static const value_string pt_vals[] = {
 /*
  * Charles Michael Heard's CRC-10 code, from
  *
- *	http://cell-relay.indiana.edu/cell-relay/publications/software/CRC/crc10.html
+ *	http://www.cell-relay.com/cell-relay/publications/software/CRC/crc10.html
  *
  * with the CRC table initialized with values computed by
  * his "gen_byte_crc10_table()" routine, rather than by calling that
@@ -1427,7 +1523,7 @@ static const value_string ft_ad_vals[] = {
 
 static void
 dissect_atm_cell(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
-    proto_tree *atm_tree)
+    proto_tree *atm_tree, guint aal)
 {
   int          offset;
   proto_tree   *aal_tree;
@@ -1437,7 +1533,6 @@ dissect_atm_cell(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
   guint8       vpi;
   guint16      vci;
   guint8       pt;
-  guint8       aal;
   guint16      aal3_4_hdr, aal3_4_trlr;
   guint16      oam_crc;
   gint         length;
@@ -1477,7 +1572,6 @@ dissect_atm_cell(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
    * XXX - do this for all AAL values, overriding whatever information
    * Wiretap got from the file?
    */
-  aal = pinfo->pseudo_header->atm.aal;
   if (aal == AAL_USER) {
 	/*
 	 * OAM F4 is VCI 3 or 4 and PT 0X0.
@@ -1687,7 +1781,7 @@ dissect_atm_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
   if (pinfo->pseudo_header->atm.flags & ATM_RAW_CELL) {
     /* This is a single cell, with the cell header at the beginning. */
     proto_item_set_len(atm_ti, 5);
-    dissect_atm_cell(tvb, pinfo, tree, atm_tree);
+    dissect_atm_cell(tvb, pinfo, tree, atm_tree, pinfo->pseudo_header->atm.aal);
   } else {
     /* This is a reassembled PDU. */
     dissect_reassembled_pdu(tvb, pinfo, tree, atm_tree, atm_ti, truncated);
@@ -1706,6 +1800,23 @@ dissect_atm_untruncated(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   dissect_atm_common(tvb, pinfo, tree, FALSE);
 }
 
+static void
+dissect_atm_oam_cell(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
+{
+  proto_tree   *atm_tree = NULL;
+  proto_item   *atm_ti = NULL;
+  
+  if (check_col(pinfo->cinfo, COL_PROTOCOL))
+    col_set_str(pinfo->cinfo, COL_PROTOCOL, "ATM");
+
+  if (tree) {
+    atm_ti = proto_tree_add_protocol_format(tree, proto_atm, tvb, 0, 0, "ATM");
+    atm_tree = proto_item_add_subtree(atm_ti, ett_atm);
+  }
+
+  dissect_atm_cell(tvb, pinfo, tree, atm_tree, AAL_OAMCELL);
+}
+
 void
 proto_register_atm(void)
 {
@@ -1758,6 +1869,7 @@ proto_register_atm(void)
 
 	register_dissector("lane", dissect_lane, proto_atm_lane);
 	register_dissector("atm_untruncated", dissect_atm_untruncated, proto_atm);
+	register_dissector("atm_oam_cell", dissect_atm_oam_cell, proto_oamaal);
 	
 	atm_module = prefs_register_protocol ( proto_atm, NULL );
 	prefs_register_bool_preference ( atm_module, "dissect_lane_as_sscop", "Dissect LANE as SSCOP",
