@@ -6,7 +6,7 @@
  * Refactored for wireshark checkin
  *   Ronnie Sahlberg 2006
  *
- * $Id: packet-btl2cap.c 26383 2008-10-08 20:33:55Z jake $
+ * $Id: packet-btl2cap.c 27050 2008-12-18 19:08:49Z wmeier $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -173,7 +173,7 @@ static const value_string configuration_result_vals[] = {
 
 static const value_string status_vals[] = {
 	{ 0x0000,	"No further information available" },
-	{ 0x0001,	"Authentication panding" },
+	{ 0x0001,	"Authentication pending" },
 	{ 0x0002,	"Authorization pending" },
 	{ 0, NULL }
 };
@@ -304,7 +304,7 @@ dissect_connrequest(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *t
 
 
 static int
-dissect_options(tvbuff_t *tvb, int offset, packet_info *pinfo _U_, proto_tree *tree, int length, psm_data_t *psm_data _U_, config_data_t *config_data)
+dissect_options(tvbuff_t *tvb, int offset, packet_info *pinfo _U_, proto_tree *tree, int length, config_data_t *config_data)
 {
 	proto_item *ti_option=NULL;
 	proto_tree *ti_option_subtree=NULL;
@@ -420,11 +420,14 @@ dissect_configrequest(tvbuff_t *tvb, int offset, packet_info *pinfo _U_, proto_t
 	offset+=2;
 
 	if(tvb_length_remaining(tvb, offset)){
-		if(pinfo->p2p_dir==P2P_DIR_RECV)
-			config_data = &(psm_data->out);
+		if (psm_data)
+			if(pinfo->p2p_dir==P2P_DIR_RECV)
+				config_data = &(psm_data->out);
+			else
+				config_data = &(psm_data->in);
 		else
-			config_data = &(psm_data->in);
-		offset=dissect_options(tvb, offset, pinfo, tree, length - 4, psm_data, config_data);
+			config_data = NULL;
+		offset=dissect_options(tvb, offset, pinfo, tree, length - 4, config_data);
 	}
 
 	return offset;
@@ -511,11 +514,14 @@ dissect_configresponse(tvbuff_t *tvb, int offset, packet_info *pinfo _U_, proto_
 	offset+=2;
 
 	if(tvb_length_remaining(tvb, offset)){
-		if(pinfo->p2p_dir==P2P_DIR_RECV)
-			config_data = &(psm_data->in);
+		if (psm_data)
+			if(pinfo->p2p_dir==P2P_DIR_RECV)
+				config_data = &(psm_data->out);
+			else
+				config_data = &(psm_data->in);
 		else
-			config_data = &(psm_data->out);
-		offset=dissect_options(tvb, offset, pinfo, tree, length - 6, psm_data, config_data);
+			config_data = NULL;
+		offset=dissect_options(tvb, offset, pinfo, tree, length - 6, config_data);
 	}
 
 	return offset;
@@ -1032,7 +1038,7 @@ proto_register_btl2cap(void)
 		{ &hf_btl2cap_psm,
 			{ "PSM",           "btl2cap.psm",
 				FT_UINT16, BASE_HEX, VALS(psm_vals), 0x0,          
-				"Protocol/Service Multiplexor", HFILL }
+				"Protocol/Service Multiplexer", HFILL }
 		},
 		{ &hf_btl2cap_scid,
 			{ "Source CID",           "btl2cap.scid",
@@ -1067,7 +1073,7 @@ proto_register_btl2cap(void)
 		{ &hf_btl2cap_info_mtu,
 			{ "Remote Entity MTU",           "btl2cap.info_mtu",
 				FT_UINT16, BASE_DEC, NULL, 0x0,          
-				"Remote entitiys acceptable connectionless MTU", HFILL }
+				"Remote entity acceptable connectionless MTU", HFILL }
 		},
 		{ &hf_btl2cap_info_flowcontrol,
 			{ "Flow Control Mode",           "btl2cap.info_flowcontrol",
@@ -1157,10 +1163,10 @@ proto_register_btl2cap(void)
 		{ &hf_btl2cap_option_latency,
 			{ "Latency (microseconds)",           "btl2cap.option_latency",
 				FT_UINT32, BASE_DEC, NULL, 0x0,          
-				"Maximal acceptable dealy (microseconds)", HFILL }
+				"Maximal acceptable delay (microseconds)", HFILL }
 		},
 		{ &hf_btl2cap_option_delayvariation,
-			{ "Delay Variation (microseconds)",           "btl2cap.option_dealyvar",
+			{ "Delay Variation (microseconds)",           "btl2cap.option_delayvar",
 				FT_UINT32, BASE_DEC, NULL, 0x0,          
 				"Difference between maximum and minimum delay (microseconds)", HFILL }
 		},
