@@ -11,7 +11,7 @@
  * Added Additional Status text in Forward Open Response
  *   ryan wamsley * Copyright 2008
  *
- * $Id: packet-cip.c 26091 2008-08-26 01:28:50Z wmeier $
+ * $Id: packet-cip.c 27050 2008-12-18 19:08:49Z wmeier $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -352,7 +352,7 @@ const value_string cip_devtype_vals[] = {
    { DP_LIMIT_SWITCH,         "Limit Switch"                },
    { DP_IND_PROX_SWITCH,      "Inductive Proximity Switch"  },
    { DP_PHOTO_SENSOR,         "Photoelectric Sensor"        },
-   { DP_GENP_DISC_IO,         "General Purpose Dicrete I/O" },
+   { DP_GENP_DISC_IO,         "General Purpose Discrete I/O"},
    { DP_RESOLVER,             "Resolver"                    },
    { DP_COM_ADAPTER,          "Communications Adapter"      },
    { DP_POS_CNT,              "Position Controller",        },
@@ -363,7 +363,7 @@ const value_string cip_devtype_vals[] = {
    { DP_HMI,                  "Human-Machine Interface"     },
    { DP_MASS_FLOW_CNT,        "Mass Flow Controller"        },
    { DP_PNEUM_VALVE,          "Pneumatic Valve"             },
-   { DP_VACUUM_PRES_GAUGE,    "Vaccuum Pressure Gauge"      },
+   { DP_VACUUM_PRES_GAUGE,    "Vacuum Pressure Gauge"       },
 
    { 0, NULL }
 };
@@ -1007,7 +1007,7 @@ dissect_cip_data( proto_tree *item_tree, tvbuff_t *tvb, int offset, int item_len
       /* F3 is the CCO */
       if ( temp_data == 0xF3 )
       {
- 	 collision = 1;
+         collision = 1;
          proto_item_append_text( rrsc_item, "%s (%s)",
                      val_to_str( ( tvb_get_guint8( tvb, offset ) & 0x7F ),
                         cip_sc_vals_cco , "Unknown Service (%x)"),
@@ -1078,8 +1078,8 @@ dissect_cip_data( proto_tree *item_tree, tvbuff_t *tvb, int offset, int item_len
          pi = proto_tree_add_text( item_tree, tvb, offset+4+add_stat_size, item_length-4-add_stat_size, "Command Specific data" );
          cmd_data_tree = proto_item_add_subtree( pi, ett_cmd_data );
 
-	 if( gen_status == CI_GRC_SUCCESS || gen_status == CI_GRC_SERVICE_ERROR )
-	 {
+         if( gen_status == CI_GRC_SUCCESS || gen_status == CI_GRC_SERVICE_ERROR )
+         {
             /* Success responses */
 
             if( ( tvb_get_guint8( tvb, offset ) & 0x7F ) == SC_FWD_OPEN )
@@ -1135,7 +1135,7 @@ dissect_cip_data( proto_tree *item_tree, tvbuff_t *tvb, int offset, int item_len
                } /* End of if reply data */
 
             } /* End of if forward open response */
-	    else if( ( tvb_get_guint8( tvb, offset ) & 0x7F ) == SC_FWD_CLOSE )
+            else if( ( tvb_get_guint8( tvb, offset ) & 0x7F ) == SC_FWD_CLOSE )
             {
                /* Forward close response (Success) */
 
@@ -1468,7 +1468,7 @@ dissect_cip_data( proto_tree *item_tree, tvbuff_t *tvb, int offset, int item_len
          else if( tvb_get_guint8( tvb, offset ) == SC_UNCON_SEND )
          {
 	    /* check for collision */
- 	    if ( collision )
+	    if ( collision )
 	    {
 	       /* Audit Change */
 
@@ -1484,7 +1484,7 @@ dissect_cip_data( proto_tree *item_tree, tvbuff_t *tvb, int offset, int item_len
 	    }
 	    else
 	    {
- 	       /* Unconnected send */
+	       /* Unconnected send */
 
                /* Display the priority/tick timer */
 	       temp_byte = tvb_get_guint8( tvb, offset+2+req_path_size );
@@ -1633,7 +1633,7 @@ static int
 dissect_cip(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
    proto_item *ti;
-   proto_tree *cip_tree;
+   proto_tree *cip_tree = NULL;
 
    /* Make entries in Protocol column and Info column on summary display */
    if( check_col( pinfo->cinfo, COL_PROTOCOL ) )
@@ -1647,10 +1647,10 @@ dissect_cip(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
       /* Create display subtree for the protocol */
       ti = proto_tree_add_item(tree, proto_cip, tvb, 0, -1, FALSE);
       cip_tree = proto_item_add_subtree( ti, ett_cip );
-
-      dissect_cip_data( cip_tree, tvb, 0, tvb_length(tvb), pinfo );
    }
 
+   dissect_cip_data( cip_tree, tvb, 0, tvb_length(tvb), pinfo );
+
    return tvb_length(tvb);
 }
 
