@@ -2,7 +2,7 @@
  * This is a dissector for the DirectPlay protocol.
  * Copyright 2006 - 2008 by Kai Blin
  *
- * $Id: packet-dplay.c 26280 2008-09-26 20:06:40Z wmeier $
+ * $Id: packet-dplay.c 26899 2008-12-02 08:11:23Z jmayer $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -1210,7 +1210,7 @@ static gboolean heur_dissect_dplay(tvbuff_t *tvb, packet_info *pinfo, proto_tree
     return FALSE;
 }
 
-void proto_register_dplay()
+void proto_register_dplay(void)
 {
     static hf_register_info hf [] = {
     /* Common data fields */
