@@ -2,9 +2,9 @@
  * Routines for DPNNS/DASS2 dissection
  * Copyright 2007, Anders Broman <anders.broman[at]ericsson.com>
  * 
- * Supplementarry string parameter table and testing by Tomas Muehlhoff.
+ * Supplementary string parameter table and testing by Tomas Muehlhoff.
  *
- * $Id: packet-dpnss.c 26265 2008-09-25 02:21:04Z wmeier $
+ * $Id: packet-dpnss.c 27050 2008-12-18 19:08:49Z wmeier $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -821,9 +821,9 @@ static const value_string dpnss_sup_serv_par_str_vals[] = {
 	{ DPNSS_ROUTE_RES_CLASS,		"Route Restriction Class"},
 	{ DPNSS_CBR_GRP,				"Call Barring Group"},
 	{ DPNSS_FAC_LST_CODE,			"Facility list code"},
-	{ DPNSS_NO_OF_FUR_TRANS,		"Number of Furhter Transits"},
+	{ DPNSS_NO_OF_FUR_TRANS,		"Number of Further Transits"},
 	{ DPNSS_NO_OF_FUR_ALT_R,		"Number of Further Alternative routes"},
-	{ DPNSS_INT_CAP_LEV,			"Intrution Cspability level"},
+	{ DPNSS_INT_CAP_LEV,			"Intrusion Capability level"},
 	{ DPNSS_NESTING_LEVEL,			"Nesting level"},
 	{ DPNSS_C_PARTY_ADDR,			"C Party Address"},
 	{ DPNSS_B_PARTY_ADDR,			"B Party Address"},
@@ -851,7 +851,7 @@ static const value_string dpnss_sup_serv_par_str_vals[] = {
 	{ DPNSS_CALL_ID_LENGTH,			"Call Identity Length"},
 	{ DPNSS_STATE_OF_DEST,			"State of Destination"},
 	{ DPNSS_STATE_OF_DEST_QUAL,		"State of Destination Qualifier"},
-	{ DPNSS_REASON_FOR_REDIR,		"Reson For Redirection"},
+	{ DPNSS_REASON_FOR_REDIR,		"Reason For Redirection"},
 	{ DPNSS_CLEARING_CAUSE,			"Clearing Cause"},
 	{ DPNSS_RECONT_ADDR,			"Reconnect Address"},
 	{ DPNSS_STATE_OF_OPERATOR,		"State of Operator"},
@@ -863,7 +863,7 @@ static const value_string dpnss_sup_serv_par_str_vals[] = {
 	{ DPNSS_LOCATION,				"Location"},
 	{ DPNSS_SUBADDRESS,				"Subaddress"},
 	{ DPNSS_ALARM_LEVEL,			"Alarm Level"},
-	{ DPNSS_STAFF_PRESENT,			"Staff Pressent"},
+	{ DPNSS_STAFF_PRESENT,			"Staff Present"},
 	{ DPNSS_TIME_AND_DATE,			"Time and Date"},
 	{ DPNSS_SERVICES,				"Services"},
 	{ DPNSS_PBX_REFERENCE,			"PBX Reference"},
@@ -891,8 +891,8 @@ static const value_string dpnss_sup_serv_par_str_vals[] = {
 	{ DPNSS_ISDN_NUM_ATTR,			"ISDM number attribute"},
 	{ DPNSS_ISDN_DPNSS_SUBADDRESS,	"ISDN DPNNS Subaddress"},
 	{ DPNSS_ISDN_NUMBER_DIGITS,		"ISDN Number Digits"},
-	{ DPNSS_HIGH_LAYER_COMP,		"High Layer Compability"},
-	{ DPNSS_LOW_LAYER_COMP,			"Low layer Compability"},
+	{ DPNSS_HIGH_LAYER_COMP,		"High Layer Compatibility"},
+	{ DPNSS_LOW_LAYER_COMP,			"Low layer Compatibility"},
 	{ DPNSS_PROGRESS_INDICATOR,		"Progress Indicator"},
 	{ DPNSS_VPN_ACCESS_REF_NUM,		"VPN Access reference Number"},
 	{ DPNSS_INDEX_NUMBER,			"Index Number"},
