@@ -2,7 +2,7 @@
  * Routines for dsi packet dissection
  * Copyright 2001, Randy McEoin <rmceoin@pe.com>
  *
- * $Id: packet-dsi.c 26265 2008-09-25 02:21:04Z wmeier $
+ * $Id: packet-dsi.c 27050 2008-12-18 19:08:49Z wmeier $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -438,7 +438,7 @@ dissect_dsi_reply_get_status(tvbuff_t *tvb, proto_tree *tree, gint offset)
 						ip6_to_str((const struct e_in6_addr *)ip),port);
 				break;
 			default:
-				ti = proto_tree_add_text(adr_tree, tvb, ofs, len,"Unknow type : %d", type);
+				ti = proto_tree_add_text(adr_tree, tvb, ofs, len,"Unknown type : %d", type);
 				break;
 			}
 			len -= 2;
