@@ -6,7 +6,7 @@
  *
  * Split from packet-gsm_a.c by Neil Piercy <Neil [AT] littlebriars.co.uk>
  *
- * $Id: packet-gsm_a_common.c 27147 2009-01-01 22:54:30Z etxrab $
+ * $Id: packet-gsm_a_common.c 27778 2009-03-18 19:40:54Z etxrab $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -587,6 +587,9 @@ const char* get_gsm_a_msg_string(int pdu_type, int idx)
 		case NAS_PDU_TYPE_EMM:
 			msg_string = nas_emm_elem_strings[idx].strptr;
 			break;
+		case NAS_PDU_TYPE_ESM:
+			msg_string = nas_esm_elem_strings[idx].strptr;
+			break;
 		default:
 			DISSECTOR_ASSERT_NOT_REACHED();
 	}
@@ -627,7 +630,10 @@ static int get_hf_elem_id(int pdu_type)
 			hf_elem_id = hf_nas_eps_common_elem_id;
 			break;
 		case NAS_PDU_TYPE_EMM:
-			hf_elem_id = hf_nas_emm_elem_id;
+			hf_elem_id = hf_nas_eps_emm_elem_id;
+			break;
+		case NAS_PDU_TYPE_ESM:
+			hf_elem_id = hf_nas_eps_esm_elem_id;
 			break;
 		default:
 			DISSECTOR_ASSERT_NOT_REACHED();
@@ -660,17 +666,7 @@ guint16 elem_tlv(tvbuff_t *tvb, proto_tree *tree, guint8 iei, gint pdu_type, int
 	oct = tvb_get_guint8(tvb, curr_offset);
 
 	if (oct == iei){
-		if (oct == GSM_BSSMAP_APDU_IE){
-			/* This elements length is in two octets (a bit of a hack here)*/
-			lengt_length = 2;
-			parm_len = tvb_get_ntohs(tvb, curr_offset + 1);
-			if(parm_len > 255){
-				/* The rest of the logic can't handle length > 255 */
-				DISSECTOR_ASSERT_NOT_REACHED();
-			}
-		}else{
-			parm_len = tvb_get_guint8(tvb, curr_offset + 1);
-		}
+		parm_len = tvb_get_guint8(tvb, curr_offset + 1);
 
 		item =
 		proto_tree_add_text(tree,
@@ -780,7 +776,7 @@ guint16 elem_tlv_e(tvbuff_t *tvb, proto_tree *tree, guint8 iei, gint pdu_type, i
 				a_add_string=ep_alloc(1024);
 				a_add_string[0] = '\0';
 				consumed =
-				(*elem_funcs[idx])(tvb, subtree, curr_offset + 2,
+				(*elem_funcs[idx])(tvb, subtree, curr_offset + 1 + 2,
 					parm_len, a_add_string, 1024);
 
 				if (a_add_string[0] != '\0')
@@ -1074,7 +1070,7 @@ guint16 elem_lv_e(tvbuff_t *tvb, proto_tree *tree, gint pdu_type, int idx, guint
 		if (elem_funcs[idx] == NULL)
 		{
 			proto_tree_add_text(subtree,
-				tvb, curr_offset + 1, parm_len,
+				tvb, curr_offset + 2, parm_len,
 				"Element Value");
 
 			consumed = parm_len;
@@ -1086,7 +1082,7 @@ guint16 elem_lv_e(tvbuff_t *tvb, proto_tree *tree, gint pdu_type, int idx, guint
 			a_add_string=ep_alloc(1024);
 			a_add_string[0] = '\0';
 			consumed =
-				(*elem_funcs[idx])(tvb, subtree, curr_offset + 1,
+				(*elem_funcs[idx])(tvb, subtree, curr_offset + 2,
 					parm_len, a_add_string, 1024);
 
 			if (a_add_string[0] != '\0')
@@ -1096,7 +1092,7 @@ guint16 elem_lv_e(tvbuff_t *tvb, proto_tree *tree, gint pdu_type, int idx, guint
 		}
 	}
 
-	return(consumed + 1);
+	return(consumed + 2);
 }
 /*
  * Value (V) element dissector
