@@ -4,7 +4,7 @@
  * Copyright 2002, Richard Sharpe <rsharpe@samba.org> Added a few 
  *		   bits and pieces ...
  *
- * $Id: packet-gssapi.c 26295 2008-09-29 18:52:17Z sahlberg $
+ * $Id: packet-gssapi.c 26822 2008-11-22 15:55:01Z stig $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -172,8 +172,8 @@ static int
 dissect_gssapi_work(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
     gboolean is_verifier)
 {
-	proto_item *item;
-	proto_tree *subtree;
+	proto_item *volatile item;
+	proto_tree *volatile subtree;
 	volatile int return_offset = 0;
 	gssapi_conv_info_t *volatile gss_info;
 	gssapi_oid_value *oidvalue;
