@@ -10,7 +10,7 @@
  * packet-imap.c, Copyright 1999, Richard Sharpe <rsharpe@ns.aus.com>
  * Copied from packet-pop.c, packet-jabber.c, packet-udp.c, packet-http.c
  *
- * $Id: packet-jxta.c 26588 2008-10-27 20:43:47Z wmeier $
+ * $Id: packet-jxta.c 27254 2009-01-17 17:33:50Z wmeier $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -43,6 +43,8 @@
 
 #include <glib.h>
 
+#include <wsutil/str_util.h>
+
 #include <epan/packet.h>
 #include <epan/conversation.h>
 #include <epan/strutil.h>
@@ -2261,11 +2263,7 @@ static int dissect_media( const gchar* fullmediatype, tvbuff_t * tvb, packet_inf
         pinfo->match_string = ep_strdup(mediatype);
 
         /* force to lower case */
-        {
-            gchar *mediatype_lowercase = g_ascii_strdown(mediatype, -1);
-            mediatype = ep_strdup(mediatype_lowercase);
-            g_free(mediatype_lowercase);
-        }
+        ascii_strdown_inplace(mediatype);
 
         if (0 == strcmp("application/x-jxta-tls-block", mediatype)) {
             /* If we recognize it as a TLS packet then we shuffle it off to ssl dissector. */
