@@ -4,7 +4,7 @@
  * 
  * Copyright 2004, Takeshi Nakashima <T.Nakashima@jp.yokogawa.com>
  *
- * $Id: packet-kink.c 23252 2007-10-23 05:50:00Z etxrab $
+ * $Id: packet-kink.c 26350 2008-10-05 00:28:36Z sahlberg $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -745,7 +745,7 @@ dissect_payload_kink_encrypt(packet_info *pinfo, tvbuff_t *tvb, int offset, prot
 
   if(keytype != 0){
 #ifdef HAVE_KERBEROS
-    plaintext=decrypt_krb5_data(tree, pinfo, 0, encrypt_length, data_value, keytype);    
+    plaintext=decrypt_krb5_data(tree, pinfo, 0, encrypt_length, data_value, keytype, NULL);    
     if(plaintext){
       next_tvb=tvb_new_real_data(plaintext, encrypt_length, encrypt_length);
       tvb_set_child_real_data_tvbuff(tvb, next_tvb);
