@@ -1,7 +1,7 @@
 /* packet-mac-lte.h
  *
  * Martin Mathieson
- * $Id: packet-mac-lte.h 27101 2008-12-23 21:00:04Z martinm $
+ * $Id: packet-mac-lte.h 27739 2009-03-16 15:39:52Z martinm $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -22,8 +22,23 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
+/* radioType */
+#define FDD_RADIO 1
+#define TDD_RADIO 2
 
-/* Info attached to each LTE MAC frame */
+/* direction */
+#define DIRECTION_UPLINK   0
+#define DIRECTION_DOWNLINK 1
+
+/* rntiType */
+#define NO_RNTI 0
+#define P_RNTI  1
+#define RA_RNTI 2
+#define C_RNTI  3
+#define SI_RNTI 4
+
+
+/* Context info attached to each LTE MAC frame */
 typedef struct mac_lte_info
 {
     /* Needed for decode */
@@ -35,7 +50,23 @@ typedef struct mac_lte_info
     guint16         rnti;
     guint16         ueid;
     guint16         subframeNumber;
-    guint8          is_predefined_data;
+    guint8          isPredefinedData;
     guint16         length;
+    guint8          reTxCount;
 } mac_lte_info;
 
+
+typedef struct mac_lte_tap_info {
+    /* Info from context */
+    guint16  rnti;
+    guint8   rnti_type;
+    guint8   is_predefined_data;
+    guint8   reTxCount;
+    guint8   direction;
+
+    /* Number of bytes (which part is used depends upon context settings) */
+    guint32  single_number_of_bytes;
+    guint32  bytes_for_lcid[11];
+    guint32  sdus_for_lcid[11];
+} mac_lte_tap_info;
+
