@@ -1,7 +1,7 @@
 /* packet-mount.c
  * Routines for mount dissection
  *
- * $Id: packet-mount.c 18693 2006-07-09 22:12:02Z gerald $
+ * $Id: packet-mount.c 26091 2008-08-26 01:28:50Z wmeier $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -35,7 +35,6 @@
 #include "packet-mount.h"
 #include "packet-nfs.h"
 
-#include <epan/emem.h>
 
 
 static int proto_mount = -1;
@@ -168,7 +167,8 @@ dissect_mount_dirpath_call(tvbuff_t *tvb, int offset, packet_info *pinfo,
 		rpc_call_info_value *civ=pinfo->private_data;
 
 		if(civ->request && (civ->proc==1)){
-			unsigned char *host, *name;
+			const gchar *host;
+			unsigned char *name;
 			unsigned const char *dir;
 			int len;
 
