@@ -2,7 +2,7 @@
  *
  * Routines for RFC 2250 MPEG2 (ISO/IEC 13818-1) Transport Stream dissection
  *
- * $Id: packet-mp2t.c 26483 2008-10-16 20:21:55Z gerald $
+ * $Id: packet-mp2t.c 27486 2009-02-19 09:23:34Z guy $
  *
  * Copyright 2006, Erwin Rol <erwin@erwinrol.com>
  *
@@ -36,6 +36,11 @@
 #include <string.h>
 
 #include <epan/rtp_pt.h>
+#include "packet-frame.h"
+
+#include <epan/emem.h>
+#include <epan/conversation.h>
+#include <epan/expert.h>
 
 /* The MPEG2 TS packet size */
 #define MP2T_PACKET_SIZE 188
@@ -57,6 +62,7 @@ static int hf_mp2t_pid = -1;
 static int hf_mp2t_tsc = -1;
 static int hf_mp2t_afc = -1;
 static int hf_mp2t_cc = -1;
+static int hf_mp2t_cc_drop = -1;
 
 #define MP2T_SYNC_BYTE_MASK	0xFF000000
 #define MP2T_TEI_MASK		0x00800000
@@ -186,14 +192,267 @@ static const value_string mp2t_afc_vals[] = {
 	{ 0, NULL }
 };
 
+/* Data structure used for detecting CC drops
+ *
+ *  conversation
+ *    |
+ *    +-> mp2t_analysis_data
+ *          |
+ *          +-> pid_table (RB tree)
+ *          |     |
+ *          |     +-> pid_analysis_data (per pid)
+ *          |     +-> pid_analysis_data
+ *          |     +-> pid_analysis_data
+ *          |
+ *          +-> frame_table (RB tree)
+ *                |
+ *                +-> frame_analysis_data (only created if drop detected)
+ *                      |
+ *                      +-> ts_table (RB tree)
+ *                            |
+ *                            +-> pid_analysis_data (per TS subframe)
+ *                            +-> pid_analysis_data
+ *                            +-> pid_analysis_data
+ */
+
+typedef struct mp2t_analysis_data {
+
+	/* This structure contains a tree containing data for the
+	 * individual pid's, this is only used when packets are
+	 * processed sequencially.
+	 */
+	emem_tree_t	*pid_table;
+
+	/* When detecting a CC drop, store that information for the
+	 * given frame.  This info is needed, when clicking around in
+	 * wireshark, as the pid table data only makes sence during
+	 * sequencial processing. The flag pinfo->fd->flags.visited is
+	 * used to tell the difference.
+	 *
+	 */
+	emem_tree_t	*frame_table;
+
+	guint32 cc_drops;	/* Number of detected CC losses per conv */
+
+} mp2t_analysis_data_t;
+
+typedef struct pid_analysis_data {
+	guint16 pid;
+	gint16  cc_prev;  	/* Previous CC number */
+} pid_analysis_data_t;
+
+typedef struct frame_analysis_data {
+
+	/* As each frame has several pid's, thus need a pid data
+	 * structure per TS frame.
+	 */
+	emem_tree_t	*ts_table;
+
+} frame_analysis_data_t;
+
+conversation_t *
+get_the_conversation(packet_info *pinfo)
+{
+	conversation_t *conv = NULL;
+
+	conv = find_conversation(pinfo->fd->num, &pinfo->src,
+				 &pinfo->dst, pinfo->ptype,
+				 pinfo->srcport, pinfo->destport, 0);
+
+	if (conv == NULL) { /* Create a new conversation */
+		conv = conversation_new(pinfo->fd->num, &pinfo->src,
+					&pinfo->dst, pinfo->ptype,
+					pinfo->srcport, pinfo->destport, 0);
+	}
+	return conv;
+}
+
+mp2t_analysis_data_t *
+init_mp2t_conversation_data()
+{
+	mp2t_analysis_data_t *mp2t_data = NULL;
+
+	mp2t_data = se_alloc0(sizeof(struct mp2t_analysis_data));
+
+	mp2t_data->pid_table =
+		se_tree_create_non_persistent(EMEM_TREE_TYPE_RED_BLACK,
+					      "mp2t_pid_table");
+	mp2t_data->frame_table =
+		se_tree_create_non_persistent(EMEM_TREE_TYPE_RED_BLACK,
+					      "mp2t_frame_table");
+	mp2t_data->cc_drops = 0;
+
+	return mp2t_data;
+}
+
+mp2t_analysis_data_t *
+get_mp2t_conversation_data(conversation_t *conv)
+{
+	mp2t_analysis_data_t *mp2t_data = NULL;
+
+	mp2t_data = conversation_get_proto_data(conv, proto_mp2t);
+	if (!mp2t_data) {
+		mp2t_data = init_mp2t_conversation_data();
+		conversation_add_proto_data(conv, proto_mp2t, mp2t_data);
+	}
+
+	return mp2t_data;
+}
+
+frame_analysis_data_t *
+init_frame_analysis_data(mp2t_analysis_data_t *mp2t_data, packet_info *pinfo)
+{
+	frame_analysis_data_t *frame_data = NULL;
+
+	frame_data = se_alloc0(sizeof(struct frame_analysis_data));
+	frame_data->ts_table =
+	  se_tree_create_non_persistent(EMEM_TREE_TYPE_RED_BLACK,
+					"mp2t_frame_pid_table");
+	/* Insert into mp2t tree */
+	se_tree_insert32(mp2t_data->frame_table, pinfo->fd->num,
+			 (void *)frame_data);
+
+	return frame_data;
+}
+
+
+frame_analysis_data_t *
+get_frame_analysis_data(mp2t_analysis_data_t *mp2t_data, packet_info *pinfo)
+{
+	frame_analysis_data_t *frame_data = NULL;
+	frame_data = se_tree_lookup32(mp2t_data->frame_table, pinfo->fd->num);
+	return frame_data;
+}
+
+pid_analysis_data_t *
+get_pid_analysis(guint32 pid, conversation_t *conv)
+{
+
+	pid_analysis_data_t  *pid_data  = NULL;
+	mp2t_analysis_data_t *mp2t_data = NULL;
+	mp2t_data = get_mp2t_conversation_data(conv);
+
+	pid_data = se_tree_lookup32(mp2t_data->pid_table, pid);
+	if (!pid_data) {
+		pid_data          = se_alloc0(sizeof(struct pid_analysis_data));
+		pid_data->cc_prev = -1;
+		pid_data->pid     = pid;
+
+		se_tree_insert32(mp2t_data->pid_table, pid, (void *)pid_data);
+	}
+	return pid_data;
+}
+
+#define KEY(pid, cc) ((pid << 4)|cc)
+
+void
+detect_cc_drops(tvbuff_t *tvb, proto_tree *tree, packet_info *pinfo,
+		guint32 pid, gint32 cc_curr, conversation_t *conv)
+{
+	gint32 cc_prev;
+	pid_analysis_data_t   *pid_data   = NULL;
+	pid_analysis_data_t   *ts_data    = NULL;
+	mp2t_analysis_data_t  *mp2t_data  = NULL;
+	frame_analysis_data_t *frame_data = NULL;
+	proto_item            *flags_item;
+
+	guint32 detected_drop = 0;
+
+	mp2t_data = get_mp2t_conversation_data(conv);
+
+	/* The initial sequencial processing stage */
+	if (!pinfo->fd->flags.visited) {
+
+		/* This is the sequencial processing stage */
+		pid_data = get_pid_analysis(pid, conv);
+
+		cc_prev = pid_data->cc_prev;
+		pid_data->cc_prev = cc_curr;
+
+		/* Null packet always have a CC value equal 0 */
+		if (pid == 0x1fff)
+			return;
+
+		/* Its allowed that (cc_prev == cc_curr) if adaptation field */
+		if (cc_prev == cc_curr)
+			return;
+
+		/* Have not seen this pid before */
+		if (cc_prev == -1)
+			return;
+
+		/* Detect if CC is not increasing by one all the time */
+		if (cc_curr != ((cc_prev+1) & MP2T_CC_MASK)) {
+			detected_drop = 1;
+			mp2t_data->cc_drops++;
+		}
+	}
+
+	/* Save the info about the dropped packet */
+	if (detected_drop && !pinfo->fd->flags.visited) {
+
+		/* Lookup frame data, contains TS pid data objects */
+		frame_data = get_frame_analysis_data(mp2t_data, pinfo);
+		if (!frame_data)
+			frame_data = init_frame_analysis_data(mp2t_data, pinfo);
+
+		/* Create and store a new TS frame pid_data object.
+		   This indicate that we have a drop
+		 */
+		ts_data = se_alloc0(sizeof(struct pid_analysis_data));
+		ts_data->cc_prev = cc_prev;
+		ts_data->pid = pid;
+		se_tree_insert32(frame_data->ts_table, KEY(pid, cc_curr),
+				 (void *)ts_data);
+	}
+
+	/* See if we stored info about drops */
+	if (pinfo->fd->flags.visited) {
+
+		/* Lookup frame data, contains TS pid data objects */
+		frame_data = get_frame_analysis_data(mp2t_data, pinfo);
+		if (!frame_data)
+			return; /* No stored frame data -> no drops*/
+		else {
+			ts_data = se_tree_lookup32(frame_data->ts_table,
+						   KEY(pid, cc_curr));
+			if (ts_data)
+				detected_drop = 1;
+		}
+
+	}
+
+	/* Add info to the proto tree about drops */
+	if (detected_drop) {
+
+		flags_item =
+			proto_tree_add_none_format(
+				tree, hf_mp2t_cc_drop, tvb, 0, 0,
+				"Detected missing CC frame before this"
+				" (accumulated CC loss count:%d)",
+				mp2t_data->cc_drops
+				);
+
+		PROTO_ITEM_SET_GENERATED(flags_item);
+		expert_add_info_format(pinfo, flags_item, PI_MALFORMED,
+				       PI_ERROR, "Detected CC loss");
+
+	}
+}
+
+
 static gint
-dissect_tsp( tvbuff_t *tvb, gint offset, packet_info *pinfo, proto_tree *tree )
+dissect_tsp(tvbuff_t *tvb, volatile gint offset, packet_info *pinfo,
+	    proto_tree *tree, conversation_t *conv)
 {
 	guint32 header;
 	guint afc;
 	gint start_offset = offset;
 	gint payload_len;
 
+	guint32 pid;
+	guint32 cc;
+
 	proto_item *ti = NULL;
 	proto_item *hi = NULL;
 	proto_tree *mp2t_tree = NULL;
@@ -205,8 +464,11 @@ dissect_tsp( tvbuff_t *tvb, gint offset, packet_info *pinfo, proto_tree *tree )
 	
 	header = tvb_get_ntohl(tvb, offset);
 
-	proto_item_append_text(ti, " PID=0x%x CC=%d", (header & MP2T_PID_MASK) >> MP2T_PID_SHIFT, (header & MP2T_CC_MASK) >> MP2T_CC_SHIFT );
+	pid = (header & MP2T_PID_MASK) >> MP2T_PID_SHIFT;
+	cc  = (header & MP2T_CC_MASK)  >> MP2T_CC_SHIFT;
+	proto_item_append_text(ti, " PID=0x%x CC=%d", pid, cc);
 
+	detect_cc_drops(tvb, tree, pinfo, pid, cc, conv);
 
 	hi = proto_tree_add_item( mp2t_tree, hf_mp2t_header, tvb, offset, 4, FALSE);
 	mp2t_header_tree = proto_item_add_subtree( hi, ett_mp2t_header );
@@ -236,164 +498,170 @@ dissect_tsp( tvbuff_t *tvb, gint offset, packet_info *pinfo, proto_tree *tree )
 
 		proto_tree_add_item( mp2t_tree, hf_mp2t_af_length, tvb, offset, 1, FALSE);
 		offset += 1;
-
-		hi = proto_tree_add_item( mp2t_tree, hf_mp2t_af, tvb, offset, af_length, FALSE);
-		mp2t_af_tree = proto_item_add_subtree( hi, ett_mp2t_af );
-
-		af_flags = tvb_get_guint8(tvb, offset);
-
-		proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_di, tvb, offset, 1, FALSE);
-		proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_rai, tvb, offset, 1, FALSE);
-		proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_espi, tvb, offset, 1, FALSE);
-		proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_pcr_flag, tvb, offset, 1, FALSE);
-		proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_opcr_flag, tvb, offset, 1, FALSE);
-		proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_sp_flag, tvb, offset, 1, FALSE);
-		proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_tpd_flag, tvb, offset, 1, FALSE);
-		proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_afe_flag, tvb, offset, 1, FALSE);
-
-		offset += 1;
-
-		if (af_flags &  MP2T_AF_PCR_MASK) {
-			guint64 pcr_base = 0;
-			guint32 pcr_ext = 0;
-			guint8 tmp;
-
-			tmp = tvb_get_guint8(tvb, offset);
-			pcr_base = (pcr_base << 8) | tmp;
-			offset += 1;
-			
-			tmp = tvb_get_guint8(tvb, offset);
-			pcr_base = (pcr_base << 8) | tmp;
-			offset += 1;
-			
-			tmp = tvb_get_guint8(tvb, offset);
-			pcr_base = (pcr_base << 8) | tmp;
+		/* fix issues where afc==3 but af_length==0 
+		 *  Adaptaion field...spec section 2.4.3.5: The value 0 is for inserting a single 
+		 *  stuffing byte in a Transport Stream packet. When the adaptation_field_control 
+		 *  value is '11', the value of the adaptation_field_length shall be in the range 0 to 182.
+		 */
+		if (af_length > 0 ) {
+			hi = proto_tree_add_item( mp2t_tree, hf_mp2t_af, tvb, offset, af_length, FALSE);
+			mp2t_af_tree = proto_item_add_subtree( hi, ett_mp2t_af );
+	
+			af_flags = tvb_get_guint8(tvb, offset);
+	
+			proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_di, tvb, offset, 1, FALSE);
+			proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_rai, tvb, offset, 1, FALSE);
+			proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_espi, tvb, offset, 1, FALSE);
+			proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_pcr_flag, tvb, offset, 1, FALSE);
+			proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_opcr_flag, tvb, offset, 1, FALSE);
+			proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_sp_flag, tvb, offset, 1, FALSE);
+			proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_tpd_flag, tvb, offset, 1, FALSE);
+			proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_afe_flag, tvb, offset, 1, FALSE);
+	
 			offset += 1;
 	
-			tmp = tvb_get_guint8(tvb, offset);
-			pcr_base = (pcr_base << 8) | tmp;
-			offset += 1;
+			if (af_flags &  MP2T_AF_PCR_MASK) {
+				guint64 pcr_base = 0;
+				guint32 pcr_ext = 0;
+				guint8 tmp;
 
-			tmp = tvb_get_guint8(tvb, offset);
-			pcr_base = (pcr_base << 1) | ((tmp >> 7) & 0x01);
-			pcr_ext = (tmp & 0x01);
-			offset += 1;
+				tmp = tvb_get_guint8(tvb, offset);
+				pcr_base = (pcr_base << 8) | tmp;
+				offset += 1;
+			
+				tmp = tvb_get_guint8(tvb, offset);
+				pcr_base = (pcr_base << 8) | tmp;
+				offset += 1;
+				
+				tmp = tvb_get_guint8(tvb, offset);
+				pcr_base = (pcr_base << 8) | tmp;
+				offset += 1;
+	
+				tmp = tvb_get_guint8(tvb, offset);
+				pcr_base = (pcr_base << 8) | tmp;
+				offset += 1;
 
-			tmp = tvb_get_guint8(tvb, offset);
-			pcr_ext = (pcr_ext << 8) | tmp;
-			offset += 1;
+				tmp = tvb_get_guint8(tvb, offset);
+				pcr_base = (pcr_base << 1) | ((tmp >> 7) & 0x01);
+				pcr_ext = (tmp & 0x01);
+				offset += 1;
 
-			proto_tree_add_none_format(mp2t_af_tree, hf_mp2t_af_pcr, tvb, offset - 6, 6, 
+				tmp = tvb_get_guint8(tvb, offset);
+				pcr_ext = (pcr_ext << 8) | tmp;
+				offset += 1;
+
+				proto_tree_add_none_format(mp2t_af_tree, hf_mp2t_af_pcr, tvb, offset - 6, 6, 
 						"Program Clock Reference: base(%" G_GINT64_MODIFIER "u) * 300 + ext(%u) = %" G_GINT64_MODIFIER "u", 
 						pcr_base, pcr_ext, pcr_base * 300 + pcr_ext);
-		}
+			}
 
-		if (af_flags &  MP2T_AF_OPCR_MASK) {
-			guint64 opcr_base = 0;
-			guint32 opcr_ext = 0;
-			guint8 tmp = 0;
+			if (af_flags &  MP2T_AF_OPCR_MASK) {
+				guint64 opcr_base = 0;
+				guint32 opcr_ext = 0;
+				guint8 tmp = 0;
 
-			tmp = tvb_get_guint8(tvb, offset);
-			opcr_base = (opcr_base << 8) | tmp;
-			offset += 1;
+				tmp = tvb_get_guint8(tvb, offset);
+				opcr_base = (opcr_base << 8) | tmp;
+				offset += 1;
 			
-			tmp = tvb_get_guint8(tvb, offset);
-			opcr_base = (opcr_base << 8) | tmp;
-			offset += 1;
+				tmp = tvb_get_guint8(tvb, offset);
+				opcr_base = (opcr_base << 8) | tmp;
+				offset += 1;
 			
-			tmp = tvb_get_guint8(tvb, offset);
-			opcr_base = (opcr_base << 8) | tmp;
-			offset += 1;
+				tmp = tvb_get_guint8(tvb, offset);
+				opcr_base = (opcr_base << 8) | tmp;
+				offset += 1;
 	
-			tmp = tvb_get_guint8(tvb, offset);
-			opcr_base = (opcr_base << 8) | tmp;
-			offset += 1;
+				tmp = tvb_get_guint8(tvb, offset);
+				opcr_base = (opcr_base << 8) | tmp;
+				offset += 1;
 
-			tmp = tvb_get_guint8(tvb, offset);
-			opcr_base = (opcr_base << 1) | ((tmp >> 7) & 0x01);
-			opcr_ext = (tmp & 0x01);
-			offset += 1;
+				tmp = tvb_get_guint8(tvb, offset);
+				opcr_base = (opcr_base << 1) | ((tmp >> 7) & 0x01);
+				opcr_ext = (tmp & 0x01);
+				offset += 1;
 
-			tmp = tvb_get_guint8(tvb, offset);
-			opcr_ext = (opcr_ext << 8) | tmp;
-			offset += 1;
+				tmp = tvb_get_guint8(tvb, offset);
+				opcr_ext = (opcr_ext << 8) | tmp;
+				offset += 1;
 
-			proto_tree_add_none_format(mp2t_af_tree, hf_mp2t_af_opcr, tvb, offset - 6, 6, 
+				proto_tree_add_none_format(mp2t_af_tree, hf_mp2t_af_opcr, tvb, offset - 6, 6, 
 						"Original Program Clock Reference: base(%" G_GINT64_MODIFIER "u) * 300 + ext(%u) = %" G_GINT64_MODIFIER "u", 
 						opcr_base, opcr_ext, opcr_base * 300 + opcr_ext);
 	
-			offset += 6;
-		}
-
-		if (af_flags &  MP2T_AF_SP_MASK) {
-			proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_sc, tvb, offset, 1, FALSE);
-			offset += 1;
-		}
-
-		if (af_flags &  MP2T_AF_TPD_MASK) {
-			guint8 tpd_len;
-		
-			tpd_len = tvb_get_guint8(tvb, offset);
-			proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_tpd_length, tvb, offset, 1, FALSE);
-			offset += 1;
-
-			proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_tpd, tvb, offset, tpd_len, FALSE);
-			offset += tpd_len;
-		}
-
-		if (af_flags &  MP2T_AF_AFE_MASK) {
-			guint8 e_len;
-			guint8 e_flags;
-			gint e_start_offset = offset;
-			gint reserved_len = 0;
-
-			e_len = tvb_get_guint8(tvb, offset);
-			proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_e_length, tvb, offset, 1, FALSE);
-			offset += 1;
-
-			e_flags = tvb_get_guint8(tvb, offset);
-			proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_e_ltw_flag, tvb, offset, 1, FALSE);
-			proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_e_pr_flag, tvb, offset, 1, FALSE);
-			proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_e_ss_flag, tvb, offset, 1, FALSE);
-			proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_e_reserved, tvb, offset, 1, FALSE);			
-			offset += 1;
-			
-			if (e_flags & MP2T_AF_E_LTW_FLAG_MASK) {
-				proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_e_ltwv_flag, tvb, offset, 2, FALSE);
-				proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_e_ltwo, tvb, offset, 2, FALSE);
-				offset += 2;
+				offset += 6;
 			}
-
-			if (e_flags & MP2T_AF_E_PR_FLAG_MASK) {
-				proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_e_pr_reserved, tvb, offset, 3, FALSE);
-				proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_e_pr, tvb, offset, 3, FALSE);
-				offset += 3;
-			}
-
-			if (e_flags & MP2T_AF_E_SS_FLAG_MASK) {
-				proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_e_st, tvb, offset, 1, FALSE);
-				proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_e_dnau_32_30, tvb, offset, 1, FALSE);
-				proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_e_m_1, tvb, offset, 1, FALSE);
+	
+			if (af_flags &  MP2T_AF_SP_MASK) {
+				proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_sc, tvb, offset, 1, FALSE);
 				offset += 1;
-				proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_e_dnau_29_15, tvb, offset, 2, FALSE);
-				proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_e_m_2, tvb, offset, 2, FALSE);
-				offset += 2;
-				proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_e_dnau_14_0, tvb, offset, 2, FALSE);
-				proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_e_m_3, tvb, offset, 2, FALSE);
-				offset += 2;
 			}
 
-			reserved_len = (e_len + 1) - (offset - e_start_offset);
-			if (reserved_len > 0) {
-				proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_e_reserved_bytes, tvb, offset, reserved_len, FALSE);
-				offset += reserved_len;
-			}
-		}
+			if (af_flags &  MP2T_AF_TPD_MASK) {
+				guint8 tpd_len;
+			
+				tpd_len = tvb_get_guint8(tvb, offset);
+				proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_tpd_length, tvb, offset, 1, FALSE);
+				offset += 1;
 
-		stuffing_len = (af_length + 1) - (offset - af_start_offset);
-		if (stuffing_len > 0) {
-			proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_stuffing_bytes, tvb, offset, stuffing_len, FALSE);
-			offset += stuffing_len;
+				proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_tpd, tvb, offset, tpd_len, FALSE);
+				offset += tpd_len;
+			}
+
+			if (af_flags &  MP2T_AF_AFE_MASK) {
+				guint8 e_len;
+				guint8 e_flags;
+				gint e_start_offset = offset;
+				gint reserved_len = 0;
+
+				e_len = tvb_get_guint8(tvb, offset);
+				proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_e_length, tvb, offset, 1, FALSE);
+				offset += 1;
+
+				e_flags = tvb_get_guint8(tvb, offset);
+				proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_e_ltw_flag, tvb, offset, 1, FALSE);
+				proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_e_pr_flag, tvb, offset, 1, FALSE);
+				proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_e_ss_flag, tvb, offset, 1, FALSE);
+				proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_e_reserved, tvb, offset, 1, FALSE);			
+				offset += 1;
+			
+				if (e_flags & MP2T_AF_E_LTW_FLAG_MASK) {
+					proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_e_ltwv_flag, tvb, offset, 2, FALSE);
+					proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_e_ltwo, tvb, offset, 2, FALSE);
+					offset += 2;
+				}
+
+				if (e_flags & MP2T_AF_E_PR_FLAG_MASK) {
+					proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_e_pr_reserved, tvb, offset, 3, FALSE);
+					proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_e_pr, tvb, offset, 3, FALSE);
+					offset += 3;
+				}
+
+				if (e_flags & MP2T_AF_E_SS_FLAG_MASK) {
+					proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_e_st, tvb, offset, 1, FALSE);
+					proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_e_dnau_32_30, tvb, offset, 1, FALSE);
+					proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_e_m_1, tvb, offset, 1, FALSE);
+					offset += 1;
+					proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_e_dnau_29_15, tvb, offset, 2, FALSE);
+					proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_e_m_2, tvb, offset, 2, FALSE);
+					offset += 2;
+					proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_e_dnau_14_0, tvb, offset, 2, FALSE);
+					proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_e_m_3, tvb, offset, 2, FALSE);
+					offset += 2;
+				}
+
+				reserved_len = (e_len + 1) - (offset - e_start_offset);
+				if (reserved_len > 0) {
+					proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_e_reserved_bytes, tvb, offset, reserved_len, FALSE);
+					offset += reserved_len;
+				}
+			}
+
+			stuffing_len = (af_length + 1) - (offset - af_start_offset);
+			if (stuffing_len > 0) {
+				proto_tree_add_item( mp2t_af_tree, hf_mp2t_af_stuffing_bytes, tvb, offset, stuffing_len, FALSE);
+				offset += stuffing_len;
+			}
 		}
 	}
 
@@ -404,11 +672,35 @@ dissect_tsp( tvbuff_t *tvb, gint offset, packet_info *pinfo, proto_tree *tree )
 			proto_tree_add_item( mp2t_tree, hf_mp2t_malformed_payload, tvb, offset, payload_len, FALSE);
 			offset += payload_len;
 		} else {
-			if (tvb_get_ntoh24(tvb, offset) == 0x000001) {
-				tvbuff_t *next_tvb = tvb_new_subset(tvb, offset, payload_len, payload_len);
-				call_dissector(pes_handle, next_tvb, pinfo, mp2t_tree);
-			} else
+			/* Check to make sure if we are not at end of payload, if we have less than 3 bytes, the tvb_get_ntoh24 fails. */
+			if (payload_len >=3 ) {
+				if (tvb_get_ntoh24(tvb, offset) == 0x000001) {
+					tvbuff_t *next_tvb = tvb_new_subset(tvb, offset, payload_len, payload_len);
+					const char *saved_proto = pinfo->current_proto;
+
+					TRY {
+						call_dissector(pes_handle, next_tvb, pinfo, mp2t_tree);
+					}
+
+					/*
+					 Don't stop processing TS packets if somebody threw
+					 BoundsError, which means that dissecting the payload found
+					 that the packet was cut off by before the end of the
+					 payload.  This is very likely as this protocol splits the
+					 media stream up into chunks of MP2T_PACKET_SIZE.
+					*/
+					CATCH2(BoundsError, ReportedBoundsError) {
+						show_exception(next_tvb, pinfo, tree, EXCEPT_CODE, GET_MESSAGE);
+						pinfo->current_proto = saved_proto;
+					}
+
+					ENDTRY;
+				} else {
+					proto_tree_add_item( mp2t_tree, hf_mp2t_payload, tvb, offset, payload_len, FALSE);
+				}
+			} else {
 				proto_tree_add_item( mp2t_tree, hf_mp2t_payload, tvb, offset, payload_len, FALSE);
+			}
 			offset += payload_len;
 		}
 	}
@@ -421,9 +713,11 @@ static void
 dissect_mp2t( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 {
 	guint offset = 0;
+	conversation_t *conv;
+	conv = get_the_conversation(pinfo);
 
 	while ( tvb_reported_length_remaining(tvb, offset) >= MP2T_PACKET_SIZE ) {
-		offset = dissect_tsp( tvb, offset, pinfo, tree);
+		offset = dissect_tsp(tvb, offset, pinfo, tree, conv);
 	}
 }
 
@@ -486,6 +780,10 @@ proto_register_mp2t(void)
 			"Continuity Counter", "mp2t.cc",
 			FT_UINT32, BASE_DEC, NULL, MP2T_CC_MASK, NULL, HFILL
 		} } ,
+		{ &hf_mp2t_cc_drop, {
+			"Continuity Counter Drops", "mp2t.cc.drop",
+			FT_NONE, BASE_DEC, NULL, 0x0, NULL, HFILL
+		} } ,
 		{ &hf_mp2t_af, {
 			"Adaption field", "mp2t.af",
 			FT_NONE, BASE_HEX, NULL, 0, NULL, HFILL
@@ -651,7 +949,7 @@ proto_reg_handoff_mp2t(void)
 
 	mp2t_handle = create_dissector_handle(dissect_mp2t, proto_mp2t);
 	dissector_add("rtp.pt", PT_MP2T, mp2t_handle);
-	dissector_add("udp.port", 0, mp2t_handle);
+	dissector_add_handle("udp.port", mp2t_handle);  /* for decode-as */
 
 	pes_handle = find_dissector("mpeg-pes");
 }
