@@ -2,7 +2,7 @@
  * Routines for Message Session Relay Protocol(MSRP) dissection
  * Copyright 2005, Anders Broman <anders.broman[at]ericsson.com>
  *
- * $Id: packet-msrp.c 24829 2008-04-07 05:22:54Z sfisher $
+ * $Id: packet-msrp.c 27254 2009-01-17 17:33:50Z wmeier $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -36,6 +36,9 @@
 #include <ctype.h>
 
 #include <glib.h>
+
+#include <wsutil/str_util.h>
+
 #include <epan/conversation.h>
 #include <epan/strutil.h>
 #include <epan/packet.h>
@@ -485,8 +488,7 @@ dissect_msrp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	gboolean have_body = FALSE;
 	gboolean found_match = FALSE;
 	gint content_type_len, content_type_parameter_str_len;
-	char *media_type_str = NULL;
-	char *media_type_str_lower_case = NULL;
+	gchar *media_type_str_lower_case = NULL;
 	char *content_type_parameter_str = NULL;
 	tvbuff_t *next_tvb;
 	gint parameter_offset;
@@ -686,8 +688,8 @@ dissect_msrp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 								content_type_parameter_str = tvb_get_ephemeral_string(tvb,
 										     parameter_offset, content_type_parameter_str_len);
 							}
-							media_type_str = tvb_get_ephemeral_string(tvb, value_offset, content_type_len);
-							media_type_str_lower_case = g_ascii_strdown(media_type_str, -1);
+							media_type_str_lower_case = ascii_strdown_inplace(
+                                                            (gchar *)tvb_get_ephemeral_string(tvb, value_offset, content_type_len));
 							break;
 
 						default:
@@ -720,7 +722,6 @@ dissect_msrp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			                                   media_type_str_lower_case,
 			                                   next_tvb, pinfo,
 			                                   msrp_data_tree);
-				g_free(media_type_str_lower_case);
 				pinfo->private_data = save_private_data;
 				/* If no match dump as text */
 			}
@@ -770,9 +771,10 @@ dissect_msrp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 void
 proto_reg_handoff_msrp(void)
 {
-	msrp_handle = new_create_dissector_handle(dissect_msrp, proto_msrp);
-	dissector_add("tcp.port", 0, msrp_handle);
+	msrp_handle = find_dissector("msrp");
+	dissector_add_handle("tcp.port", msrp_handle);   /* for "decode-as" */
 	heur_dissector_add("tcp", dissect_msrp_heur, proto_msrp);
+	media_type_dissector_table = find_dissector_table("media_type");
 }
 
 void
@@ -937,8 +939,6 @@ proto_register_msrp(void)
 	proto_register_field_array(proto_msrp, hf, array_length(hf));
 	proto_register_subtree_array(ett, array_length(ett));
 
-	media_type_dissector_table = find_dissector_table("media_type");
-
 	msrp_module = prefs_register_protocol(proto_msrp, NULL);
 
 	prefs_register_bool_preference(msrp_module, "display_raw_text",
