@@ -3,7 +3,7 @@
  * to when it had only NBNS)
  * Guy Harris <guy@alum.mit.edu>
  *
- * $Id: packet-nbns.c 25981 2008-08-11 12:00:27Z stig $
+ * $Id: packet-nbns.c 26647 2008-10-31 15:11:57Z stig $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -1015,7 +1015,7 @@ dissect_answer_records(tvbuff_t *tvb, int cur_off, int nbns_data_offset,
 
 	start_off = cur_off;
 	if (nbns_tree != NULL) {
-		ti = proto_tree_add_text(nbns_tree, tvb, start_off, -1, name);
+		ti = proto_tree_add_text(nbns_tree, tvb, start_off, -1, "%s", name);
 		qatree = proto_item_add_subtree(ti, ett_nbns_ans);
 	}
 	while (count-- > 0) {
