@@ -1,7 +1,7 @@
 /* packet-ositp.c
  * Routines for ISO/OSI transport protocol packet disassembly
  *
- * $Id: packet-ositp.c 26357 2008-10-06 15:24:20Z stig $
+ * $Id: packet-ositp.c 27324 2009-01-28 21:52:57Z wmeier $
  * Laurent Deniel <laurent.deniel@free.fr>
  * Ralf Schneider <Ralf.Schneider@t-online.de>
  *
@@ -726,7 +726,7 @@ static int ositp_decode_DR(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
     case (128+4): str = "Mismatched references"; break;
     case (128+5): str = "Protocol error"; break;
     case (128+7): str = "Reference overflow"; break;
-    case (128+8): str = "Connection requestion refused"; break;
+    case (128+8): str = "Connection request refused"; break;
     case (128+10):str = "Header or parameter length invalid"; break;
     case (0):     str = "Reason not specified"; break;
     case (1):     str = "Congestion at TSAP"; break;
@@ -1826,7 +1826,7 @@ void proto_register_cotp(void)
   prefs_register_bool_preference(cotp_module, "reassemble",
 	 "Reassemble segmented COTP datagrams",
 	 "Whether segmented COTP datagrams should be reassembled."
-    " To use this option, you must also enable \"Allow subdissectors to reassemble TCP streams\" in the TCP protocol settings.",
+	 " To use this option, you must also enable \"Allow subdissectors to reassemble TCP streams\" in the TCP protocol settings.",
 	&cotp_reassemble);
 
   prefs_register_enum_preference(cotp_module, "tsap_display",
@@ -1849,19 +1849,6 @@ void proto_register_cotp(void)
   register_init_routine(cotp_reassemble_init);
 }
 
-void
-proto_reg_handoff_cotp(void)
-{
-  dissector_handle_t ositp_handle;
-
-  ositp_handle = find_dissector("ositp");
-  dissector_add("ip.proto", IP_PROTO_TP, ositp_handle);
-
-  data_handle = find_dissector("data");
-
-  proto_clnp = proto_get_id_by_filter_name("clnp");
-}
-
 void proto_register_cltp(void)
 {
   static hf_register_info hf[] = {
@@ -1880,3 +1867,17 @@ void proto_register_cltp(void)
   proto_register_field_array(proto_cltp, hf, array_length(hf));
   proto_register_subtree_array(ett, array_length(ett));
 }
+
+void
+proto_reg_handoff_cotp(void)
+{
+  dissector_handle_t ositp_handle;
+
+  ositp_handle = find_dissector("ositp");
+  dissector_add("ip.proto", IP_PROTO_TP, ositp_handle);
+
+  data_handle = find_dissector("data");
+
+  proto_clnp = proto_get_id_by_filter_name("clnp");
+}
+
