@@ -8,7 +8,7 @@
  *
  * Copied from README.developer
  *
- * $Id: packet-ppi.c 26275 2008-09-26 10:15:58Z stig $
+ * $Id: packet-ppi.c 27076 2008-12-22 00:06:12Z guy $
  *
  * Copyright (c) 2006 CACE Technologies, Davis (California)
  * All rights reserved.
@@ -60,7 +60,8 @@
 #include <epan/range.h>
 #include <epan/frequency-utils.h>
 
-#include <wsutil/encap_util.h>
+/* Needed for wtap_pcap_encap_to_wtap_encap(). */
+#include <wiretap/libpcap.h>
 
 #include "packet-frame.h"
 #include "packet-eth.h"
@@ -68,8 +69,7 @@
 
 /*
  * Per-Packet Information (PPI) header.
- * See the PPI Packet Header documentation at
- * www.cacetech.com/documents/PPI_Header_format_1.0.pdf
+ * See the PPI Packet Header documentation at http://www.cacetech.com/documents
  * for details.
  */
 
@@ -170,6 +170,7 @@ typedef enum {
     /* 30000 - 65535: Private types */
     INTEL_CORP_PRIVATE          = 30000,
     MOHAMED_THAGA_PRIVATE       = 30001,
+    GPS_TAGGING_PRIVATE         = 30002,
     CACE_PRIVATE                = 0xCACE
     /* All others RESERVED.  Contact the WinPcap team for an assignment */
 } ppi_field_type;
@@ -283,12 +284,11 @@ static int hf_aggregation_extension_interface_id = -1;
 /* 802.3 Extension */
 static int hf_8023_extension_flags = -1;
 static int hf_8023_extension_flags_fcs_present = -1;
-static int hf_8023_extension_flags_flag2 = -1;
-static int hf_8023_extension_flags_flag3 = -1;
 static int hf_8023_extension_errors = -1;
-static int hf_8023_extension_errors_error1 = -1;
-static int hf_8023_extension_errors_error2 = -1;
-static int hf_8023_extension_errors_error3 = -1;
+static int hf_8023_extension_errors_fcs = -1;
+static int hf_8023_extension_errors_sequence = -1;
+static int hf_8023_extension_errors_symbol = -1;
+static int hf_8023_extension_errors_data = -1;
 
 static gint ett_ppi_pph = -1;
 static gint ett_ppi_flags = -1;
@@ -325,6 +325,11 @@ static const value_string vs_ppi_field_type[] = {
     {PPI_CAPTURE_INFO, "Capture-Info"},
     {PPI_AGGREGATION_EXTENSION, "Aggregation Extension"},
     {PPI_8023_EXTENSION, "802.3 Extension"},
+
+    {INTEL_CORP_PRIVATE, "Intel Corporation (private)"},
+    {MOHAMED_THAGA_PRIVATE, "Mohamed Thaga (private)"},
+    {GPS_TAGGING_PRIVATE, "GPS Tagging (private)"},
+    {CACE_PRIVATE, "CACE Technologies (private)"},
     {0, NULL}
 };
 
@@ -703,15 +708,14 @@ static void dissect_8023_extension(tvbuff_t *tvb, packet_info *pinfo _U_, proto_
     csr = ptvcursor_new(ftree, tvb, offset);
 
     ptvcursor_add_with_subtree(csr, hf_8023_extension_flags, 4, TRUE, ett_8023_extension_flags);
-    ptvcursor_add_no_advance(csr, hf_8023_extension_flags_fcs_present, 4, TRUE);
-    ptvcursor_add_no_advance(csr, hf_8023_extension_flags_flag2, 4, TRUE);
-    ptvcursor_add(csr, hf_8023_extension_flags_flag3, 4, TRUE);
+    ptvcursor_add(csr, hf_8023_extension_flags_fcs_present, 4, TRUE);
     ptvcursor_pop_subtree(csr);
 
     ptvcursor_add_with_subtree(csr, hf_8023_extension_errors, 4, TRUE, ett_8023_extension_errors);
-    ptvcursor_add_no_advance(csr, hf_8023_extension_errors_error1, 4, TRUE);
-    ptvcursor_add_no_advance(csr, hf_8023_extension_errors_error2, 4, TRUE);
-    ptvcursor_add(csr, hf_8023_extension_errors_error3, 4, TRUE);
+    ptvcursor_add_no_advance(csr, hf_8023_extension_errors_fcs, 4, TRUE);
+    ptvcursor_add_no_advance(csr, hf_8023_extension_errors_sequence, 4, TRUE);
+    ptvcursor_add_no_advance(csr, hf_8023_extension_errors_symbol, 4, TRUE);
+    ptvcursor_add(csr, hf_8023_extension_errors_data, 4, TRUE);
     ptvcursor_pop_subtree(csr);
 
     ptvcursor_free(csr);
@@ -927,7 +931,7 @@ dissect_ppi(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
                     add_new_data_source(pinfo, next_tvb, mpdu_str);
 
                     if (agg_tree) {
-                        ti = proto_tree_add_text(agg_tree, next_tvb, 0, -1, mpdu_str);
+                        ti = proto_tree_add_text(agg_tree, next_tvb, 0, -1, "%s", mpdu_str);
                         ampdu_tree = proto_item_add_subtree(ti, ett_ampdu_segment);
                     }
                     call_dissector(ieee80211_ht_handle, next_tvb, pinfo, ampdu_tree);
@@ -1241,24 +1245,25 @@ proto_register_ppi(void)
     { &hf_8023_extension_flags_fcs_present,
        { "FCS Present Flag", "ppi.8023_extension.flags.fcs_present",
             FT_BOOLEAN, 32, TFS(&tfs_true_false), 0x0001, "FCS (4 bytes) is present at the end of the packet", HFILL } },
-    { &hf_8023_extension_flags_flag2,
-       { "Flag 2", "ppi.8023_extension.flags.flag2",
-            FT_BOOLEAN, 32, TFS(&tfs_true_false), 0x0002, "Debug Flag 2", HFILL } },
-    { &hf_8023_extension_flags_flag3,
-       { "Flag 3", "ppi.8023_extension.flags.flag3",
-            FT_BOOLEAN, 32, TFS(&tfs_true_false), 0x0004, "Debug Flag 3", HFILL } },
     { &hf_8023_extension_errors,
        { "Errors", "ppi.8023_extension.errors",
             FT_UINT32, BASE_HEX, NULL, 0x0, "PPI 802.3 Extension Errors", HFILL } },
-    { &hf_8023_extension_errors_error1,
-       { "Error 1", "ppi.8023_extension.errors.error1",
-            FT_BOOLEAN, 32, TFS(&tfs_true_false), 0x0001, "Debug Error 1", HFILL } },
-    { &hf_8023_extension_errors_error2,
-       { "Error 2", "ppi.8023_extension.errors.error2",
-            FT_BOOLEAN, 32, TFS(&tfs_true_false), 0x0002, "Debug Error 2", HFILL } },
-    { &hf_8023_extension_errors_error3,
-       { "Error 3", "ppi.8023_extension.errors.error3",
-            FT_BOOLEAN, 32, TFS(&tfs_true_false), 0x0004, "Debug Error 3", HFILL } },
+    { &hf_8023_extension_errors_fcs,
+       { "FCS Error", "ppi.8023_extension.errors.fcs",
+            FT_BOOLEAN, 32, TFS(&tfs_true_false), 0x0001,
+            "PPI 802.3 Extension FCS Error", HFILL } },
+    { &hf_8023_extension_errors_sequence,
+       { "Sequence Error", "ppi.8023_extension.errors.sequence",
+            FT_BOOLEAN, 32, TFS(&tfs_true_false), 0x0002,
+            "PPI 802.3 Extension Sequence Error", HFILL } },
+    { &hf_8023_extension_errors_symbol,
+       { "Symbol Error", "ppi.8023_extension.errors.symbol",
+            FT_BOOLEAN, 32, TFS(&tfs_true_false), 0x0004,
+            "PPI 802.3 Extension Symbol Error", HFILL } },
+    { &hf_8023_extension_errors_data,
+       { "Data Error", "ppi.8023_extension.errors.data",
+            FT_BOOLEAN, 32, TFS(&tfs_true_false), 0x0008,
+            "PPI 802.3 Extension Data Error", HFILL } },
 
     };
 
