@@ -2,7 +2,7 @@
  *
  * Martin Mathieson
  *
- * $Id: packet-rlc-lte.c 27195 2009-01-08 13:28:30Z kukosa $
+ * $Id: packet-rlc-lte.c 27699 2009-03-11 11:17:16Z martinm $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -38,6 +38,15 @@
  * Radio Link Control (RLC) Protocol specification
  */
 
+/* TODO:
+   Working towards re-assembly:
+   - track frames within same channel, so can show:
+       - frame link back to start of current segment
+       - show length (and possibly contents) of re-assembled segment
+       - report inconsistent framing info values, i.e. grumble if see for consecutive SNs:
+           -   "... ["   OR     "] ..."
+*/
+ 
 /* Initialize the protocol and registered fields. */
 int proto_rlc_lte = -1;
 
@@ -90,6 +99,8 @@ static int hf_rlc_lte_am_nack_sn = -1;
 static int hf_rlc_lte_am_so_start = -1;
 static int hf_rlc_lte_am_so_end = -1;
 
+static int hf_rlc_lte_predefined_pdu = -1;
+
 /* Subtrees. */
 static int ett_rlc_lte = -1;
 static int ett_rlc_lte_um_header = -1;
@@ -99,22 +110,17 @@ static int ett_rlc_lte_extension_part = -1;
 
 static const value_string direction_vals[] =
 {
-    { 0,      "Uplink"},
-    { 1,      "Downlink"},
+    { DIRECTION_UPLINK,      "Uplink"},
+    { DIRECTION_DOWNLINK,    "Downlink"},
     { 0, NULL }
 };
 
-
-#define RLC_TM_MODE 1
-#define RLC_UM_MODE 2
-#define RLC_AM_MODE 3
-
-
 static const value_string rlc_mode_short_vals[] =
 {
     { RLC_TM_MODE,      "TM"},
     { RLC_UM_MODE,      "UM"},
     { RLC_AM_MODE,      "AM"},
+    { RLC_PREDEF,       "PREDEFINED"},
     { 0, NULL }
 };
 
@@ -129,11 +135,11 @@ static const value_string rlc_mode_vals[] =
 
 static const value_string rlc_channel_type_vals[] =
 {
-    { 1,      "CCCH"},
-    { 2,      "BCCH"},
-    { 3,      "PCCH"},
-    { 4,      "SRB"},
-    { 5,      "DRB"},
+    { CHANNEL_TYPE_CCCH,     "CCCH"},
+    { CHANNEL_TYPE_BCCH,     "BCCH"},
+    { CHANNEL_TYPE_PCCH,     "PCCH"},
+    { CHANNEL_TYPE_SRB,      "SRB"},
+    { CHANNEL_TYPE_DRB,      "DRB"},
     { 0, NULL }
 };
 
@@ -298,9 +304,10 @@ static void show_PDU_in_info(packet_info *pinfo,
 {
     /* Reflect this PDU in the info column */
     if (check_col(pinfo->cinfo, COL_INFO)) {
-        col_append_fstr(pinfo->cinfo, COL_INFO, "  %s%u-bytes%s",
+        col_append_fstr(pinfo->cinfo, COL_INFO, "  %s%u-byte%s%s",
                         (first_includes_start) ? "[" : "..",
                         length,
+                        (length > 1) ? "s" : "",
                         (last_includes_end) ? "]" : "..");
     }
 }
@@ -334,7 +341,7 @@ static void dissect_rlc_lte_um(tvbuff_t *tvb, packet_info *pinfo,
 
     /*******************************/
     /* Fixed UM header             */
-    if (p_rlc_lte_info->UMSequenceNumberLength == 5) {
+    if (p_rlc_lte_info->UMSequenceNumberLength == UM_SN_LENGTH_5_BITS) {
         /* Framing info (2 bits) */
         proto_tree_add_bits_ret_val(um_header_tree, hf_rlc_lte_um_fi,
                                     tvb, offset*8, 2,
@@ -351,7 +358,7 @@ static void dissect_rlc_lte_um(tvbuff_t *tvb, packet_info *pinfo,
                                     &sn, FALSE);
         offset++;
     }
-    else if (p_rlc_lte_info->UMSequenceNumberLength == 10) {
+    else if (p_rlc_lte_info->UMSequenceNumberLength == UM_SN_LENGTH_10_BITS) {
         guint8 reserved;
         proto_item *ti;
 
@@ -423,6 +430,7 @@ static void dissect_rlc_lte_um(tvbuff_t *tvb, packet_info *pinfo,
             show_PDU_in_info(pinfo, s_lengths[n],
                              (n==0) ? first_includes_start : TRUE,
                              TRUE);
+            tvb_ensure_bytes_exist(tvb, offset, s_lengths[n]);
             offset += s_lengths[n];
         }
     }
@@ -473,6 +481,10 @@ static void dissect_rlc_lte_am_status_pdu(tvbuff_t *tvb, packet_info *pinfo,
     proto_tree_add_bits_ret_val(tree, hf_rlc_lte_am_ack_sn, tvb,
                                 bit_offset, 10, &ack_sn, FALSE);
     bit_offset += 10;
+    if (check_col(pinfo->cinfo, COL_INFO)) {
+        col_append_fstr(pinfo->cinfo, COL_INFO, "  ACK_SN=%u", (guint16)ack_sn);
+    }
+
 
     /* E1 */
     proto_tree_add_bits_ret_val(tree, hf_rlc_lte_am_e1, tvb,
@@ -484,13 +496,21 @@ static void dissect_rlc_lte_am_status_pdu(tvbuff_t *tvb, packet_info *pinfo,
     /* Optional, extra fields */
     do {
         if (e1) {
+            proto_item *nack_ti;
+
             /****************************/
             /* Read NACK_SN, E1, E2     */
 
-            /* ACK_SN */
-            proto_tree_add_bits_ret_val(tree, hf_rlc_lte_am_nack_sn, tvb,
-                                        bit_offset, 10, &nack_sn, FALSE);
+            /* NACK_SN */
+            nack_ti = proto_tree_add_bits_ret_val(tree, hf_rlc_lte_am_nack_sn, tvb,
+                                                  bit_offset, 10, &nack_sn, FALSE);
             bit_offset += 10;
+            if (check_col(pinfo->cinfo, COL_INFO)) {
+                col_append_fstr(pinfo->cinfo, COL_INFO, "  NACK_SN=%u", (guint16)nack_sn);
+            }
+            expert_add_info_format(pinfo, nack_ti, PI_SEQUENCE, PI_WARN,
+                                   "Status PDU reports NACK for SN=%u", (guint16)nack_sn);
+
 
             /* E1 */
             proto_tree_add_bits_ret_val(tree, hf_rlc_lte_am_e1, tvb,
@@ -501,10 +521,8 @@ static void dissect_rlc_lte_am_status_pdu(tvbuff_t *tvb, packet_info *pinfo,
             proto_tree_add_bits_ret_val(tree, hf_rlc_lte_am_e2, tvb,
                                         bit_offset, 1, &e2, FALSE);
             bit_offset++;
-
-            /* Reset this flag here */
-            e1 = 0;
         }
+
         if (e2) {
             /* Read SOstart, SOend */
             proto_tree_add_bits_ret_val(tree, hf_rlc_lte_am_so_start, tvb,
@@ -515,6 +533,15 @@ static void dissect_rlc_lte_am_status_pdu(tvbuff_t *tvb, packet_info *pinfo,
                                         bit_offset, 15, &so_end, FALSE);
             bit_offset += 15;
 
+            if (check_col(pinfo->cinfo, COL_INFO)) {
+                col_append_fstr(pinfo->cinfo, COL_INFO, "  (SOstart=%u SOend=%u)",
+                                (guint16)so_start, (guint16)so_end);
+
+                if ((guint16)so_end == 0x7fff) {
+                    col_append_str(pinfo->cinfo, COL_INFO, " (missing portion reaches end of AMD PDU)");
+                }
+            }
+
             /* Reset this flag here */
             e2 = 0;
         }
@@ -531,6 +558,7 @@ static void dissect_rlc_lte_am(tvbuff_t *tvb, packet_info *pinfo,
 {
     guint8 is_data;
     guint8 is_segment;
+    guint8 polling;
     guint8 fixed_extension;
     guint8 framing_info;
     gboolean first_includes_start;
@@ -575,7 +603,14 @@ static void dissect_rlc_lte_am(tvbuff_t *tvb, packet_info *pinfo,
     proto_tree_add_item(am_header_tree, hf_rlc_lte_am_rf, tvb, offset, 1, FALSE);
 
     /* Polling bit */
+    polling = (tvb_get_guint8(tvb, offset) & 0x20) >> 5;
     proto_tree_add_item(am_header_tree, hf_rlc_lte_am_p, tvb, offset, 1, FALSE);
+    if (check_col(pinfo->cinfo, COL_INFO)) {
+        col_append_str(pinfo->cinfo, COL_INFO, (polling) ? " (P) " : "     ");
+    }
+    if (polling) {
+        proto_item_append_text(am_header_ti, " (P)");
+    }
 
     /* Framing Info */
     framing_info = (tvb_get_guint8(tvb, offset) & 0x18) >> 3;
@@ -588,9 +623,13 @@ static void dissect_rlc_lte_am(tvbuff_t *tvb, packet_info *pinfo,
     /* Sequence Number */
     sn = tvb_get_ntohs(tvb, offset) & 0x03ff;
     proto_tree_add_item(am_header_tree, hf_rlc_lte_am_fixed_sn, tvb, offset, 2, FALSE);
-
     offset += 2;
 
+    if (check_col(pinfo->cinfo, COL_INFO)) {
+        col_append_fstr(pinfo->cinfo, COL_INFO, "sn=%u", sn);
+    }
+
+
     /* Show SN in AM header root */
     proto_item_append_text(am_header_ti, " (SN=%u)", sn);
     proto_item_set_len(am_header_ti, offset-start_offset);
@@ -629,6 +668,7 @@ static void dissect_rlc_lte_am(tvbuff_t *tvb, packet_info *pinfo,
             show_PDU_in_info(pinfo, s_lengths[n],
                              (n==0) ? first_includes_start : TRUE,
                              TRUE);
+            tvb_ensure_bytes_exist(tvb, offset, s_lengths[n]);
             offset += s_lengths[n];
         }
     }
@@ -688,9 +728,11 @@ void dissect_rlc_lte(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
                                   tvb, 0, 0, p_rlc_lte_info->rlcMode);
     PROTO_ITEM_SET_GENERATED(mode_ti);
 
-    ti = proto_tree_add_uint(rlc_lte_tree, hf_rlc_lte_context_ueid,
-                             tvb, 0, 0, p_rlc_lte_info->ueid);
-    PROTO_ITEM_SET_GENERATED(ti);
+    if (p_rlc_lte_info->ueid != 0) {
+        ti = proto_tree_add_uint(rlc_lte_tree, hf_rlc_lte_context_ueid,
+                                 tvb, 0, 0, p_rlc_lte_info->ueid);
+        PROTO_ITEM_SET_GENERATED(ti);
+    }
 
     ti = proto_tree_add_uint(rlc_lte_tree, hf_rlc_lte_context_priority,
                              tvb, 0, 0, p_rlc_lte_info->priority);
@@ -700,9 +742,12 @@ void dissect_rlc_lte(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
                              tvb, 0, 0, p_rlc_lte_info->channelType);
     PROTO_ITEM_SET_GENERATED(ti);
 
-    ti = proto_tree_add_uint(rlc_lte_tree, hf_rlc_lte_context_channel_id,
-                             tvb, 0, 0, p_rlc_lte_info->channelId);
-    PROTO_ITEM_SET_GENERATED(ti);
+    if ((p_rlc_lte_info->channelType == CHANNEL_TYPE_SRB) ||
+        (p_rlc_lte_info->channelType == CHANNEL_TYPE_DRB)) {
+        ti = proto_tree_add_uint(rlc_lte_tree, hf_rlc_lte_context_channel_id,
+                                 tvb, 0, 0, p_rlc_lte_info->channelId);
+        PROTO_ITEM_SET_GENERATED(ti);
+    }
 
     ti = proto_tree_add_uint(rlc_lte_tree, hf_rlc_lte_context_pdu_length,
                              tvb, 0, 0, p_rlc_lte_info->pduLength);
@@ -718,12 +763,21 @@ void dissect_rlc_lte(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     /* Append context highlights to info column */
     if (check_col(pinfo->cinfo, COL_INFO)) {
         col_add_fstr(pinfo->cinfo, COL_INFO,
-                     "[%s] [%s] UEId=%u %s:%u",
+                     "[%s] [%s] ",
                      (p_rlc_lte_info->direction == 0) ? "UL" : "DL",
-                     val_to_str(p_rlc_lte_info->rlcMode, rlc_mode_short_vals, "Unknown"),
-                     p_rlc_lte_info->ueid,
-                     val_to_str(p_rlc_lte_info->channelType, rlc_channel_type_vals, "Unknown"),
-                     p_rlc_lte_info->channelId);
+                     val_to_str(p_rlc_lte_info->rlcMode, rlc_mode_short_vals, "Unknown"));
+        if (p_rlc_lte_info->ueid != 0) {
+            col_append_fstr(pinfo->cinfo, COL_INFO, "UEId=%u ", p_rlc_lte_info->ueid);
+        }
+        if (p_rlc_lte_info->channelId == 0) {
+            col_append_fstr(pinfo->cinfo, COL_INFO, "%s",
+                            val_to_str(p_rlc_lte_info->channelType, rlc_channel_type_vals, "Unknown"));
+        }
+        else {
+            col_append_fstr(pinfo->cinfo, COL_INFO, "%s:%u",
+                            val_to_str(p_rlc_lte_info->channelType, rlc_channel_type_vals, "Unknown"),
+                            p_rlc_lte_info->channelId);
+        }
     }
 
     /* Reset this count */
@@ -749,6 +803,11 @@ void dissect_rlc_lte(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
             dissect_rlc_lte_am(tvb, pinfo, rlc_lte_tree, offset);
             break;
 
+        case RLC_PREDEF:
+            /* Predefined data (i.e. not containing a valid RLC header */
+            proto_tree_add_item(rlc_lte_tree, hf_rlc_lte_predefined_pdu, tvb, offset, -1, FALSE);
+            break;
+
         default:
             /* Error - unrecognised mode */
             expert_add_info_format(pinfo, mode_ti, PI_MALFORMED, PI_ERROR,
@@ -937,7 +996,7 @@ void proto_register_rlc_lte(void)
         },
         { &hf_rlc_lte_am_segment_so,
             { "Segment Offset",
-              "rlc-lte.am.segment.so", FT_UINT16, BASE_HEX, 0, 0x7fff,
+              "rlc-lte.am.segment.offset", FT_UINT16, BASE_DEC, 0, 0x7fff,
               "Segment Offset", HFILL
             }
         },
@@ -957,7 +1016,7 @@ void proto_register_rlc_lte(void)
         },
         { &hf_rlc_lte_am_ack_sn,
             { "ACK Sequence Number",
-              "rlc-lte.am.ack-sn", FT_UINT16, BASE_DEC, 0, 0x0ffc,
+              "rlc-lte.am.ack-sn", FT_UINT16, BASE_DEC, 0, 0x0,
               "Sequence Number we're next expecting to receive", HFILL
             }
         },
@@ -992,6 +1051,12 @@ void proto_register_rlc_lte(void)
             }
         },
 
+        { &hf_rlc_lte_predefined_pdu,
+            { "Predefined data",
+              "rlc-lte.predefined-data", FT_BYTES, BASE_HEX, 0, 0x0,
+              "Predefined test data", HFILL
+            }
+        },
     };
 
     static gint *ett[] =
