@@ -6,7 +6,7 @@
  * Copyright 2000, Philips Electronics N.V.
  * Written by Andreas Sikkema <h323@ramdyne.nl>
  *
- * $Id: packet-rtp.c 26274 2008-09-25 22:06:58Z jake $
+ * $Id: packet-rtp.c 27628 2009-03-06 22:55:02Z wmeier $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -81,10 +81,10 @@
 /* #define DEBUG_FRAGMENTS   1 */
 
 typedef struct _rfc2198_hdr {
-  unsigned int pt;
-  int offset;
-  int len;
-  struct _rfc2198_hdr *next;
+	unsigned int pt;
+	int offset;
+	int len;
+	struct _rfc2198_hdr *next;
 } rfc2198_hdr;
 
 /* we have one of these for each pdu which spans more than one segment
@@ -135,6 +135,7 @@ static const fragment_items rtp_fragment_items = {
 
 static dissector_handle_t rtp_handle;
 static dissector_handle_t stun_handle;
+static dissector_handle_t stun_heur_handle;
 static dissector_handle_t t38_handle;
 static dissector_handle_t zrtp_handle;
 
@@ -481,16 +482,16 @@ dissect_rtp_heur( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 	if (version == 0) {
 		if (!(tvb_memeql(tvb, 4, "ZRTP", 4)))
 		{
-			call_dissector(zrtp_handle, tvb, pinfo, tree);
+			call_dissector_only(zrtp_handle, tvb, pinfo, tree);
 			return TRUE;
 		} else {
 			switch (global_rtp_version0_type) {
 			case RTP0_STUN:
-				call_dissector(stun_handle, tvb, pinfo, tree);
-				return TRUE;
+				return call_dissector_only(stun_heur_handle, tvb, pinfo, tree);
 				
 			case RTP0_T38:
-				call_dissector(t38_handle, tvb, pinfo, tree);
+				/* XXX: Should really be calling a heuristic dissector for T38 ??? */
+				call_dissector_only(t38_handle, tvb, pinfo, tree);
 				return TRUE;
 				
 			case RTP0_INVALID:
@@ -530,8 +531,8 @@ dissect_rtp_heur( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
  */
 static void
 process_rtp_payload(tvbuff_t *newtvb, packet_info *pinfo, proto_tree *tree,
-    proto_tree *rtp_tree,
-    unsigned int payload_type)
+		    proto_tree *rtp_tree,
+		    unsigned int payload_type)
 {
 	struct _rtp_conversation_info *p_conv_data = NULL;
 	gboolean found_match = FALSE;
@@ -579,7 +580,7 @@ process_rtp_payload(tvbuff_t *newtvb, packet_info *pinfo, proto_tree *tree,
 			payload_type_str = g_hash_table_lookup(p_conv_data->rtp_dyn_payload, &payload_type);
 			if (payload_type_str){
 				found_match = dissector_try_string(rtp_dyn_pt_dissector_table,
-													payload_type_str, newtvb, pinfo, tree);
+								   payload_type_str, newtvb, pinfo, tree);
 				/* If payload type string set from conversation and
 				 * no matching dissector found it's probably because no subdissector
 				 * exists. Don't call the dissectors based on payload number
@@ -615,9 +616,9 @@ process_rtp_payload(tvbuff_t *newtvb, packet_info *pinfo, proto_tree *tree,
  */
 static void
 dissect_rtp_data( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
-    proto_tree *rtp_tree, int offset, unsigned int data_len,
-    unsigned int data_reported_len,
-	unsigned int payload_type )
+		  proto_tree *rtp_tree, int offset, unsigned int data_len,
+		  unsigned int data_reported_len,
+		  unsigned int payload_type )
 {
 	tvbuff_t *newtvb;
 	struct _rtp_conversation_info *p_conv_data= NULL;
@@ -878,7 +879,9 @@ dissect_rtp_rfc2198(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 		    offset, 1, octet1, "Payload type: %s (%u)",
 			payload_type_str ? payload_type_str : val_to_str(hdr_new->pt, rtp_payload_type_vals, "Unknown"),
 			hdr_new->pt);
-		proto_item_append_text(ti, ": PT=%s", payload_type_str ? payload_type_str : val_to_str(hdr_new->pt, rtp_payload_type_vals, "Unknown (%u)"));
+		proto_item_append_text(ti, ": PT=%s", 
+				       payload_type_str ? payload_type_str : 
+				                          val_to_str(hdr_new->pt, rtp_payload_type_vals, "Unknown (%u)"));
 		offset += 1;
 
 		/* Timestamp offset and block length don't apply to last header */
@@ -1283,9 +1286,9 @@ dissect_rtp( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 		 * No padding.
 		 */
 		dissect_rtp_data( tvb, pinfo, tree, rtp_tree, offset,
-		    tvb_length_remaining( tvb, offset ),
-		    tvb_reported_length_remaining( tvb, offset ),
-		    payload_type );
+				  tvb_length_remaining( tvb, offset ),
+				  tvb_reported_length_remaining( tvb, offset ),
+				  payload_type );
 		rtp_info->info_payload_offset = offset;
 		rtp_info->info_payload_len = tvb_length_remaining(tvb, offset);
 	}
@@ -1362,7 +1365,7 @@ static void show_setup_info(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
 	/* Conversation and current data */
 	struct _rtp_conversation_info *p_conv_data = NULL;
-		proto_tree *rtp_setup_tree;
+	proto_tree *rtp_setup_tree;
 	proto_item *ti;
 
 	/* Use existing packet info if available */
@@ -1405,7 +1408,7 @@ dissect_pkt_ccc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
 		proto_tree_add_item(pkt_ccc_tree, hf_pkt_ccc_id, tvb, 0, 4, FALSE);
 		proto_tree_add_bytes_format(pkt_ccc_tree, hf_pkt_ccc_ts, tvb,
-		    4, 8, ptime, "NTP timestamp: %s", ntp_fmt_ts(ptime));
+					    4, 8, ptime, "NTP timestamp: %s", ntp_fmt_ts(ptime));
 	}
 
 	dissect_rtp(tvb, pinfo, tree);
@@ -1882,7 +1885,7 @@ proto_register_rtp(void)
 
 
 	proto_rtp = proto_register_protocol("Real-Time Transport Protocol",
-	    "RTP", "rtp");
+					    "RTP", "rtp");
 	proto_register_field_array(proto_rtp, hf, array_length(hf));
 	proto_register_subtree_array(ett, array_length(ett));
 
@@ -1894,11 +1897,11 @@ proto_register_rtp(void)
 	rtp_pt_dissector_table = register_dissector_table("rtp.pt",
 	                                                  "RTP payload type", FT_UINT8, BASE_DEC);
 	rtp_dyn_pt_dissector_table = register_dissector_table("rtp_dyn_payload_type",
-												    "Dynamic RTP payload type", FT_STRING, BASE_NONE);
+							      "Dynamic RTP payload type", FT_STRING, BASE_NONE);
 
 
 	rtp_hdr_ext_dissector_table = register_dissector_table("rtp_hdr_ext",
-													"RTP header extension", FT_STRING, BASE_NONE);
+							       "RTP header extension", FT_STRING, BASE_NONE);
 
 	rtp_module = prefs_register_protocol(proto_rtp, proto_reg_handoff_rtp);
 
@@ -1951,6 +1954,7 @@ proto_reg_handoff_rtp(void)
 
 		data_handle = find_dissector("data");
 		stun_handle = find_dissector("stun");
+		stun_heur_handle = find_dissector("stun-heur");
 		t38_handle = find_dissector("t38");
 		zrtp_handle = find_dissector("zrtp");
 
