@@ -4,7 +4,7 @@
  * Jason Lango <jal@netapp.com>
  * Liberally copied from packet-http.c, by Guy Harris <guy@alum.mit.edu>
  *
- * $Id: packet-rtsp.c 26196 2008-09-15 15:38:34Z wmeier $
+ * $Id: packet-rtsp.c 27254 2009-01-17 17:33:50Z wmeier $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -38,6 +38,9 @@
 #include <epan/prefs.h>
 
 #include <glib.h>
+
+#include <wsutil/str_util.h>
+
 #include <epan/packet.h>
 #include <epan/req_resp_hdrs.h>
 #include "packet-rtp.h"
@@ -547,9 +550,8 @@ dissect_rtspmessage(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	int			value_offset;
 	int			value_len;
 	e164_info_t		e164_info;
-	gint		rdt_feature_level = 0;
-	char		*media_type_str_lower_case = NULL;
-	char		*media_type_str = NULL;
+	gint			rdt_feature_level = 0;
+	gchar			*media_type_str_lower_case = NULL;
 	int			semi_colon_offset;
 	int			par_end_offset;
 
@@ -889,10 +891,8 @@ dissect_rtspmessage(tvbuff_t *tvb, int offset, packet_info *pinfo,
 					value_len = par_end_offset - offset;
 				}
 
-				media_type_str = tvb_get_ephemeral_string(tvb, offset,
-							                             value_len);
-
-				media_type_str_lower_case = g_ascii_strdown(media_type_str, -1);
+				media_type_str_lower_case = ascii_strdown_inplace(
+                                    (gchar *)tvb_get_ephemeral_string(tvb, offset, value_len));
 
 			} else if (HDR_MATCHES(rtsp_content_length))
 			{
@@ -1250,7 +1250,6 @@ proto_register_rtsp(void)
 
 	proto_rtsp = proto_register_protocol("Real Time Streaming Protocol",
 		"RTSP", "rtsp");
-	media_type_dissector_table = find_dissector_table("media_type");
 
 	proto_register_field_array(proto_rtsp, hf, array_length(hf));
 	proto_register_subtree_array(ett, array_length(ett));
@@ -1302,6 +1301,7 @@ proto_reg_handoff_rtsp(void)
 		rtp_handle = find_dissector("rtp");
 		rtcp_handle = find_dissector("rtcp");
 		rdt_handle = find_dissector("rdt");
+		media_type_dissector_table = find_dissector_table("media_type");
 		rtsp_prefs_initialized = TRUE;
 	}
 	else {
