@@ -4,7 +4,7 @@
  * Huagang XIE <huagang@intruvert.com>
  * Kees Cook <kees@outflux.net>
  *
- * $Id: packet-ssh.c 25861 2008-07-29 17:29:09Z gerald $
+ * $Id: packet-ssh.c 27170 2009-01-06 19:06:43Z sake $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -376,9 +376,9 @@ dissect_ssh(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			}
 		}
 
+		if(need_desegmentation) return;
                 if(offset <= last_offset)
                         THROW(ReportedBoundsError);
-		if(need_desegmentation) return;
 	}
 }
 
@@ -411,7 +411,7 @@ ssh_dissect_ssh2(tvbuff_t *tvb, packet_info *pinfo,
 			g_string_append_printf(title,")");
 		}
 
-		ti=proto_tree_add_text(tree,tvb,offset,-1,title->str);
+		ti=proto_tree_add_text(tree,tvb,offset,-1, "%s", title->str);
 		ssh2_tree = proto_item_add_subtree(ti ,ett_ssh2);
 		if (title) g_string_free(title,TRUE);
 	}
