@@ -7,7 +7,7 @@
  * IANA's full name is "TiVoConnect Beacon", where as TiVo's own
  * documentation calls this protocol "TiVoConnect Discovery Protocol".
  *
- * $Id: packet-tivoconnect.c 26089 2008-08-25 22:06:20Z wmeier $
+ * $Id: packet-tivoconnect.c 26281 2008-09-26 22:07:45Z wmeier $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -260,17 +260,9 @@ proto_register_tivoconnect(void)
 void
 proto_reg_handoff_tivoconnect(void)
 {
-        static gboolean inited = FALSE;
-        
-        if( !inited ) {
+    dissector_handle_t tivoconnect_handle;
 
-            dissector_handle_t tivoconnect_handle;
-
-            tivoconnect_handle = new_create_dissector_handle(dissect_tivoconnect,
-                                                         proto_tivoconnect);
-            dissector_add("udp.port", 2190, tivoconnect_handle);
-            dissector_add("tcp.port", 2190, tivoconnect_handle);
-        
-            inited = TRUE;
-        }
+    tivoconnect_handle = new_create_dissector_handle(dissect_tivoconnect, proto_tivoconnect);
+    dissector_add("udp.port", 2190, tivoconnect_handle);
+    dissector_add("tcp.port", 2190, tivoconnect_handle);
 }
