@@ -2,7 +2,7 @@
  * Routines for Universal Computer Protocol dissection
  * Copyright 2001, Tom Uijldert <tom.uijldert@cmg.nl>
  *
- * $Id: packet-ucp.c 27065 2008-12-20 00:09:02Z wmeier $
+ * $Id: packet-ucp.c 27407 2009-02-10 03:54:55Z ulfl $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -2757,7 +2757,7 @@ proto_reg_handoff_ucp(void)
     dissector_add_handle("tcp.port", ucp_handle);
 
     /* Tapping setup */
-    stats_tree_register("ucp", "ucp_messages", st_str_ucp,
+    stats_tree_register_with_group("ucp", "ucp_messages", st_str_ucp,
 			ucp_stats_tree_per_packet, ucp_stats_tree_init,
-			NULL);
+			NULL, REGISTER_STAT_GROUP_TELEPHONY);
 }
