@@ -2,7 +2,7 @@
  * Routines for Web Cache Coordination Protocol dissection
  * Jerry Talkington <jtalkington@users.sourceforge.net>
  *
- * $Id: packet-wccp.c 27265 2009-01-18 21:07:06Z gerald $
+ * $Id: packet-wccp.c 27165 2009-01-06 14:23:38Z morriss $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -479,7 +479,7 @@ dissect_wccp2_info(tvbuff_t *tvb, int offset, guint16 length,
 			break;
 		}
 
-		ti = proto_tree_add_text(wccp_tree, tvb, offset, item_length + 4,
+		ti = proto_tree_add_text(wccp_tree, tvb, offset, item_length + 4, "%s",
 		    val_to_str(type, info_type_vals, "Unknown info type (%u)"));
 		info_tree = proto_item_add_subtree(ti, ett);
 		proto_tree_add_text(info_tree, tvb, offset, 2,
