@@ -1,9 +1,9 @@
 /* packet-zrtp.c
  * Routines for zrtp packet dissection
- * IETF draft draft-zimmermann-avt-zrtp-08
+ * IETF draft draft-zimmermann-avt-zrtp-15
  * Copyright 2007, Sagar Pai <sagar@gmail.com>
  *
- * $Id: packet-zrtp.c 26274 2008-09-25 22:06:58Z jake $
+ * $Id: packet-zrtp.c 27615 2009-03-05 07:21:43Z jake $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -36,7 +36,7 @@
 #include <glib.h>
 #include <epan/packet.h>
 #include <epan/strutil.h>
-
+#include <epan/crc32.h>
 
 /*
   RTP header
@@ -103,10 +103,20 @@ static int hf_zrtp_msg_cfb = -1;
 */
 static int hf_zrtp_msg_error = -1;
 
+/*
+  Ping Data
+*/
+static int hf_zrtp_msg_ping_version = -1;
+static int hf_zrtp_msg_ping_endpointhash = -1;
+static int hf_zrtp_msg_pingack_endpointhash = -1;
+static int hf_zrtp_msg_ping_ssrc = -1;
+
 /*
   Checksum Data
 */
 static int hf_zrtp_checksum = -1;
+static int hf_zrtp_checksum_good = -1;
+static int hf_zrtp_checksum_bad = -1;
 
 /*
   Sub-Tree
@@ -123,6 +133,7 @@ static gint ett_zrtp_msg_ac = -1;
 static gint ett_zrtp_msg_cc = -1;
 static gint ett_zrtp_msg_sc = -1;
 
+static gint ett_zrtp_checksum = -1;
 
 /*
   Definitions
@@ -136,12 +147,15 @@ static gint ett_zrtp_msg_sc = -1;
 #define ZRTP_ERR_53 0x53
 #define ZRTP_ERR_54 0x54
 #define ZRTP_ERR_55 0x55
+#define ZRTP_ERR_56 0x56
 #define ZRTP_ERR_61 0x61
 #define ZRTP_ERR_62 0x62
 #define ZRTP_ERR_63 0x63
 #define ZRTP_ERR_70 0x70
 #define ZRTP_ERR_80 0x80
 #define ZRTP_ERR_90 0x90
+#define ZRTP_ERR_A0 0xA0
+#define ZRTP_ERR_100 0x100
 
 /*
   Text for Display
@@ -159,6 +173,8 @@ typedef struct _value_string_keyval {
 
 const value_zrtp_versions valid_zrtp_versions[]=
   {
+    {"1.1x"},
+    {"1.0x"},
     {"0.95"},
     {"0.90"},
     {"0.85"},
@@ -173,8 +189,9 @@ const value_string_keyval zrtp_hash_type_vals[] =
 
 const value_string_keyval zrtp_cipher_type_vals[] =
   {
-    { "AES1",	"AES-CM with 128 bit Keys"},
-    { "AES3",	"AES-CM with 256 bit Keys"},
+    { "AES1",	"AES-CM with 128 bit keys"},
+    { "AES2",	"AES-CM with 192 bit keys"},
+    { "AES3",	"AES-CM with 256 bit keys"},
     { NULL,		NULL }
   };
 
@@ -194,6 +211,7 @@ const value_string_keyval zrtp_sas_type_vals[] =
 
 const value_string_keyval zrtp_key_agreement_vals[] =
   {
+    { "DH2k",	"DH mode with p=2048 bit prime"},
     { "DH3k",	"DH mode with p=3072 bit prime"},
     { "DH4k",	"DH mode with p=4096 bit prime"},
     { "Prsh",	"Preshared non-DH mode using shared secret"},
@@ -215,12 +233,15 @@ const value_string zrtp_error_vals[] =
     { ZRTP_ERR_53, "Public key exchange not supported"},
     { ZRTP_ERR_54, "SRTP auth. tag not supported"},
     { ZRTP_ERR_55, "SAS scheme not supported"},
+    { ZRTP_ERR_56, "No shared secret available, DH mode required"},
     { ZRTP_ERR_61, "DH Error: bad pv for initiator/responder value is (1,0,p-1)"},
     { ZRTP_ERR_62, "DH Error: bad hash commitment (hvi != hashed data)"},
     { ZRTP_ERR_63, "Received relayed SAS from untrusted MiTM"},
     { ZRTP_ERR_70, "Auth. Error Bad Confirm Packet HMAC"},
     { ZRTP_ERR_80, "Nonce is reused"},
     { ZRTP_ERR_90, "Equal ZID's in Hello"},
+    { ZRTP_ERR_A0, "Service unavailable"},
+    { ZRTP_ERR_100, "GoClear packet received, but not allowed"},
     { 0, NULL}
   };
 
@@ -248,6 +269,10 @@ static void
 dissect_SASrelay(tvbuff_t *tvb, packet_info *pinfo, proto_tree *zrtp_tree);
 static void
 dissect_RelayACK(packet_info *pinfo);
+static void
+dissect_Ping(tvbuff_t *tvb, packet_info *pinfo, proto_tree *zrtp_tree);
+static void
+dissect_PingACK(tvbuff_t *tvb, packet_info *pinfo, proto_tree *zrtp_tree);
 
 
 static const gchar *
@@ -265,8 +290,9 @@ key_to_val(const gchar *key, int keylen, const value_string_keyval *kv, const gc
 static const gchar *
 check_valid_version(const gchar *version){
   int i=0;
+  int match_size = (version[0] == '0') ? 4 : 3;
   while (valid_zrtp_versions[i].version) {
-    if (!strncmp(valid_zrtp_versions[i].version, version, 4)){
+    if (!strncmp(valid_zrtp_versions[i].version, version, match_size)){
       return(valid_zrtp_versions[i].version);
     }
     i++;
@@ -281,12 +307,14 @@ dissect_zrtp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   proto_tree   *zrtp_tree;
   proto_tree   *zrtp_msg_tree;
   proto_tree   *zrtp_msg_data_tree;
+  proto_tree   *checksum_tree;
   proto_item   *ti;
   int          linelen;
   int          checksum_offset;
   unsigned char message_type[9];
   unsigned int prime_offset = 0;
   unsigned int msg_offset = 12;
+  guint32      sent_crc, calc_crc;
 
   if (check_col(pinfo->cinfo, COL_PROTOCOL))
     col_set_str(pinfo->cinfo, COL_PROTOCOL, "ZRTP");
@@ -294,84 +322,112 @@ dissect_zrtp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   if (check_col(pinfo->cinfo, COL_INFO))
     col_add_fstr(pinfo->cinfo, COL_INFO, "Unknown ZRTP Packet");
 
-    ti = proto_tree_add_protocol_format(tree,proto_zrtp,tvb,0,-1,"ZRTP protocol");
-    zrtp_tree = proto_item_add_subtree(ti,ett_zrtp);
+  ti = proto_tree_add_protocol_format(tree,proto_zrtp,tvb,0,-1,"ZRTP protocol");
+  zrtp_tree = proto_item_add_subtree(ti,ett_zrtp);
 
-    proto_tree_add_item(zrtp_tree,hf_zrtp_rtpversion,tvb,prime_offset+0,1,FALSE);
-    proto_tree_add_item(zrtp_tree,hf_zrtp_rtppadding,tvb,prime_offset+0,1,FALSE);
-    proto_tree_add_item(zrtp_tree,hf_zrtp_rtpextension,tvb,prime_offset+0,1,FALSE);
+  proto_tree_add_item(zrtp_tree,hf_zrtp_rtpversion,tvb,prime_offset+0,1,FALSE);
+  proto_tree_add_item(zrtp_tree,hf_zrtp_rtppadding,tvb,prime_offset+0,1,FALSE);
+  proto_tree_add_item(zrtp_tree,hf_zrtp_rtpextension,tvb,prime_offset+0,1,FALSE);
 
-    proto_tree_add_item(zrtp_tree,hf_zrtp_sequence,tvb,prime_offset+2,2,FALSE);
+  proto_tree_add_item(zrtp_tree,hf_zrtp_sequence,tvb,prime_offset+2,2,FALSE);
 
-    proto_tree_add_item(zrtp_tree,hf_zrtp_cookie,tvb,prime_offset+4,4,FALSE);
+  proto_tree_add_item(zrtp_tree,hf_zrtp_cookie,tvb,prime_offset+4,4,FALSE);
 
-    proto_tree_add_item(zrtp_tree,hf_zrtp_source_id,tvb,prime_offset+8,4,FALSE);
+  proto_tree_add_item(zrtp_tree,hf_zrtp_source_id,tvb,prime_offset+8,4,FALSE);
 
-    linelen = tvb_reported_length_remaining(tvb,msg_offset);
-    checksum_offset = linelen-4;
+  linelen = tvb_reported_length_remaining(tvb,msg_offset);
+  checksum_offset = linelen-4;
 
-    ti = proto_tree_add_protocol_format(zrtp_tree,proto_zrtp,tvb,msg_offset,linelen-4,"Message");
-    zrtp_msg_tree = proto_item_add_subtree(ti,ett_zrtp_msg);
+  ti = proto_tree_add_protocol_format(zrtp_tree,proto_zrtp,tvb,msg_offset,linelen-4,"Message");
+  zrtp_msg_tree = proto_item_add_subtree(ti,ett_zrtp_msg);
 
-    proto_tree_add_item(zrtp_msg_tree,hf_zrtp_signature,tvb,msg_offset+0,2,FALSE);
+  proto_tree_add_item(zrtp_msg_tree,hf_zrtp_signature,tvb,msg_offset+0,2,FALSE);
 
-    proto_tree_add_item(zrtp_msg_tree,hf_zrtp_msg_length,tvb,msg_offset+2,2,FALSE);
+  proto_tree_add_item(zrtp_msg_tree,hf_zrtp_msg_length,tvb,msg_offset+2,2,FALSE);
 
-    tvb_memcpy(tvb,(void *)message_type,msg_offset+4,8);
-    message_type[8] = 0;
-    proto_tree_add_item(zrtp_msg_tree,hf_zrtp_msg_type,tvb,msg_offset+4,8,FALSE);
+  tvb_memcpy(tvb,(void *)message_type,msg_offset+4,8);
+  message_type[8] = '\0';
+  proto_tree_add_item(zrtp_msg_tree,hf_zrtp_msg_type,tvb,msg_offset+4,8,FALSE);
 
-    linelen = tvb_reported_length_remaining(tvb,msg_offset+12);
+  linelen = tvb_reported_length_remaining(tvb,msg_offset+12);
 
-    if (!strncmp(message_type,"Hello   ",8)){
-      ti = proto_tree_add_protocol_format(zrtp_msg_tree,proto_zrtp,tvb,msg_offset+12,linelen-4,"Data");
-      zrtp_msg_data_tree = proto_item_add_subtree(ti,ett_zrtp_msg_data);
-      dissect_Hello(tvb,pinfo,zrtp_msg_data_tree);
-    } else if (!strncmp(message_type,"HelloACK",8)){
-      dissect_HelloACK(pinfo);
-    } else if (!strncmp(message_type,"Commit  ",8)){
-      ti = proto_tree_add_protocol_format(zrtp_msg_tree,proto_zrtp,tvb,msg_offset+12,linelen-4,"Data");
-      zrtp_msg_data_tree = proto_item_add_subtree(ti,ett_zrtp_msg_data);
-      dissect_Commit(tvb,pinfo,zrtp_msg_data_tree);
-    } else if (!strncmp(message_type,"DHPart1 ",8)){
-      ti = proto_tree_add_protocol_format(zrtp_msg_tree,proto_zrtp,tvb,msg_offset+12,linelen-4,"Data");
-      zrtp_msg_data_tree = proto_item_add_subtree(ti,ett_zrtp_msg_data);
-      dissect_DHPart(tvb,pinfo,zrtp_msg_data_tree,1);
-    } else if (!strncmp(message_type,"DHPart2 ",8)){
-      ti = proto_tree_add_protocol_format(zrtp_msg_tree,proto_zrtp,tvb,msg_offset+12,linelen-4,"Data");
-      zrtp_msg_data_tree = proto_item_add_subtree(ti,ett_zrtp_msg_data);
-      dissect_DHPart(tvb,pinfo,zrtp_msg_data_tree,2);
-    } else if (!strncmp(message_type,"Confirm1",8)){
-      ti = proto_tree_add_protocol_format(zrtp_msg_tree,proto_zrtp,tvb,msg_offset+12,linelen-4,"Data");
-      zrtp_msg_data_tree = proto_item_add_subtree(ti,ett_zrtp_msg_data);
-      dissect_Confirm(tvb,pinfo,zrtp_msg_data_tree,1);
-    } else if (!strncmp(message_type,"Confirm2",8)){
-      ti = proto_tree_add_protocol_format(zrtp_msg_tree,proto_zrtp,tvb,msg_offset+12,linelen-4,"Data");
-      zrtp_msg_data_tree = proto_item_add_subtree(ti,ett_zrtp_msg_data);
-      dissect_Confirm(tvb,pinfo,zrtp_msg_data_tree,2);
-    } else if (!strncmp(message_type,"Conf2ACK",8)){
-      dissect_Conf2ACK(pinfo);
-    } else if (!strncmp(message_type,"Error   ",8)){
-      ti = proto_tree_add_protocol_format(zrtp_msg_tree,proto_zrtp,tvb,msg_offset+12,linelen-4,"Data");
-      zrtp_msg_data_tree = proto_item_add_subtree(ti,ett_zrtp_msg_data);
-      dissect_Error(tvb,pinfo,zrtp_msg_data_tree);
-    } else if (!strncmp(message_type,"ErrorACK",8)){
-      dissect_ErrorACK(pinfo);
-    } else if (!strncmp(message_type,"GoClear ",8)){
-      ti = proto_tree_add_protocol_format(zrtp_msg_tree,proto_zrtp,tvb,msg_offset+12,linelen-4,"Data");
-      zrtp_msg_data_tree = proto_item_add_subtree(ti,ett_zrtp_msg_data);
-      dissect_GoClear(tvb,pinfo,zrtp_msg_data_tree);
-    } else if (!strncmp(message_type,"ClearACK",8)){
-      dissect_ClearACK(pinfo);
-    } else if (!strncmp(message_type,"SASrelay",8)){
-      ti = proto_tree_add_protocol_format(zrtp_msg_tree,proto_zrtp,tvb,msg_offset+12,linelen-4,"Data");
-      zrtp_msg_data_tree = proto_item_add_subtree(ti,ett_zrtp_msg_data);
-      dissect_SASrelay(tvb,pinfo,zrtp_msg_data_tree);
-    } else if (!strncmp(message_type,"RelayACK",8)){
-      dissect_RelayACK(pinfo);
-    }
+  if (!strncmp(message_type,"Hello   ",8)){
+    ti = proto_tree_add_protocol_format(zrtp_msg_tree,proto_zrtp,tvb,msg_offset+12,linelen-4,"Data");
+    zrtp_msg_data_tree = proto_item_add_subtree(ti,ett_zrtp_msg_data);
+    dissect_Hello(tvb,pinfo,zrtp_msg_data_tree);
+  } else if (!strncmp(message_type,"HelloACK",8)){
+    dissect_HelloACK(pinfo);
+  } else if (!strncmp(message_type,"Commit  ",8)){
+    ti = proto_tree_add_protocol_format(zrtp_msg_tree,proto_zrtp,tvb,msg_offset+12,linelen-4,"Data");
+    zrtp_msg_data_tree = proto_item_add_subtree(ti,ett_zrtp_msg_data);
+    dissect_Commit(tvb,pinfo,zrtp_msg_data_tree);
+  } else if (!strncmp(message_type,"DHPart1 ",8)){
+    ti = proto_tree_add_protocol_format(zrtp_msg_tree,proto_zrtp,tvb,msg_offset+12,linelen-4,"Data");
+    zrtp_msg_data_tree = proto_item_add_subtree(ti,ett_zrtp_msg_data);
+    dissect_DHPart(tvb,pinfo,zrtp_msg_data_tree,1);
+  } else if (!strncmp(message_type,"DHPart2 ",8)){
+    ti = proto_tree_add_protocol_format(zrtp_msg_tree,proto_zrtp,tvb,msg_offset+12,linelen-4,"Data");
+    zrtp_msg_data_tree = proto_item_add_subtree(ti,ett_zrtp_msg_data);
+    dissect_DHPart(tvb,pinfo,zrtp_msg_data_tree,2);
+  } else if (!strncmp(message_type,"Confirm1",8)){
+    ti = proto_tree_add_protocol_format(zrtp_msg_tree,proto_zrtp,tvb,msg_offset+12,linelen-4,"Data");
+    zrtp_msg_data_tree = proto_item_add_subtree(ti,ett_zrtp_msg_data);
+    dissect_Confirm(tvb,pinfo,zrtp_msg_data_tree,1);
+  } else if (!strncmp(message_type,"Confirm2",8)){
+    ti = proto_tree_add_protocol_format(zrtp_msg_tree,proto_zrtp,tvb,msg_offset+12,linelen-4,"Data");
+    zrtp_msg_data_tree = proto_item_add_subtree(ti,ett_zrtp_msg_data);
+    dissect_Confirm(tvb,pinfo,zrtp_msg_data_tree,2);
+  } else if (!strncmp(message_type,"Conf2ACK",8)){
+    dissect_Conf2ACK(pinfo);
+  } else if (!strncmp(message_type,"Error   ",8)){
+    ti = proto_tree_add_protocol_format(zrtp_msg_tree,proto_zrtp,tvb,msg_offset+12,linelen-4,"Data");
+    zrtp_msg_data_tree = proto_item_add_subtree(ti,ett_zrtp_msg_data);
+    dissect_Error(tvb,pinfo,zrtp_msg_data_tree);
+  } else if (!strncmp(message_type,"ErrorACK",8)){
+    dissect_ErrorACK(pinfo);
+  } else if (!strncmp(message_type,"GoClear ",8)){
+    ti = proto_tree_add_protocol_format(zrtp_msg_tree,proto_zrtp,tvb,msg_offset+12,linelen-4,"Data");
+    zrtp_msg_data_tree = proto_item_add_subtree(ti,ett_zrtp_msg_data);
+    dissect_GoClear(tvb,pinfo,zrtp_msg_data_tree);
+  } else if (!strncmp(message_type,"ClearACK",8)){
+    dissect_ClearACK(pinfo);
+  } else if (!strncmp(message_type,"SASrelay",8)){
+    ti = proto_tree_add_protocol_format(zrtp_msg_tree,proto_zrtp,tvb,msg_offset+12,linelen-4,"Data");
+    zrtp_msg_data_tree = proto_item_add_subtree(ti,ett_zrtp_msg_data);
+    dissect_SASrelay(tvb,pinfo,zrtp_msg_data_tree);
+  } else if (!strncmp(message_type,"RelayACK",8)){
+    dissect_RelayACK(pinfo);
+  } else if (!strncmp(message_type,"Ping    ",8)){
+    ti = proto_tree_add_protocol_format(zrtp_msg_tree,proto_zrtp,tvb,msg_offset+12,linelen-4,"Data");
+    zrtp_msg_data_tree = proto_item_add_subtree(ti,ett_zrtp_msg_data);
+    dissect_Ping(tvb,pinfo,zrtp_msg_data_tree);
+  } else if (!strncmp(message_type,"PingACK ",8)){
+    ti = proto_tree_add_protocol_format(zrtp_msg_tree,proto_zrtp,tvb,msg_offset+12,linelen-4,"Data");
+    zrtp_msg_data_tree = proto_item_add_subtree(ti,ett_zrtp_msg_data);
+    dissect_PingACK(tvb,pinfo,zrtp_msg_data_tree);
+  }
+
+  sent_crc = tvb_get_ntohl(tvb,msg_offset+checksum_offset);
+  calc_crc = ~calculate_crc32c(tvb_get_ptr(tvb,0,msg_offset+checksum_offset),msg_offset+checksum_offset,CRC32C_PRELOAD);
+
+  if (sent_crc == calc_crc) {
+    ti = proto_tree_add_uint_format_value(zrtp_tree, hf_zrtp_checksum, tvb, msg_offset+checksum_offset, 4, sent_crc,
+                                          "0x%04x [correct]", sent_crc);
+    checksum_tree = proto_item_add_subtree(ti, ett_zrtp_checksum);
+    ti = proto_tree_add_boolean(checksum_tree, hf_zrtp_checksum_good, tvb, msg_offset+checksum_offset, 4, TRUE);
+    PROTO_ITEM_SET_GENERATED(ti);
+    ti = proto_tree_add_boolean(checksum_tree, hf_zrtp_checksum_bad, tvb, msg_offset+checksum_offset, 4, FALSE);
+    PROTO_ITEM_SET_GENERATED(ti);
+  } else {
+    ti = proto_tree_add_uint_format_value(zrtp_tree, hf_zrtp_checksum, tvb, msg_offset+checksum_offset, 4, sent_crc,
+                                          "0x%04x [incorrect, should be 0x%04x]", sent_crc, calc_crc);
+    checksum_tree = proto_item_add_subtree(ti, ett_zrtp_checksum);
+    ti = proto_tree_add_boolean(checksum_tree, hf_zrtp_checksum_good, tvb, msg_offset+checksum_offset, 4, FALSE);
+    PROTO_ITEM_SET_GENERATED(ti);
+    ti = proto_tree_add_boolean(checksum_tree, hf_zrtp_checksum_bad, tvb, msg_offset+checksum_offset, 4, TRUE);
+    PROTO_ITEM_SET_GENERATED(ti);
+  }
 
-    proto_tree_add_item(zrtp_tree,hf_zrtp_checksum,tvb,msg_offset+checksum_offset,4,FALSE);
 }
 
 static void
@@ -409,6 +465,32 @@ dissect_HelloACK(packet_info *pinfo) {
   }
 }
 
+static void
+dissect_Ping(tvbuff_t *tvb, packet_info *pinfo, proto_tree *zrtp_tree) {
+  unsigned int data_offset=24;
+
+  if (check_col(pinfo->cinfo, COL_INFO)) {
+    col_add_fstr(pinfo->cinfo, COL_INFO, "Ping Packet");
+  }
+
+  proto_tree_add_item(zrtp_tree,hf_zrtp_msg_ping_version,tvb,data_offset,4,FALSE);
+  proto_tree_add_item(zrtp_tree,hf_zrtp_msg_ping_endpointhash,tvb,data_offset+4,8,FALSE);
+}
+
+static void
+dissect_PingACK(tvbuff_t *tvb, packet_info *pinfo, proto_tree *zrtp_tree) {
+  unsigned int data_offset=24;
+
+  if (check_col(pinfo->cinfo, COL_INFO)) {
+    col_add_fstr(pinfo->cinfo, COL_INFO, "PingACK Packet");
+  }
+
+  proto_tree_add_item(zrtp_tree,hf_zrtp_msg_ping_version,tvb,data_offset,4,FALSE);
+  proto_tree_add_item(zrtp_tree,hf_zrtp_msg_pingack_endpointhash,tvb,data_offset+4,8,FALSE);
+  proto_tree_add_item(zrtp_tree,hf_zrtp_msg_ping_endpointhash,tvb,data_offset+12,8,FALSE);
+  proto_tree_add_item(zrtp_tree,hf_zrtp_msg_ping_ssrc,tvb,data_offset+20,4,FALSE);
+}
+
 static void
 dissect_GoClear(tvbuff_t *tvb, packet_info *pinfo, proto_tree *zrtp_tree) {
   unsigned int data_offset=24;
@@ -486,7 +568,7 @@ dissect_DHPart(tvbuff_t *tvb, packet_info *pinfo, proto_tree *zrtp_tree,int part
   proto_tree_add_item(zrtp_tree,hf_zrtp_msg_pbxs,tvb,data_offset+24,8,FALSE);
   linelen = tvb_reported_length_remaining(tvb,data_offset+32);
   pvr_len = linelen-8-4;
-  ti = proto_tree_add_protocol_format(zrtp_tree,proto_zrtp,tvb,data_offset+32,pvr_len,"pvr/nouncer Data");
+  ti = proto_tree_add_protocol_format(zrtp_tree,proto_zrtp,tvb,data_offset+32,pvr_len,(part==1)?"pvr Data":"pvi Data");
   zrtp_msg_pvr_tree = proto_item_add_subtree(ti,ett_zrtp_msg_pvr);
   proto_tree_add_item(zrtp_tree,hf_zrtp_msg_hmac,tvb,data_offset+32+pvr_len,8,FALSE);
 }
@@ -512,19 +594,19 @@ dissect_Commit(tvbuff_t *tvb, packet_info *pinfo, proto_tree *zrtp_tree) {
   /* ZID */
   proto_tree_add_item(zrtp_tree,hf_zrtp_msg_zid,tvb,data_offset+0,12,FALSE);
   tvb_memcpy(tvb,(void *)value,data_offset+12,4);
-  value[4]=0;
+  value[4]='\0';
   proto_tree_add_string_format(zrtp_tree,hf_zrtp_msg_hash,tvb,data_offset+12,4,value,
 				  "Hash: %s",key_to_val(value,4,zrtp_hash_type_vals,"Unknown hash type %s"));
   tvb_memcpy(tvb,(void *)value,data_offset+16,4);
-  value[4]=0;
+  value[4]='\0';
   proto_tree_add_string_format(zrtp_tree,hf_zrtp_msg_cipher,tvb,data_offset+16,4,value,"Cipher: %s",
 				  key_to_val(value,4,zrtp_cipher_type_vals,"Unknown cipher type %s"));
   tvb_memcpy(tvb,(void *)value,data_offset+20,4);
-  value[4]=0;
+  value[4]='\0';
   proto_tree_add_string_format(zrtp_tree,hf_zrtp_msg_at,tvb,data_offset+20,4,value,
 				  "Auth tag: %s",key_to_val(value,4,zrtp_auth_tag_vals,"Unknown auth tag %s"));
   tvb_memcpy(tvb,(void *)value,data_offset+24,4);
-  value[4]=0;
+  value[4]='\0';
   proto_tree_add_string_format(zrtp_tree,hf_zrtp_msg_keya,tvb,data_offset+24,4,value,
 				  "Key agreement: %s",key_to_val(value,4,zrtp_key_agreement_vals,"Unknown key agreement %s"));
 
@@ -534,9 +616,9 @@ dissect_Commit(tvbuff_t *tvb, packet_info *pinfo, proto_tree *zrtp_tree) {
     key_type = 2;
   }
   tvb_memcpy(tvb,(void *)value,data_offset+28,4);
-  value[4]=0;
+  value[4]='\0';
   proto_tree_add_string_format(zrtp_tree,hf_zrtp_msg_sas,tvb,data_offset+28,4,value,
-				  "Sas type: %s",key_to_val(value,4,zrtp_sas_type_vals,"Unknown sas type %s"));
+				  "SAS type: %s",key_to_val(value,4,zrtp_sas_type_vals,"Unknown SAS type %s"));
 
   switch(key_type){
   case 1: /* 
@@ -616,7 +698,7 @@ dissect_Hello(tvbuff_t *tvb, packet_info *pinfo, proto_tree *zrtp_tree) {
   run_offset = data_offset+4;
   for(i=0;i<vhc;i++){
     tvb_memcpy(tvb,(void *)value,run_offset,4);
-    value[4]=0;
+    value[4]='\0';
     proto_tree_add_string_format(tmp_tree,hf_zrtp_msg_hash,tvb,run_offset,4,value,
 				    "Hash[%d]: %s",i,key_to_val(value,4,zrtp_hash_type_vals,"Unknown hash type %s"));
     run_offset+=4;
@@ -625,7 +707,7 @@ dissect_Hello(tvbuff_t *tvb, packet_info *pinfo, proto_tree *zrtp_tree) {
   tmp_tree = proto_item_add_subtree(ti,ett_zrtp_msg_cc);
   for(i=0;i<vcc;i++){
     tvb_memcpy(tvb,(void *)value,run_offset,4);
-    value[4]=0;
+    value[4]='\0';
     proto_tree_add_string_format(tmp_tree,hf_zrtp_msg_cipher,tvb,run_offset,4,value,"Cipher[%d]: %s",i,
 				    key_to_val(value,4,zrtp_cipher_type_vals,"Unknown cipher type %s"));
     run_offset+=4;
@@ -634,7 +716,7 @@ dissect_Hello(tvbuff_t *tvb, packet_info *pinfo, proto_tree *zrtp_tree) {
   tmp_tree = proto_item_add_subtree(ti,ett_zrtp_msg_ac);  
   for(i=0;i<vac;i++){
     tvb_memcpy(tvb,(void *)value,run_offset,4);
-    value[4]=0;
+    value[4]='\0';
     proto_tree_add_string_format(tmp_tree,hf_zrtp_msg_at,tvb,run_offset,4,value,
 				    "Auth tag[%d]: %s",i,key_to_val(value,4,zrtp_auth_tag_vals,"Unknown auth tag %s"));
     run_offset+=4;
@@ -643,18 +725,18 @@ dissect_Hello(tvbuff_t *tvb, packet_info *pinfo, proto_tree *zrtp_tree) {
   tmp_tree = proto_item_add_subtree(ti,ett_zrtp_msg_kc);  
   for(i=0;i<vkc;i++){
     tvb_memcpy(tvb,(void *)value,run_offset,4);
-    value[4]=0;
+    value[4]='\0';
     proto_tree_add_string_format(tmp_tree,hf_zrtp_msg_keya,tvb,run_offset,4,value,
 				    "Key agreement[%d]: %s",i,key_to_val(value,4,zrtp_key_agreement_vals,"Unknown key agreement %s"));
     run_offset+=4;
   }
-  ti=proto_tree_add_uint_format(zrtp_tree,hf_zrtp_msg_sas_count,tvb,data_offset+3,1,sc,"Sas type count = %d",vsc);
+  ti=proto_tree_add_uint_format(zrtp_tree,hf_zrtp_msg_sas_count,tvb,data_offset+3,1,sc,"SAS type count = %d",vsc);
   tmp_tree = proto_item_add_subtree(ti,ett_zrtp_msg_sc);  
   for(i=0;i<vsc;i++){
     tvb_memcpy(tvb,(void *)value,run_offset,4);
-    value[4]=0;
+    value[4]='\0';
     proto_tree_add_string_format(tmp_tree,hf_zrtp_msg_sas,tvb,run_offset,4,value,
-				    "Sas type[%d]: %s",i,key_to_val(value,4,zrtp_sas_type_vals,"Unknown sas type %s"));
+				    "SAS type[%d]: %s",i,key_to_val(value,4,zrtp_sas_type_vals,"Unknown SAS type %s"));
     run_offset+=4;
   }
 
@@ -670,7 +752,7 @@ proto_register_zrtp(void)
        "RTP Version", "zrtp.rtpversion",
        FT_UINT8, BASE_DEC,
        NULL, 0xC0,
-       "", HFILL
+       NULL, HFILL
      }
     },
 
@@ -679,7 +761,7 @@ proto_register_zrtp(void)
        "RTP padding", "zrtp.rtppadding",
        FT_BOOLEAN, 8,
        NULL, 0x20,
-       "", HFILL
+       NULL, HFILL
      }
     },
 
@@ -688,7 +770,7 @@ proto_register_zrtp(void)
        "RTP Extension", "zrtp.rtpextension",
        FT_BOOLEAN, 8,
        NULL, 0x10,
-       "", HFILL
+       NULL, HFILL
      }
     },
 
@@ -697,7 +779,7 @@ proto_register_zrtp(void)
        "ID", "zrtp.id",
        FT_UINT8, BASE_HEX,
        NULL, 0x0,
-       "", HFILL
+       NULL, HFILL
      }
     },
 
@@ -706,7 +788,7 @@ proto_register_zrtp(void)
        "Sequence", "zrtp.sequence",
        FT_UINT16, BASE_DEC,
        NULL, 0x0,
-       "", HFILL
+       NULL, HFILL
      }
     },
 
@@ -715,7 +797,7 @@ proto_register_zrtp(void)
        "Magic Cookie", "zrtp.cookie",
        FT_STRING, BASE_NONE,
        NULL, 0x0,
-       "", HFILL
+       NULL, HFILL
      }
     },
 
@@ -724,7 +806,7 @@ proto_register_zrtp(void)
        "Source Identifier", "zrtp.source_id",
        FT_UINT32, BASE_HEX,
        NULL, 0x0,
-       "", HFILL
+       NULL, HFILL
      }
     },
 
@@ -736,7 +818,7 @@ proto_register_zrtp(void)
        "Signature", "zrtp.signature",
        FT_UINT16, BASE_HEX,
        NULL, 0x0,
-       "", HFILL
+       NULL, HFILL
      }
     },
 
@@ -745,7 +827,7 @@ proto_register_zrtp(void)
        "Length", "zrtp.length",
        FT_UINT16, BASE_DEC,
        NULL, 0x0,
-       "", HFILL
+       NULL, HFILL
      }
     },
 
@@ -754,15 +836,16 @@ proto_register_zrtp(void)
        "Type", "zrtp.type",
        FT_STRING, BASE_NONE,
        NULL, 0x0,
-       "", HFILL
+       NULL, HFILL
      }
     },
+
     {&hf_zrtp_msg_version,
      {
        "ZRTP protocol version", "zrtp.version",
        FT_STRING, BASE_NONE,
        NULL, 0x0,
-       "", HFILL
+       NULL, HFILL
      }
     },
 
@@ -771,7 +854,7 @@ proto_register_zrtp(void)
        "Client Identifier", "zrtp.client_source_id",
        FT_STRING, BASE_NONE,
        NULL, 0x0,
-       "", HFILL
+       NULL, HFILL
      }
     },
 
@@ -780,7 +863,7 @@ proto_register_zrtp(void)
        "Hash Image", "zrtp.hash_image",
        FT_BYTES, BASE_NONE,
        NULL, 0x0,
-       "", HFILL
+       NULL, HFILL
      }
     },
 
@@ -789,7 +872,7 @@ proto_register_zrtp(void)
        "ZID", "zrtp.zid",
        FT_BYTES, BASE_DEC,
        NULL, 0x0,
-       "", HFILL
+       NULL, HFILL
      }
     },
 
@@ -798,7 +881,7 @@ proto_register_zrtp(void)
        "MiTM", "zrtp.mitm",
        FT_BOOLEAN, 8,
        NULL, 0x20,
-       "", HFILL
+       NULL, HFILL
      }
     },
 
@@ -807,7 +890,7 @@ proto_register_zrtp(void)
        "Passive", "zrtp.passive",
        FT_BOOLEAN, 8,
        NULL, 0x10,
-       "", HFILL
+       NULL, HFILL
      }
     },
 
@@ -816,7 +899,7 @@ proto_register_zrtp(void)
        "Hash Count", "zrtp.hc",
        FT_UINT8, BASE_DEC,
        NULL, 0x0F,
-       "", HFILL
+       NULL, HFILL
      }
     },
 
@@ -825,7 +908,7 @@ proto_register_zrtp(void)
        "Cipher Count", "zrtp.cc",
        FT_UINT8, BASE_DEC,
        NULL, 0xF0,
-       "", HFILL
+       NULL, HFILL
      }
     },
 
@@ -834,7 +917,7 @@ proto_register_zrtp(void)
        "Auth tag Count", "zrtp.ac",
        FT_UINT8, BASE_DEC,
        NULL, 0x0F,
-       "", HFILL
+       NULL, HFILL
      }
     },
 
@@ -843,7 +926,7 @@ proto_register_zrtp(void)
        "Key Agreement Count", "zrtp.kc",
        FT_UINT8, BASE_DEC,
        NULL, 0xF0,
-       "", HFILL
+       NULL, HFILL
      }
     },
 
@@ -852,7 +935,7 @@ proto_register_zrtp(void)
        "SAS Count", "zrtp.sc",
        FT_UINT8, BASE_DEC,
        NULL, 0x0F,
-       "", HFILL
+       NULL, HFILL
      }
     },
 
@@ -861,7 +944,7 @@ proto_register_zrtp(void)
        "Hash", "zrtp.hash",
        FT_STRING, BASE_NONE,
        NULL, 0x0,
-       "", HFILL
+       NULL, HFILL
      }
     },
 
@@ -870,7 +953,7 @@ proto_register_zrtp(void)
        "Cipher", "zrtp.cipher",
        FT_STRING, BASE_NONE,
        NULL, 0x0,
-       "", HFILL
+       NULL, HFILL
      }
     },
 
@@ -879,7 +962,7 @@ proto_register_zrtp(void)
        "AT", "zrtp.at",
        FT_STRING, BASE_NONE,
        NULL, 0x0,
-       "", HFILL
+       NULL, HFILL
      }
     },
 
@@ -888,7 +971,7 @@ proto_register_zrtp(void)
        "Key Agreement", "zrtp.keya",
        FT_STRING, BASE_NONE,
        NULL, 0x0,
-       "", HFILL
+       NULL, HFILL
      }
     },
 
@@ -897,7 +980,7 @@ proto_register_zrtp(void)
        "SAS", "zrtp.sas",
        FT_STRING, BASE_NONE,
        NULL, 0x0,
-       "", HFILL
+       NULL, HFILL
      }
     },
 
@@ -906,7 +989,7 @@ proto_register_zrtp(void)
        "rs1ID", "zrtp.rs1id",
        FT_BYTES, BASE_NONE,
        NULL, 0x0,
-       "", HFILL 
+       NULL, HFILL 
      }
     },
 
@@ -915,7 +998,7 @@ proto_register_zrtp(void)
        "rs2ID", "zrtp.rs2id",
        FT_BYTES, BASE_NONE,
        NULL, 0x0,
-       "", HFILL
+       NULL, HFILL
      }
     },
 
@@ -924,7 +1007,7 @@ proto_register_zrtp(void)
        "auxs", "zrtp.auxs",
        FT_BYTES, BASE_NONE,
        NULL, 0x0,
-       "", HFILL
+       NULL, HFILL
      }
     },
 
@@ -933,7 +1016,7 @@ proto_register_zrtp(void)
        "pbxs", "zrtp.pbxs",
        FT_BYTES, BASE_NONE,
        NULL, 0x0,
-       "", HFILL
+       NULL, HFILL
      }
     },
 
@@ -942,7 +1025,7 @@ proto_register_zrtp(void)
        "HMAC", "zrtp.hmac",
        FT_BYTES, BASE_NONE,
        NULL, 0x0,
-       "", HFILL
+       NULL, HFILL
      }
     },
 
@@ -951,7 +1034,7 @@ proto_register_zrtp(void)
        "CFB", "zrtp.cfb",
        FT_BYTES, BASE_NONE,
        NULL, 0x0,
-       "", HFILL
+       NULL, HFILL
      }
     },
 
@@ -960,7 +1043,43 @@ proto_register_zrtp(void)
        "Error", "zrtp.error",
        FT_UINT32, BASE_DEC,
        VALS(zrtp_error_vals), 0x0,
-       "", HFILL
+       NULL, HFILL
+     }
+    },
+
+    {&hf_zrtp_msg_ping_version,
+     {
+       "Ping Version", "zrtp.ping_version",
+       FT_STRING, BASE_NONE,
+       NULL, 0x0,
+       NULL, HFILL
+     }
+    },
+
+    {&hf_zrtp_msg_ping_endpointhash,
+     {
+       "Ping Endpoint Hash", "zrtp.ping_endpointhash",
+       FT_UINT64, BASE_HEX,
+       NULL, 0x0,
+       NULL, HFILL
+     }
+    },
+
+    {&hf_zrtp_msg_pingack_endpointhash,
+     {
+       "PingAck Endpoint Hash", "zrtp.pingack_endpointhash",
+       FT_UINT64, BASE_HEX,
+       NULL, 0x0,
+       NULL, HFILL
+     }
+    },
+
+    {&hf_zrtp_msg_ping_ssrc,
+     {
+       "Ping SSRC", "zrtp.ping_ssrc",
+       FT_UINT32, BASE_HEX,
+       NULL, 0x0,
+       NULL, HFILL
      }
     },
 
@@ -969,7 +1088,25 @@ proto_register_zrtp(void)
        "Checksum", "zrtp.checksum",
        FT_UINT32, BASE_HEX,
        NULL, 0x0,
-       "", HFILL
+       NULL, HFILL
+     }
+    },
+
+    {&hf_zrtp_checksum_good,
+     {
+       "Good", "zrtp.checksum_good",
+       FT_BOOLEAN, BASE_NONE,
+       NULL, 0x0,
+       "True: checksum matches packet content; False: doesn't match content", HFILL
+     }
+    },
+
+    {&hf_zrtp_checksum_bad,
+     {
+       "Bad", "zrtp.checksum_bad",
+       FT_BOOLEAN, BASE_NONE,
+       NULL, 0x0,
+       "True: checksum doesn't match packet content; False: matches content", HFILL
      }
     },
 
@@ -978,7 +1115,7 @@ proto_register_zrtp(void)
        "hvi", "zrtp.hvi",
        FT_BYTES, BASE_NONE,
        NULL, 0x0,
-       "", HFILL
+       NULL, HFILL
      }
     },
 
@@ -987,7 +1124,7 @@ proto_register_zrtp(void)
        "nonce", "zrtp.nonce",
        FT_BYTES, BASE_NONE,
        NULL, 0x0,
-       "", HFILL
+       NULL, HFILL
      }
     },
 
@@ -996,7 +1133,7 @@ proto_register_zrtp(void)
        "key ID", "zrtp.key_id",
        FT_BYTES, BASE_NONE,
        NULL, 0x0,
-       "", HFILL
+       NULL, HFILL
      }
     }
   };
@@ -1011,7 +1148,8 @@ proto_register_zrtp(void)
     &ett_zrtp_msg_kc,
     &ett_zrtp_msg_ac,
     &ett_zrtp_msg_cc,
-    &ett_zrtp_msg_sc
+    &ett_zrtp_msg_sc,
+    &ett_zrtp_checksum
   };
 
   proto_zrtp = proto_register_protocol("ZRTP", "ZRTP", "zrtp");
@@ -1023,13 +1161,8 @@ proto_register_zrtp(void)
 void
 proto_reg_handoff_zrtp(void)
 {
-  static gboolean inited = FALSE;
+  dissector_handle_t zrtp_handle;
 
-  if (!inited){
-    dissector_handle_t zrtp_handle;
-
-    zrtp_handle = create_dissector_handle(dissect_zrtp, proto_zrtp);
-    dissector_add_handle("udp.port", zrtp_handle);
-    inited = TRUE;
-  }
+  zrtp_handle = find_dissector("zrtp");
+  dissector_add_handle("udp.port", zrtp_handle);
 }
