@@ -1,7 +1,7 @@
 /* filesystem.c
  * Filesystem utility routines
  *
- * $Id: filesystem.c 26113 2008-09-02 11:17:03Z wmeier $
+ * $Id: filesystem.c 27718 2009-03-13 22:06:48Z stig $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -42,6 +42,10 @@
 #include <unistd.h>
 #endif
 
+#ifdef HAVE_FCNTL_H
+#include <fcntl.h>
+#endif
+
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
@@ -56,9 +60,12 @@
 #endif
 
 #include "filesystem.h"
+#include "report_err.h"
 #include <wsutil/privileges.h>
 #include <wsutil/file_util.h>
 
+#include <wiretap/wtap.h>	/* for WTAP_ERR_SHORT_WRITE */
+
 #define PROFILES_DIR    "profiles"
 #define U3_MY_CAPTURES  "\\My Captures"
 
@@ -246,22 +253,22 @@ init_progfile_dir(const char *arg0
 #ifdef _WIN32
 	TCHAR prog_pathname_w[_MAX_PATH+2];
 	size_t progfile_dir_len;
-        char *prog_pathname;
-        DWORD error;
-        TCHAR *msg_w;
-        guchar *msg;
-        size_t msglen;
+	char *prog_pathname;
+	DWORD error;
+	TCHAR *msg_w;
+	guchar *msg;
+	size_t msglen;
 
 	/*
 	 * Attempt to get the full pathname of the currently running
 	 * program.
 	 */
 	if (GetModuleFileName(NULL, prog_pathname_w, sizeof prog_pathname_w) != 0) {
-                /*
-                 * XXX - Should we use g_utf16_to_utf8(), as in
-                 * getenv_utf8()?
-                 */
-                prog_pathname = utf_16to8(prog_pathname_w);
+		/*
+		 * XXX - Should we use g_utf16_to_utf8(), as in
+		 * getenv_utf8()?
+		 */
+		prog_pathname = utf_16to8(prog_pathname_w);
 		/*
 		 * We got it; strip off the last component, which would be
 		 * the file name of the executable, giving us the pathname
@@ -805,9 +812,7 @@ char *getenv_utf8(const char *varname)
 void
 set_profile_name(const gchar *profilename)
 {
-	if (persconfprofile) {
-		g_free (persconfprofile);
-	}
+	g_free (persconfprofile);
 
 	if (profilename && strlen(profilename) > 0 &&
 	    strcmp(profilename, DEFAULT_PROFILE) != 0) {
@@ -927,10 +932,7 @@ get_profiles_dir(void)
 {
 	static char *profiles_dir = NULL;
 
-	if (profiles_dir) {
-		g_free (profiles_dir);
-	}
-
+	g_free (profiles_dir);
 	profiles_dir = g_strdup_printf ("%s%s%s", get_persconffile_dir_no_profile (),
 					G_DIR_SEPARATOR_S, PROFILES_DIR);
 
@@ -942,16 +944,14 @@ get_persconffile_dir(const gchar *profilename)
 {
 	static char *persconffile_profile_dir = NULL;
 
-	if (persconffile_profile_dir) {
-		g_free (persconffile_profile_dir);
-	}
+	g_free (persconffile_profile_dir);
 
 	if (profilename && strlen(profilename) > 0 &&
 	    strcmp(profilename, DEFAULT_PROFILE) != 0) {
 	  persconffile_profile_dir = g_strdup_printf ("%s%s%s", get_profiles_dir (),
 						      G_DIR_SEPARATOR_S, profilename);
 	} else {
-	  persconffile_profile_dir = g_strdup_printf (get_persconffile_dir_no_profile ());
+	  persconffile_profile_dir = g_strdup (get_persconffile_dir_no_profile ());
 	}
 
 	return persconffile_profile_dir;
@@ -970,8 +970,8 @@ profile_exists(const gchar *profilename)
 static int
 delete_directory (const char *directory, char **pf_dir_path_return)
 {
-	ETH_DIR *dir;
-	ETH_DIRENT *file;
+	WS_DIR *dir;
+	WS_DIRENT *file;
 	gchar *filename;
 	int ret = 0;
 
@@ -1552,6 +1552,72 @@ files_identical(const char *fname1, const char *fname2)
 #endif
 }
 
+/*
+ * Copy a file in binary mode, for those operating systems that care about
+ * such things.  This should be OK for all files, even text files, as
+ * we'll copy the raw bytes, and we don't look at the bytes as we copy
+ * them.
+ *
+ * Returns TRUE on success, FALSE on failure. If a failure, it also
+ * displays a simple dialog window with the error message.
+ */
+gboolean
+copy_file_binary_mode(const char *from_filename, const char *to_filename)
+{
+  int           from_fd, to_fd, nread, nwritten, err;
+  guint8        pd[65536];
+
+  /* Copy the raw bytes of the file. */
+  from_fd = ws_open(from_filename, O_RDONLY | O_BINARY, 0000 /* no creation so don't matter */);
+  if (from_fd < 0) {
+    report_open_failure(from_filename, errno, FALSE);
+    goto done;
+  }
+
+  /* Use open() instead of creat() so that we can pass the O_BINARY
+     flag, which is relevant on Win32; it appears that "creat()"
+     may open the file in text mode, not binary mode, but we want
+     to copy the raw bytes of the file, so we need the output file
+     to be open in binary mode. */
+  to_fd = ws_open(to_filename, O_WRONLY | O_CREAT | O_TRUNC | O_BINARY, 0644);
+  if (to_fd < 0) {
+    report_open_failure(to_filename, errno, TRUE);
+    ws_close(from_fd);
+    goto done;
+  }
+
+  while ((nread = ws_read(from_fd, pd, sizeof pd)) > 0) {
+    nwritten = ws_write(to_fd, pd, nread);
+    if (nwritten < nread) {
+      if (nwritten < 0)
+	err = errno;
+      else
+	err = WTAP_ERR_SHORT_WRITE;
+      report_write_failure(to_filename, err);
+      ws_close(from_fd);
+      ws_close(to_fd);
+      goto done;
+    }
+  }
+  if (nread < 0) {
+    err = errno;
+    report_read_failure(from_filename, err);
+    ws_close(from_fd);
+    ws_close(to_fd);
+    goto done;
+  }
+  ws_close(from_fd);
+  if (ws_close(to_fd) < 0) {
+    report_write_failure(to_filename, errno);
+    goto done;
+  }
+
+  return TRUE;
+
+done:
+  return FALSE;
+}
+
 /*
  * Editor modelines
  *
@@ -1564,4 +1630,3 @@ files_identical(const char *fname1, const char *fname2)
  * ex: set shiftwidth=4 tabstop=4 noexpandtab
  * :indentSize=4:tabSize=4:noTabs=false:
  */
-
