@@ -1,7 +1,7 @@
 /* strutil.c
  * String utility routines
  *
- * $Id: strutil.c 25726 2008-07-12 19:28:24Z stig $
+ * $Id: strutil.c 27478 2009-02-17 23:15:35Z gerald $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -951,3 +951,11 @@ epan_strcasestr(const char *haystack, const char *needle)
 	}
 	return NULL;
 }
+
+const char *
+string_or_null(const char *string)
+{
+  if (string)
+    return string;
+  return "[NULL]";
+}
