@@ -1,7 +1,7 @@
 /*
  *  uat.c
  *
- * $Id: uat.c 27134 2008-12-30 17:21:19Z martinm $
+ * $Id: uat.c 27664 2009-03-08 22:36:43Z stig $
  *
  *  User Accessible Tables
  *  Mantain an array of user accessible data strucures
@@ -100,7 +100,7 @@ uat_t* uat_new(const char* name,
 	uat->help = help;
 	uat->category = category;
 
-	for (i=0;flds_array[i].name;i++) {
+	for (i=0;flds_array[i].title;i++) {
 		fld_data_t* f = g_malloc(sizeof(fld_data_t));
 
 		f->colnum = i+1;
@@ -248,12 +248,24 @@ gboolean uat_save(uat_t* uat, char** error) {
 
 	fp = ws_fopen(fname,"w");
 
+    if (!fp && errno == ENOENT) {
+        /* Parent directory does not exist, try creating first */
+        gchar *pf_dir_path = NULL;
+        if (create_persconffile_dir(&pf_dir_path) != 0) {
+            *error = ep_strdup_printf("uat_save: error creating '%s'", pf_dir_path);
+            g_free (pf_dir_path);
+            return FALSE;
+        }
+        fp = ws_fopen(fname,"w");
+    }
+
 	if (!fp) {
 		*error = ep_strdup_printf("uat_save: error opening '%s': %s",fname,strerror(errno));
 		return FALSE;
 	}
 
 	*error = NULL;
+	g_free (fname);
 
 	fprintf(fp,"# This file is automatically generated, DO NOT MODIFY.\n");
 
@@ -404,26 +416,30 @@ gboolean uat_fld_chk_proto(void* u1 _U_, const char* strptr, unsigned len, void*
 }
 
 gboolean uat_fld_chk_num_dec(void* u1 _U_, const char* strptr, unsigned len, void* u2 _U_, void* u3 _U_, const char** err) {
-	char* str = ep_strndup(strptr,len);
-	long i = strtol(str,&str,10);
+    if (len > 0) {
+        char* str = ep_strndup(strptr,len);
+        long i = strtol(str,&str,10);
 
-	if ( ( i == 0) && (errno == ERANGE || errno == EINVAL) ) {
-		*err = strerror(errno);
-		return FALSE;
-	}
+        if ( ( i == 0) && (errno == ERANGE || errno == EINVAL) ) {
+            *err = strerror(errno);
+            return FALSE;
+        }
+    }
 
 	*err = NULL;
 	return TRUE;
 }
 
 gboolean uat_fld_chk_num_hex(void* u1 _U_, const char* strptr, unsigned len, void* u2 _U_, void* u3 _U_, const char** err) {
-	char* str = ep_strndup(strptr,len);
-	long i = strtol(str,&str,16);
+    if (len > 0) {
+        char* str = ep_strndup(strptr,len);
+        long i = strtol(str,&str,16);
 
-	if ( ( i == 0) && (errno == ERANGE || errno == EINVAL) ) {
-		*err = strerror(errno);
-		return FALSE;
-	}
+        if ( ( i == 0) && (errno == ERANGE || errno == EINVAL) ) {
+            *err = strerror(errno);
+            return FALSE;
+        }
+    }
 
 	*err = NULL;
 	return TRUE;
