@@ -3,9 +3,9 @@
  *
  * Wireshark's interface to the Lua Programming Language
  *
- * (c) 2006, Luis E. Garcia Ontanon <luis.ontanon@gmail.com>
+ * (c) 2006, Luis E. Garcia Ontanon <luis@ontanon.org>
  *
- * $Id: wslua_dumper.c 20330 2007-01-05 15:09:33Z lego $
+ * $Id: wslua_dumper.c 27309 2009-01-27 14:36:46Z stig $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -26,7 +26,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-/* WSLUA_MODULE Dumper saving capture files */
+/* WSLUA_MODULE Dumper Saving capture files */
 
 #include "wslua.h"
 #include <math.h>
@@ -77,12 +77,12 @@ WSLUA_CONSTRUCTOR PseudoHeader_eth(lua_State* L) {
 	 Creates an ethernet pseudoheader
 	 */
 
-#define WSLUA_OPTARG_PseudoHeader_eth_FCSLEN 1 /* the fcs lenght */
+#define WSLUA_OPTARG_PseudoHeader_eth_FCSLEN 1 /* the fcs length */
 	
     PseudoHeader ph = g_malloc(sizeof(struct lua_pseudo_header));
     ph->type = PHDR_ETH;
     ph->wph = g_malloc(sizeof(union wtap_pseudo_header));
-    ph->wph->eth.fcs_len = luaL_optint(L,1,-1);
+    ph->wph->eth.fcs_len = luaL_optint(L,WSLUA_OPTARG_PseudoHeader_eth_FCSLEN,-1);
     
     pushPseudoHeader(L,ph);
 	
@@ -125,9 +125,9 @@ WSLUA_CONSTRUCTOR PseudoHeader_mtp2(lua_State* L) {
     PseudoHeader ph = g_malloc(sizeof(struct lua_pseudo_header));
     ph->type = PHDR_MTP2;
     ph->wph = g_malloc(sizeof(union wtap_pseudo_header));
-    ph->wph->mtp2.sent = luaL_optint(L,1,0);
-    ph->wph->mtp2.annex_a_used = luaL_optint(L,2,0);
-    ph->wph->mtp2.link_number = luaL_optint(L,3,0);
+    ph->wph->mtp2.sent = luaL_optint(L,WSLUA_OPTARG_PseudoHeader_mtp2_SENT,0);
+    ph->wph->mtp2.annex_a_used = luaL_optint(L,WSLUA_OPTARG_PseudoHeader_mtp2_ANNEXA,0);
+    ph->wph->mtp2.link_number = luaL_optint(L,WSLUA_OPTARG_PseudoHeader_mtp2_LINKNUM,0);
 
     pushPseudoHeader(L,ph);
 	WSLUA_RETURN(1); /* The MTP2 pseudoheader */
@@ -195,9 +195,9 @@ WSLUA_CONSTRUCTOR Dumper_new(lua_State* L) {
 #define WSLUA_OPTARG_Dumper_new_FILETYPE 2 /* The type of the file to be created */
 #define WSLUA_OPTARG_Dumper_new_ENCAP 3 /* The encapsulation to be used in the file to be created */
     Dumper d;
-    const char* fname = luaL_checkstring(L,1);
-    int filetype = luaL_optint(L,2,WTAP_FILE_PCAP);
-    int encap  = luaL_optint(L,3,WTAP_ENCAP_ETHERNET);
+    const char* fname = luaL_checkstring(L,WSLUA_ARG_Dumper_new_FILENAME);
+    int filetype = luaL_optint(L,WSLUA_OPTARG_Dumper_new_FILETYPE,WTAP_FILE_PCAP);
+    int encap  = luaL_optint(L,WSLUA_OPTARG_Dumper_new_ENCAP,WTAP_ENCAP_ETHERNET);
     int err = 0;
     const char* filename;
 	
@@ -286,8 +286,9 @@ WSLUA_METHOD Dumper_dump(lua_State* L) {
     
 	if (! ba) WSLUA_ARG_ERROR(Dumper_dump,BYTEARRAY,"must be a ByteArray");
     
-    pkthdr.ts.secs = (int)floor(ts);
-    pkthdr.ts.nsecs = (int)floor(ts - pkthdr.ts.secs) * 1000000000;
+	pkthdr.ts.secs = (unsigned)floor(ts);
+	pkthdr.ts.nsecs = (unsigned)floor((ts - (double)pkthdr.ts.secs) * 1000000000);
+
     pkthdr.len  = ba->len;
     pkthdr.caplen  = ba->len;    
     pkthdr.pkt_encap = DUMPER_ENCAP(d);
@@ -308,7 +309,7 @@ WSLUA_METHOD Dumper_new_for_current(lua_State* L) {
 #define WSLUA_OPTARG_Dumper_new_for_current_FILETYPE 2 /* The file type. Defaults to pcap. */
 	Dumper d;
     const char* fname = luaL_checkstring(L,1);
-    int filetype = luaL_optint(L,2,WTAP_FILE_PCAP);
+    int filetype = luaL_optint(L,WSLUA_OPTARG_Dumper_new_for_current_FILETYPE,WTAP_FILE_PCAP);
     int encap;
     int err = 0;
 	const char* filename;
