@@ -7,7 +7,7 @@
  * Copyright 2000, Jeffrey C. Foster <jfoste@woodward.com> and
  * Guy Harris <guy@alum.mit.edu>
  *
- * $Id: dfilter_expr_dlg.c 25634 2008-06-29 15:51:43Z wmeier $
+ * $Id: dfilter_expr_dlg.c 27718 2009-03-13 22:06:48Z stig $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -568,6 +568,7 @@ value_list_sel_cb(GtkTreeSelection *sel, gpointer value_entry_arg)
          */
         switch (hfinfo->display) {
 
+        case BASE_NONE:
         case BASE_DEC:
             switch (hfinfo->type) {
 
@@ -732,8 +733,7 @@ dfilter_expr_dlg_accept_cb(GtkWidget *w, gpointer filter_te_arg)
                           "Ranges of that field can't be tested with \"%s\".",
                           item_str);
         }
-        if (range_str != NULL)
-            g_free(range_str);
+        g_free(range_str);
         return;
     }
 
@@ -752,8 +752,7 @@ dfilter_expr_dlg_accept_cb(GtkWidget *w, gpointer filter_te_arg)
                           "That field must be compared with a value, "
                           "but you didn't specify a value with which to "
                           "compare it.");
-            if (range_str != NULL)
-                g_free(range_str);
+            g_free(range_str);
             g_free(value_str);
             return;
         }
@@ -780,8 +779,7 @@ dfilter_expr_dlg_accept_cb(GtkWidget *w, gpointer filter_te_arg)
              * The dialog box was already popped up by
              * "dfilter_report_bad_value()".
              */
-            if (range_str != NULL)
-                g_free(range_str);
+            g_free(range_str);
             g_free(value_str);
             return;
         }
@@ -1119,7 +1117,6 @@ dfilter_expr_dlg_new(GtkWidget *filter_te)
 
     GtkTreeIter iter, child_iter;
     void *cookie, *cookie2;
-    const gchar *name;
 
     for (i = proto_get_first_protocol(&cookie); i != -1;
 	 i = proto_get_next_protocol(&cookie)) {
@@ -1131,11 +1128,16 @@ dfilter_expr_dlg_new(GtkWidget *filter_te)
 	    continue;
 	}
 
-	name = proto_get_protocol_short_name(protocol); /* name, short_name or filter name ? */
+	g_snprintf(str, TAG_STRING_LEN, "%s - %s", 
+		   proto_get_protocol_short_name(protocol),
+		   proto_get_protocol_long_name(protocol));
+	str[TAG_STRING_LEN]='\0';
+	strp=str;
+
 	hfinfo = proto_registrar_get_nth(i);
 
 	gtk_tree_store_append(store, &iter, NULL);
-	gtk_tree_store_set(store, &iter, 0, name, 1, hfinfo, -1);
+	gtk_tree_store_set(store, &iter, 0, strp, 1, hfinfo, -1);
 
 	for (hfinfo = proto_get_first_protocol_field(i, &cookie2); hfinfo != NULL;
              hfinfo = proto_get_next_protocol_field(&cookie2)) {
@@ -1151,7 +1153,6 @@ dfilter_expr_dlg_new(GtkWidget *filter_te)
                            hfinfo->name);
             }
             str[TAG_STRING_LEN]='\0';
-            strp=str;
             gtk_tree_store_append(store, &child_iter, &iter);
             gtk_tree_store_set(store, &child_iter, 0, strp, 1, hfinfo, -1);
 	}
