@@ -1,7 +1,7 @@
 /* fc_stat.c
  * fc_stat   2003 Ronnie Sahlberg
  *
- * $Id: fc_stat.c 25933 2008-08-05 17:33:14Z morriss $
+ * $Id: fc_stat.c 26641 2008-10-31 09:53:56Z stig $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -181,7 +181,7 @@ gtk_fcstat_init(const char *optarg, void *userdata _U_)
 
 	error_string=register_tap_listener("fc", fc, filter, fcstat_reset, fcstat_packet, fcstat_draw);
 	if(error_string){
-		simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, error_string->str);
+		simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, "%s", error_string->str);
 		g_string_free(error_string, TRUE);
 		g_free(fc);
 		return;
@@ -201,6 +201,7 @@ gtk_fcstat_init(const char *optarg, void *userdata _U_)
 	window_present(fc->win);
 
 	cf_retap_packets(&cfile, FALSE);
+	gdk_window_raise(fc->win->window);
 }
 
 static tap_dfilter_dlg fc_stat_dlg = {
