@@ -1,7 +1,7 @@
 /* filter_dlg.c
  * Dialog boxes for (display and capture) filter editing
  *
- * $Id: filter_dlg.c 25788 2008-07-22 15:19:10Z martinm $
+ * $Id: filter_dlg.c 27718 2009-03-13 22:06:48Z stig $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -32,6 +32,7 @@
 
 #include <epan/filesystem.h>
 #include <epan/prefs.h>
+#include <epan/proto.h>
 
 #include "../filters.h"
 #include "../simple_dialog.h"
@@ -381,6 +382,7 @@ filter_dialog_new(GtkWidget *button, GtkWidget *parent_filter_te,
     GtkTreeViewColumn *column;
     GtkTreeSelection  *sel;
     GtkTreeIter       *l_select;
+    gchar *list_name = NULL;
 
     /* Get a pointer to a static variable holding the type of filter on
        which we're working, so we can pass that pointer to callback
@@ -391,12 +393,14 @@ filter_dialog_new(GtkWidget *button, GtkWidget *parent_filter_te,
         filter_dialogs = &cfilter_dialogs;
         filter_list_type_p = &cfilter_list_type;
         list_type = CFILTER_EDITED_LIST;
+        list_name = "Capture Filter";
         break;
 
     case DFILTER_LIST:
         filter_dialogs = &dfilter_dialogs;
         filter_list_type_p = &dfilter_list_type;
         list_type = DFILTER_EDITED_LIST;
+        list_name = "Display Filter";
         break;
 
     default:
@@ -456,7 +460,7 @@ filter_dialog_new(GtkWidget *button, GtkWidget *parent_filter_te,
     gtk_box_pack_start (GTK_BOX (list_bb), del_bt, FALSE, FALSE, 0);
     gtk_tooltips_set_tip (tooltips, del_bt, ("Delete the selected filter"), NULL);
 
-    filter_fr = gtk_frame_new("Filter");
+    filter_fr = gtk_frame_new(list_name);
     gtk_box_pack_start(GTK_BOX(top_hb), filter_fr, TRUE, TRUE, 0);
     gtk_widget_show(filter_fr);
 
@@ -535,9 +539,13 @@ filter_dialog_new(GtkWidget *button, GtkWidget *parent_filter_te,
     gtk_box_pack_start(GTK_BOX(bottom_hb), filter_te, TRUE, TRUE, 0);
     g_object_set_data(G_OBJECT(main_w), E_FILT_FILTER_TE_KEY, filter_te);
     g_signal_connect(filter_te, "changed", G_CALLBACK(filter_name_te_changed_cb), filter_list_type_p);
-    g_object_set_data(G_OBJECT(main_w), E_FILT_AUTOCOMP_PTR_KEY, NULL);
-    g_signal_connect(filter_te, "key-press-event", G_CALLBACK (filter_string_te_key_pressed_cb), NULL);
-    g_signal_connect(main_w, "key-press-event", G_CALLBACK (filter_parent_dlg_key_pressed_cb), NULL);
+    if (list_type == DFILTER_EDITED_LIST) {
+        colorize_filter_te_as_empty(filter_te);
+
+	g_object_set_data(G_OBJECT(main_w), E_FILT_AUTOCOMP_PTR_KEY, NULL);
+	g_signal_connect(filter_te, "key-press-event", G_CALLBACK (filter_string_te_key_pressed_cb), NULL);
+	g_signal_connect(main_w, "key-press-event", G_CALLBACK (filter_parent_dlg_key_pressed_cb), NULL);
+    }
     gtk_widget_show(filter_te);
 
     g_object_set_data(G_OBJECT(main_w), E_FILT_PARENT_FILTER_TE_KEY, parent_filter_te);
@@ -1017,10 +1025,8 @@ filter_sel_list_cb(GtkTreeSelection *sel, gpointer data _U_)
         gtk_widget_set_sensitive(copy_bt, sensitivity);
     if (del_bt != NULL)
         gtk_widget_set_sensitive(del_bt, sensitivity);
-    if (name != NULL)
-        g_free(name);
-    if (strval != NULL)
-        g_free(strval);
+    g_free(name);
+    g_free(strval);
 }
 
 /* To do: add input checking to each of these callbacks */
@@ -1242,25 +1248,30 @@ filter_add_expr_bt_cb(GtkWidget *w _U_, gpointer main_w_arg)
 static void
 color_filter_te(GtkWidget *w, guint16 red, guint16 green, guint16 blue)
 {
+    static GdkColor black = { 0, 0, 0, 0 };
     GdkColor    bg;
-    GtkStyle    *style;
 
     bg.pixel    = 0;
     bg.red      = red;
     bg.green    = green;
     bg.blue     = blue;
 
-    style = gtk_style_copy(gtk_widget_get_style(w));
-    style->base[GTK_STATE_NORMAL] = bg;
-    gtk_widget_set_style(w, style);
-    gtk_style_unref(style);
+    gtk_widget_modify_text(w, GTK_STATE_NORMAL, &black);
+    gtk_widget_modify_base(w, GTK_STATE_NORMAL, &bg);
+#if GTK_CHECK_VERSION(2,12,0)
+    gtk_widget_modify_cursor(w, &black, &black);
+#endif
 }
 
 void
 colorize_filter_te_as_empty(GtkWidget *w)
 {
-    /* white */
-    color_filter_te(w, 0xFFFF, 0xFFFF, 0xFFFF);
+    /* use defaults */
+    gtk_widget_modify_text(w, GTK_STATE_NORMAL, NULL);
+    gtk_widget_modify_base(w, GTK_STATE_NORMAL, NULL);
+#if GTK_CHECK_VERSION(2,12,0)
+    gtk_widget_modify_cursor(w, NULL, NULL);
+#endif
 }
 
 void
@@ -1290,14 +1301,28 @@ filter_te_syntax_check_cb(GtkWidget *w)
     const gchar *strval;
     dfilter_t   *dfp;
     GPtrArray   *depr = NULL;
+    gboolean     use_statusbar;
     gchar       *msg;
-
-    statusbar_pop_filter_msg();
+    guchar       c;
 
     strval = gtk_entry_get_text(GTK_ENTRY(w));
+    use_statusbar = g_object_get_data(G_OBJECT(w), E_FILT_FIELD_USE_STATUSBAR_KEY) ? TRUE : FALSE;
+
+    if (use_statusbar) {
+      statusbar_pop_filter_msg();
+    }
 
     /* colorize filter string entry */
-    if (strval && dfilter_compile(strval, &dfp)) {
+    if (g_object_get_data(G_OBJECT(w), E_FILT_FIELD_NAME_ONLY_KEY) &&
+	strval && (c = proto_check_field_name(strval)) != 0)
+    {
+        colorize_filter_te_as_invalid(w);
+	if (use_statusbar) {
+	  msg = g_strdup_printf(" Illegal character in field name: '%c'", c);
+	  statusbar_push_filter_msg(msg);
+	  g_free(msg);
+	}
+    } else if (strval && dfilter_compile(strval, &dfp)) {
     	if (dfp != NULL) {
           depr = dfilter_deprecated_tokens(dfp);
           dfilter_free(dfp);
@@ -1307,19 +1332,23 @@ filter_te_syntax_check_cb(GtkWidget *w)
         } else if (depr) {
             /* You keep using that word. I do not think it means what you think it means. */
             colorize_filter_te_as_deprecated(w);
-            /*
-             * We're being lazy and only printing the first "problem" token.
-             * Would it be better to print all of them?
-             */
-            msg = g_strdup_printf(" \"%s\" may have unexpected results (see the User's Guide)",
-                (const char *) g_ptr_array_index(depr, 0));
-            statusbar_push_filter_msg(msg);
-            g_free(msg);
+	    if (use_statusbar) {
+	      /*
+	       * We're being lazy and only printing the first "problem" token.
+	       * Would it be better to print all of them?
+	       */
+	      msg = g_strdup_printf(" \"%s\" may have unexpected results (see the User's Guide)",
+				    (const char *) g_ptr_array_index(depr, 0));
+	      statusbar_push_filter_msg(msg);
+	      g_free(msg);
+	    }
         } else {
             colorize_filter_te_as_valid(w);
         }
     } else {
         colorize_filter_te_as_invalid(w);
-        statusbar_push_filter_msg(" Invalid filter");
+	if (use_statusbar) {
+	  statusbar_push_filter_msg(" Invalid filter");
+	}
     }
 }
