@@ -1,7 +1,7 @@
 /* main_airpcap_toolbar.c
  * The airpcap toolbar
  *
- * $Id: main_airpcap_toolbar.c 24964 2008-04-12 23:34:44Z ulfl $
+ * $Id: main_airpcap_toolbar.c 26487 2008-10-17 23:48:33Z gerald $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -23,7 +23,7 @@
  */
 
 /*
- * This file implements the "filter" toolbar for Wireshark.
+ * This file implements the wireless toolbar for Wireshark.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -63,7 +63,7 @@ static GtkWidget *driver_warning_dialog;
  * Changed callback for the channel combobox
  */
 static void
-airpcap_toolbar_channel_changed_cb(GtkWidget *w _U_, gpointer data)
+airpcap_toolbar_channel_changed_cb(GtkWidget *w, gpointer data)
 {
   const gchar *s;
   ULONG ch_freq;
@@ -84,12 +84,16 @@ airpcap_toolbar_channel_changed_cb(GtkWidget *w _U_, gpointer data)
  * Changed callback for the channel offset combobox
  */
 static void
-on_channel_offset_cb_changed(GtkWidget *w _U_, gpointer data)
+on_channel_offset_cb_changed(GtkWidget *w, gpointer data _U_)
 {
     const gchar *s;
-    gchar offset;
+    int offset;
 
-    if ((data != NULL) && (w != NULL) && change_airpcap_settings)
+    if (w == NULL) {
+        return;
+    }
+
+    if (change_airpcap_settings)
     {
         s = gtk_entry_get_text(GTK_ENTRY(w));
         if ((g_ascii_strcasecmp("",s)))
@@ -98,10 +102,7 @@ on_channel_offset_cb_changed(GtkWidget *w _U_, gpointer data)
             {
                 sscanf(s,"%d",&offset);
                 airpcap_if_active->channelInfo.ExtChannel = offset;
-                if (change_airpcap_settings != NULL)
-                {
-                    airpcap_update_frequency_and_offset(airpcap_if_active);
-                }
+                airpcap_update_frequency_and_offset(airpcap_if_active);
             }
         }
     }
@@ -132,11 +133,11 @@ airpcap_toolbar_wrong_crc_combo_cb(GtkWidget *entry, gpointer user_data)
 void
 airpcap_toolbar_encryption_cb(GtkWidget *entry, gpointer user_data)
 {
-  /* We need to directly access the .ddl functions here... */
+  /* We need to directly access the .dll functions here... */
   gchar ebuf[AIRPCAP_ERRBUF_SIZE];
   PAirpcapHandle ad;
 
- gint n = 0;
+  gint n = 0;
   gint i = 0;
   airpcap_if_info_t* curr_if = NULL;
 
@@ -235,7 +236,7 @@ GtkWidget *airpcap_toolbar_new()
     		  *channel_offset_cb = NULL,
     		  *wrong_crc_lb = NULL,
     		  *wrong_crc_cm = NULL;
-    GtkWidget *airpcap_tb;
+    GtkWidget     *airpcap_tb;
 
     GtkWidget     *enable_decryption_lb;
     GtkWidget     *enable_decryption_cb;
@@ -352,7 +353,7 @@ GtkWidget *airpcap_toolbar_new()
     /* callback for channel combo box */
     g_signal_connect(GTK_COMBO(channel_cm)->entry,"changed", G_CALLBACK(airpcap_toolbar_channel_changed_cb), channel_offset_cb);
     /* callback for channel offset combo box */
-    g_signal_connect(GTK_COMBO(channel_offset_cb)->entry,"changed", G_CALLBACK(on_channel_offset_cb_changed), channel_offset_cb);
+    g_signal_connect(GTK_COMBO(channel_offset_cb)->entry,"changed", G_CALLBACK(on_channel_offset_cb_changed), NULL);
     /* End: Channel offset combo box */
 
     /* Wrong CRC Label */
@@ -397,7 +398,7 @@ GtkWidget *airpcap_toolbar_new()
     gtk_widget_set_name (enable_decryption_lb, "enable_decryption_lb");
     gtk_widget_show (enable_decryption_lb);
     gtk_toolbar_append_widget(GTK_TOOLBAR(airpcap_tb), enable_decryption_lb,
-    NULL, "Private");
+        NULL, "Private");
 
     enable_decryption_cb = gtk_combo_new ();
     gtk_widget_set_name (enable_decryption_cb, "enable_decryption_cb");
@@ -444,7 +445,7 @@ GtkWidget *airpcap_toolbar_new()
     /* If no airpcap interface is present, gray everything */
     if(airpcap_if_active == NULL) {
         if(airpcap_if_list == NULL || g_list_length(airpcap_if_list) == 0) {
-            /*No airpcap device found */
+            /* No airpcap device found */
             airpcap_enable_toolbar_widgets(airpcap_tb,FALSE);
             /* recent.airpcap_toolbar_show = TRUE; */
         } else {
@@ -481,7 +482,7 @@ void airpcap_toolbar_show(GtkWidget *airpcap_tb)
       /* Ask the user what to do ...*/
       airpcap_keys_check_w(NULL,NULL);
     } else {
-      /* Keys from lists are equals, or wireshark has got no keys */
+      /* Keys from lists are equals, or Wireshark has got no keys */
       airpcap_load_decryption_keys(airpcap_if_list);
     }
   }
