@@ -1,7 +1,7 @@
 /* column_prefs.c
  * Dialog box for column preferences
  *
- * $Id: prefs_column.c 26491 2008-10-19 13:40:52Z stig $
+ * $Id: prefs_column.c 27718 2009-03-13 22:06:48Z stig $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -42,17 +42,16 @@
 #include "gtk/filter_autocomplete.h"
 
 
-static GtkWidget *column_l, *del_bt, *title_te, *field_te, *field_lb, *fmt_cmb, *up_bt, *dn_bt;
+static GtkWidget *remove_bt, *field_te, *field_lb, *fmt_cmb;
 static gint       cur_fmt, cur_row;
 
-static void   column_list_select_cb(GtkTreeSelection *, gpointer);
-static void   column_entry_changed_cb(GtkEditable *, gpointer);
-static void   column_field_changed_cb(GtkEditable *, gpointer);
-static void   column_list_new_cb(GtkWidget *, gpointer);
-static void   column_menu_changed_cb(GtkWidget *, gpointer);
-static void   column_list_delete_cb(GtkWidget *, gpointer);
-static void   column_arrow_cb(GtkWidget *, gpointer);
-void          column_set_arrow_button_sensitivity(GList *);
+static void column_list_select_cb(GtkTreeSelection *, gpointer);
+static void column_field_changed_cb(GtkEditable *, gpointer);
+static void column_list_new_cb(GtkWidget *, gpointer);
+static void column_menu_changed_cb(GtkWidget *, gpointer);
+static void column_list_delete_cb(GtkWidget *, gpointer);
+static void column_dnd_row_deleted_cb(GtkTreeModel *, GtkTreePath *, gpointer);
+static gboolean column_title_changed_cb(GtkCellRendererText *, const gchar *, const gchar *, gpointer);
 
 #define E_COL_NAME_KEY "column_name"
 #define E_COL_LBL_KEY  "column_label"
@@ -62,11 +61,10 @@ void          column_set_arrow_button_sensitivity(GList *);
 /* Called when the 'Columns' preference notebook page is selected. */
 GtkWidget *
 column_prefs_show(GtkWidget *prefs_window) {
-  GtkWidget         *main_vb, *top_hb, *new_bt,
+  GtkWidget         *main_vb, *bottom_hb, *column_l, *add_bt,
                     *tb, *lb;
-  GtkWidget         *order_fr, *order_vb, *order_lb;
-  GtkWidget         *list_fr, *list_vb, *list_lb, *list_sc;
-  GtkWidget         *edit_fr, *edit_vb;
+  GtkWidget         *list_vb, *list_lb, *list_sc;
+  GtkWidget         *add_remove_vb;
   GtkWidget         *props_fr, *props_hb;
   GList             *clp = NULL;
   fmt_data          *cfmt;
@@ -87,43 +85,17 @@ column_prefs_show(GtkWidget *prefs_window) {
   gtk_widget_show(main_vb);
   g_object_set_data(G_OBJECT(GTK_OBJECT(main_vb)), E_COL_CM_KEY, (gpointer)FALSE);
 
-  /* Top row: Column list and buttons */
-  top_hb = gtk_hbox_new(FALSE, 5);
-  gtk_container_add(GTK_CONTAINER(main_vb), top_hb);
-  gtk_widget_show(top_hb);
-
-
-  /* edit frame */
-  edit_fr = gtk_frame_new("Edit");
-  gtk_box_pack_start (GTK_BOX (top_hb), edit_fr, FALSE, FALSE, 0);
-  gtk_widget_show(edit_fr);
-
-  edit_vb = gtk_vbox_new (TRUE, 0);
-  gtk_container_set_border_width  (GTK_CONTAINER (edit_vb), 5);
-  gtk_container_add(GTK_CONTAINER(edit_fr), edit_vb);
-  gtk_widget_show(edit_vb);
-
-  new_bt = gtk_button_new_from_stock(GTK_STOCK_NEW);
-  g_signal_connect(new_bt, "clicked", G_CALLBACK(column_list_new_cb), NULL);
-  gtk_box_pack_start (GTK_BOX (edit_vb), new_bt, FALSE, FALSE, 5);
-  gtk_widget_show(new_bt);
-
-  del_bt = gtk_button_new_from_stock(GTK_STOCK_DELETE);
-  gtk_widget_set_sensitive(del_bt, FALSE);
-  g_signal_connect(del_bt, "clicked", G_CALLBACK(column_list_delete_cb), NULL);
-  gtk_box_pack_start (GTK_BOX (edit_vb), del_bt, FALSE, FALSE, 5);
-  gtk_widget_show(del_bt);
-
-
-  /* columns list frame */
-  list_fr = gtk_frame_new("Columns");
-  gtk_box_pack_start (GTK_BOX (top_hb), list_fr, TRUE, TRUE, 0);
+  /* Top row: Columns list frame */
+  /*list_fr = gtk_frame_new("Columns");
+  gtk_box_pack_start (GTK_BOX (main_vb), list_fr, TRUE, TRUE, 0);
   gtk_widget_show(list_fr);
+  */
 
   list_vb = gtk_vbox_new (FALSE, 0);
   gtk_container_set_border_width  (GTK_CONTAINER (list_vb), 5);
   gtk_widget_show (list_vb);
-  gtk_container_add(GTK_CONTAINER(list_fr), list_vb);
+  gtk_box_pack_start (GTK_BOX (main_vb), list_vb, TRUE, TRUE, 0);
+  /*gtk_container_add(GTK_CONTAINER(list_fr), list_vb);*/
 
   list_lb = gtk_label_new (("[First list entry will be displayed left]"));
   gtk_widget_show (list_lb);
@@ -136,10 +108,15 @@ column_prefs_show(GtkWidget *prefs_window) {
   gtk_widget_show(list_sc);
 
   store = gtk_list_store_new(3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER);
+  g_signal_connect(GTK_TREE_MODEL(store), "row-deleted", G_CALLBACK(column_dnd_row_deleted_cb), NULL);
+
   column_l = tree_view_new(GTK_TREE_MODEL(store));
   gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(column_l), TRUE);
   gtk_tree_view_set_headers_clickable(GTK_TREE_VIEW(column_l), FALSE);
+  gtk_tree_view_set_reorderable(GTK_TREE_VIEW(column_l), TRUE);
   renderer = gtk_cell_renderer_text_new();
+  g_object_set(G_OBJECT(renderer), "editable", TRUE, NULL);
+  g_signal_connect (renderer, "edited", G_CALLBACK(column_title_changed_cb), GTK_TREE_MODEL(store));
   column = gtk_tree_view_column_new_with_attributes(column_titles[0], renderer,
                                                     "text", 0, NULL);
   gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
@@ -152,8 +129,8 @@ column_prefs_show(GtkWidget *prefs_window) {
   /* XXX - make this match the packet list prefs? */
   sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(column_l));
   gtk_tree_selection_set_mode(sel, GTK_SELECTION_SINGLE);
+  g_signal_connect(sel, "changed", G_CALLBACK(column_list_select_cb), column_l);
 
-  g_signal_connect(sel, "changed", G_CALLBACK(column_list_select_cb), NULL);
   gtk_container_add(GTK_CONTAINER(list_sc), column_l);
   gtk_widget_show(column_l);
 
@@ -176,71 +153,59 @@ column_prefs_show(GtkWidget *prefs_window) {
     g_free (fmt);
   }
   g_object_unref(G_OBJECT(store));
+
   
-
-  /* order frame */
-  order_fr = gtk_frame_new("Order");
-  gtk_box_pack_start (GTK_BOX (top_hb), order_fr, FALSE, FALSE, 0);
-  gtk_widget_show(order_fr);
-
-  order_vb = gtk_vbox_new (TRUE, 0);
-  gtk_container_add(GTK_CONTAINER(order_fr), order_vb);
-  gtk_container_set_border_width  (GTK_CONTAINER (order_vb), 5);
-  gtk_widget_show(order_vb);
-
-  up_bt = gtk_button_new_from_stock(GTK_STOCK_GO_UP);
-  gtk_widget_set_sensitive(up_bt, FALSE);
-  g_signal_connect(up_bt, "clicked", G_CALLBACK(column_arrow_cb), NULL);
-  gtk_box_pack_start(GTK_BOX(order_vb), up_bt, FALSE, FALSE, 0);
-  gtk_widget_show(up_bt);
-
-  order_lb = gtk_label_new (("Move\nselected\ncolumn\nup or down"));
-  gtk_widget_show (order_lb);
-  gtk_box_pack_start (GTK_BOX (order_vb), order_lb, FALSE, FALSE, 0);
-
-  dn_bt = gtk_button_new_from_stock(GTK_STOCK_GO_DOWN);
-  gtk_widget_set_sensitive(dn_bt, FALSE);
-  g_signal_connect(dn_bt, "clicked", G_CALLBACK(column_arrow_cb), NULL);
-  gtk_box_pack_start(GTK_BOX(order_vb), dn_bt, FALSE, FALSE, 0);
-  gtk_widget_show(dn_bt);
+  /* Bottom row: Add/remove buttons and properties */
+  bottom_hb = gtk_hbox_new(FALSE, 5);
+  gtk_box_pack_start (GTK_BOX (main_vb), bottom_hb, FALSE, TRUE, 0);
+  /*gtk_container_add(GTK_CONTAINER(main_vb), bottom_hb);*/
+  gtk_widget_show(bottom_hb);
 
 
+  /* Add / remove buttons */
+  add_remove_vb = gtk_vbox_new (FALSE, 0);
+  gtk_container_set_border_width  (GTK_CONTAINER (add_remove_vb), 5);
+  gtk_box_pack_start (GTK_BOX (bottom_hb), add_remove_vb, FALSE, TRUE, 0);
+  /*gtk_container_add(GTK_CONTAINER(bottom_hb), add_remove_vb);*/
+  gtk_widget_show(add_remove_vb);
+
+  add_bt = gtk_button_new_from_stock(GTK_STOCK_ADD);
+  g_signal_connect(add_bt, "clicked", G_CALLBACK(column_list_new_cb), column_l);
+  gtk_box_pack_start (GTK_BOX (add_remove_vb), add_bt, FALSE, FALSE, 5);
+  gtk_widget_show(add_bt);
+
+  remove_bt = gtk_button_new_from_stock(GTK_STOCK_REMOVE);
+  gtk_widget_set_sensitive(remove_bt, FALSE);
+  g_signal_connect(remove_bt, "clicked", G_CALLBACK(column_list_delete_cb), column_l);
+  gtk_box_pack_start (GTK_BOX (add_remove_vb), remove_bt, FALSE, FALSE, 5);
+  gtk_widget_show(remove_bt);
+  
   /* properties frame */
   props_fr = gtk_frame_new("Properties");
-  gtk_box_pack_start (GTK_BOX (main_vb), props_fr, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (bottom_hb), props_fr, TRUE, TRUE, 0);
   gtk_widget_show(props_fr);
 
-  /* Colunm name entry and format selection */
-  tb = gtk_table_new(2, 4, FALSE);
+  /* Column name entry and format selection */
+  tb = gtk_table_new(1, 4, FALSE);
   gtk_container_set_border_width(GTK_CONTAINER(tb), 5);
   gtk_container_add(GTK_CONTAINER(props_fr), tb);
   gtk_table_set_row_spacings(GTK_TABLE(tb), 10);
   gtk_table_set_col_spacings(GTK_TABLE(tb), 15);
   gtk_widget_show(tb);
 
-  lb = gtk_label_new("Title:");
+  lb = gtk_label_new("Format:");
   gtk_misc_set_alignment(GTK_MISC(lb), 1.0, 0.5);
   gtk_table_attach_defaults(GTK_TABLE(tb), lb, 0, 1, 0, 1);
   gtk_widget_show(lb);
 
-  title_te = gtk_entry_new();
-  gtk_table_attach_defaults(GTK_TABLE(tb), title_te, 1, 4, 0, 1);
-  gtk_widget_set_sensitive(title_te, FALSE);
-  gtk_widget_show(title_te);
-
-  lb = gtk_label_new("Format:");
-  gtk_misc_set_alignment(GTK_MISC(lb), 1.0, 0.5);
-  gtk_table_attach_defaults(GTK_TABLE(tb), lb, 0, 1, 1, 2);
-  gtk_widget_show(lb);
-
   props_hb = gtk_hbox_new(FALSE, 5);
-  gtk_table_attach(GTK_TABLE(tb), props_hb, 1, 2, 1, 2, GTK_FILL,
+  gtk_table_attach(GTK_TABLE(tb), props_hb, 1, 2, 0, 1, GTK_FILL,
                    GTK_SHRINK, 0, 0);
   gtk_widget_show(props_hb);
 
   field_lb = gtk_label_new("Field name:");
   gtk_misc_set_alignment(GTK_MISC(field_lb), 1.0, 0.5);
-  gtk_table_attach_defaults(GTK_TABLE(tb), field_lb, 2, 3, 1, 2);
+  gtk_table_attach_defaults(GTK_TABLE(tb), field_lb, 2, 3, 0, 1);
   gtk_widget_hide(field_lb);
 
   field_te = gtk_entry_new();
@@ -250,7 +215,7 @@ column_prefs_show(GtkWidget *prefs_window) {
   g_signal_connect(field_te, "key-press-event", G_CALLBACK (filter_string_te_key_pressed_cb), NULL);
   g_signal_connect(prefs_window, "key-press-event", G_CALLBACK (filter_parent_dlg_key_pressed_cb), NULL);
   colorize_filter_te_as_empty(field_te);
-  gtk_table_attach_defaults(GTK_TABLE(tb), field_te, 3, 4, 1, 2);
+  gtk_table_attach_defaults(GTK_TABLE(tb), field_te, 3, 4, 0, 1);
   gtk_widget_set_sensitive(field_te, FALSE);
   gtk_widget_hide(field_te);
 
@@ -259,8 +224,7 @@ column_prefs_show(GtkWidget *prefs_window) {
   for (i = 0; i < NUM_COL_FMTS; i++)
     gtk_combo_box_append_text(GTK_COMBO_BOX(fmt_cmb), col_format_desc(i));
 
-  g_signal_connect(fmt_cmb, "changed", G_CALLBACK(column_menu_changed_cb),
-	NULL);
+  g_signal_connect(fmt_cmb, "changed", G_CALLBACK(column_menu_changed_cb), column_l);
 
   cur_fmt = 0;
   gtk_combo_box_set_active(GTK_COMBO_BOX(fmt_cmb), cur_fmt);
@@ -278,8 +242,9 @@ column_prefs_show(GtkWidget *prefs_window) {
    the currently selected item.  Set the up/down button sensitivity.
    Draw focus to the entry widget. */
 static void
-column_list_select_cb(GtkTreeSelection *sel, gpointer  user_data _U_)
+column_list_select_cb(GtkTreeSelection *sel, gpointer data)
 {
+    GtkTreeView  *column_l = GTK_TREE_VIEW(data);
     fmt_data     *cfmt;
     GList        *clp;
     GtkTreeModel *model;
@@ -302,9 +267,6 @@ column_list_select_cb(GtkTreeSelection *sel, gpointer  user_data _U_)
         g_free(str_path);
         gtk_tree_path_free(path);
 
-        gtk_entry_set_text(GTK_ENTRY(title_te), cfmt->title);
-        g_signal_connect(title_te, "changed", G_CALLBACK(column_entry_changed_cb), column_l);
-
         if (cur_fmt == COL_CUSTOM) {
             gtk_entry_set_text(GTK_ENTRY(field_te), cfmt->custom_field);
             gtk_widget_show(field_lb);
@@ -315,42 +277,35 @@ column_list_select_cb(GtkTreeSelection *sel, gpointer  user_data _U_)
         }
         g_signal_connect(field_te, "changed", G_CALLBACK(column_field_changed_cb), column_l);
 
-        gtk_editable_select_region(GTK_EDITABLE(title_te), 0, -1);
-        gtk_widget_grab_focus(title_te);
-
         gtk_combo_box_set_active(GTK_COMBO_BOX(fmt_cmb), cur_fmt);
 
-        gtk_widget_set_sensitive(del_bt, TRUE);
-        gtk_widget_set_sensitive(title_te, TRUE);
+        gtk_widget_set_sensitive(remove_bt, TRUE);
         gtk_widget_set_sensitive(field_te, TRUE);
         gtk_widget_set_sensitive(fmt_cmb, TRUE);
-        column_set_arrow_button_sensitivity(clp);
     }
     else
     {
         cur_row = -1;
-        gtk_editable_delete_text(GTK_EDITABLE(title_te), 0, -1);
         gtk_editable_delete_text(GTK_EDITABLE(field_te), 0, -1);
 
-        gtk_widget_set_sensitive(del_bt, FALSE);
-        gtk_widget_set_sensitive(title_te, FALSE);
+        gtk_widget_set_sensitive(remove_bt, FALSE);
         gtk_widget_set_sensitive(field_te, FALSE);
         gtk_widget_set_sensitive(fmt_cmb, FALSE);
-        gtk_widget_set_sensitive(up_bt, FALSE);
-        gtk_widget_set_sensitive(dn_bt, FALSE);
     }
 }
 
 /* To do: add input checking to each of these callbacks */
 
 static void
-column_list_new_cb(GtkWidget *w _U_, gpointer data _U_) {
-    fmt_data     *cfmt;
-    const gchar  *title = "New Column";
-    GtkTreeModel *model;
-    GtkTreeIter   iter;
-    GtkTreePath  *path;
-    gchar        *str_path;
+column_list_new_cb(GtkWidget *w _U_, gpointer data) {
+    fmt_data          *cfmt;
+    const gchar       *title = "New Column";
+    GtkTreeView       *column_l = GTK_TREE_VIEW(data);
+    GtkTreeModel      *model;
+    GtkTreeIter        iter;
+    GtkTreePath       *path;
+    GtkTreeViewColumn *title_column;
+    gchar             *str_path;
 
     cur_fmt        = COL_NUMBER;
     cfmt           = (fmt_data *) g_malloc(sizeof(fmt_data));
@@ -359,7 +314,7 @@ column_list_new_cb(GtkWidget *w _U_, gpointer data _U_) {
     cfmt->custom_field = NULL;
     prefs.col_list = g_list_append(prefs.col_list, cfmt);
 
-    model = gtk_tree_view_get_model(GTK_TREE_VIEW(column_l));
+    model = gtk_tree_view_get_model(column_l);
     gtk_list_store_append(GTK_LIST_STORE(model), &iter);
     gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, title, 1,
                        col_format_desc(cur_fmt), 2, g_list_last(prefs.col_list),
@@ -368,23 +323,27 @@ column_list_new_cb(GtkWidget *w _U_, gpointer data _U_) {
     path = gtk_tree_model_get_path(model, &iter);
     str_path = gtk_tree_path_to_string(path);
     cur_row = atoi(str_path);
+
+    gtk_tree_selection_select_iter(gtk_tree_view_get_selection(column_l),
+                                   &iter);
+    title_column = gtk_tree_view_get_column(column_l, 0);
+    gtk_tree_view_set_cursor(column_l, path, title_column, TRUE);
+
     g_free(str_path);
     gtk_tree_path_free(path);
-
-    gtk_tree_selection_select_iter(gtk_tree_view_get_selection(GTK_TREE_VIEW(column_l)),
-                                   &iter);
     cfile.cinfo.columns_changed = TRUE;
 }
 
 static void
-column_list_delete_cb(GtkWidget *w _U_, gpointer data _U_) {
+column_list_delete_cb(GtkWidget *w _U_, gpointer data) {
+    GtkTreeView      *column_l = GTK_TREE_VIEW(data);
     GList            *clp;
     fmt_data         *cfmt;
     GtkTreeSelection *sel;
     GtkTreeModel     *model;
     GtkTreeIter       iter;
 
-    sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(column_l));
+    sel = gtk_tree_view_get_selection(column_l);
     if (gtk_tree_selection_get_selected(sel, &model, &iter))
     {
         gtk_tree_model_get(model, &iter, 2, &clp, -1);
@@ -392,9 +351,7 @@ column_list_delete_cb(GtkWidget *w _U_, gpointer data _U_) {
         cfmt = (fmt_data *) clp->data;
         g_free(cfmt->title);
         g_free(cfmt->fmt);
-        if (cfmt->custom_field) {
-          g_free (cfmt->custom_field);
-        }
+        g_free(cfmt->custom_field);
         g_free(cfmt);
         prefs.col_list = g_list_remove_link(prefs.col_list, clp);
 
@@ -403,29 +360,28 @@ column_list_delete_cb(GtkWidget *w _U_, gpointer data _U_) {
     cfile.cinfo.columns_changed = TRUE;
 }
 
-/* The user changed the column title entry box. */
-static void
-column_entry_changed_cb(GtkEditable *te, gpointer data) {
-    fmt_data         *cfmt;
-    GList            *clp;
-    gchar            *title;
-    GtkTreeView      *tree = (GtkTreeView *)data;
-    GtkTreeSelection *sel;
-    GtkTreeModel     *model;
-    GtkTreeIter       iter;
+static gboolean
+column_title_changed_cb(GtkCellRendererText *cell _U_, const gchar *str_path, const gchar *new_title, gpointer data) {
+  fmt_data     *cfmt;
+  GList        *clp;
+  GtkTreeModel *model = (GtkTreeModel *)data;
+  GtkTreePath  *path = gtk_tree_path_new_from_string (str_path);
+  GtkTreeIter   iter;
 
-    sel = gtk_tree_view_get_selection(tree);
-    if (gtk_tree_selection_get_selected(sel, &model, &iter))
-    {
-        title = gtk_editable_get_chars(te, 0, -1);
-        gtk_tree_model_get(model, &iter, 2, &clp, -1);
-        cfmt  = (fmt_data *) clp->data;
+  gtk_tree_model_get_iter(model, &iter, path); 
+  
+  gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, new_title, -1);
 
-        gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, title, -1);
-        g_free(cfmt->title);
-        cfmt->title = title;
-    }
-    cfile.cinfo.columns_changed = TRUE;
+  gtk_tree_model_get(model, &iter, 2, &clp, -1);
+  if (clp) {    
+    cfmt  = (fmt_data *) clp->data;
+    g_free(cfmt->title);
+    cfmt->title = g_strdup(new_title);
+  }
+
+  gtk_tree_path_free (path);
+  cfile.cinfo.columns_changed = TRUE; 
+  return TRUE;  
 }
 
 /* The user changed the custom field entry box. */
@@ -448,9 +404,7 @@ column_field_changed_cb(GtkEditable *te, gpointer data) {
         fmt = g_strdup_printf("%s (%s)", col_format_desc(cur_fmt), field);
         gtk_list_store_set(GTK_LIST_STORE(model), &iter, 1, fmt, -1);
 	g_free(fmt);
-        if (cfmt->custom_field) {
-          g_free(cfmt->custom_field);
-        }
+        g_free(cfmt->custom_field);
         cfmt->custom_field = field;
     }
     cfile.cinfo.columns_changed = TRUE;
@@ -458,7 +412,8 @@ column_field_changed_cb(GtkEditable *te, gpointer data) {
 
 /* The user changed the format menu. */
 static void
-column_menu_changed_cb(GtkWidget *w, gpointer data _U_) {
+column_menu_changed_cb(GtkWidget *w, gpointer data) {
+    GtkTreeView      *column_l = GTK_TREE_VIEW(data);
     fmt_data         *cfmt;
     GList            *clp;
     const gchar      *fmt;
@@ -466,7 +421,7 @@ column_menu_changed_cb(GtkWidget *w, gpointer data _U_) {
     GtkTreeModel     *model;
     GtkTreeIter       iter;
 
-    sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(column_l));
+    sel = gtk_tree_view_get_selection(column_l);
     if (gtk_tree_selection_get_selected(sel, &model, &iter))
     {
         cur_fmt = gtk_combo_box_get_active(GTK_COMBO_BOX(w));
@@ -494,78 +449,42 @@ column_menu_changed_cb(GtkWidget *w, gpointer data _U_) {
     cfile.cinfo.columns_changed = TRUE;
 }
 
+/*
+ * Callback for the "row-deleted" signal emitted when a list item is dragged.
+ * http://library.gnome.org/devel/gtk/stable/GtkTreeModel.html#GtkTreeModel-rows-reordered
+ * says that DND deletes, THEN inserts the row. If this isn't the case, we'll
+ * have to find another way to do this (e.g. by rebuilding prefs.col_list by
+ * iterating over the tree model.
+ */
 static void
-column_arrow_cb(GtkWidget *w, gpointer data _U_) {
-    fmt_data         *cfmt;
-    GList            *clp1, *clp2;
-    GtkTreeSelection *sel;
-    GtkTreeModel     *model;
-    GtkTreeIter       iter1, iter2;
-    GtkTreePath      *path;
-    gchar            *title1, *format1, *title2, *format2;
+column_dnd_row_deleted_cb(GtkTreeModel *model, GtkTreePath *path _U_, gpointer data _U_) {
+  GtkTreeIter       iter;
+  /* gpointer          cfmt; */
+  GList            *clp, *new_col_list = NULL;
+  gchar            *title, *format;
+  gboolean     items_left;
 
-    sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(column_l));
-    if (gtk_tree_selection_get_selected(sel, &model, &iter1))
-    {
-        gtk_tree_model_get(model, &iter1, 0, &title1,
-                           1, &format1, 2, &clp1, -1);
-        cfmt = (fmt_data *)clp1->data;
-        prefs.col_list = g_list_remove(prefs.col_list, cfmt);
+  /*
+   * XXX - This rebuilds prefs.col_list based on the current model. We
+   * might just want to do this when the prefs are applied
+   */
+  for (items_left = gtk_tree_model_get_iter_first (model, &iter);
+       items_left;
+       items_left = gtk_tree_model_iter_next (model, &iter)) {
 
-        if (w == up_bt)
-        {
-            cur_row--;
-            prefs.col_list = g_list_insert(prefs.col_list, cfmt, cur_row);
-            path = gtk_tree_model_get_path(model, &iter1);
-            gtk_tree_path_prev(path);
-            if (!gtk_tree_model_get_iter(model, &iter2, path))
-            {
-                gtk_tree_path_free(path);
-                return;
-            }
-            gtk_tree_path_free(path);
-        }
-        else
-        {
-            cur_row++;
-            prefs.col_list = g_list_insert(prefs.col_list, cfmt, cur_row);
-            iter2 = iter1;
-            if (!gtk_tree_model_iter_next(model, &iter2))
-            {
-                return;
-            }
-        }
-        clp1 = g_list_find(prefs.col_list, cfmt);
-        gtk_tree_model_get(model, &iter2, 0, &title2, 1, &format2, 2,
-                           &clp2, -1);
-        gtk_list_store_set(GTK_LIST_STORE(model), &iter2, 0, title1, 1,
-                           format1, 2, clp1, -1);
-        gtk_list_store_set(GTK_LIST_STORE(model), &iter1, 0, title2, 1,
-                           format2, 2, clp2, -1);
-        gtk_tree_selection_select_iter(sel, &iter2);
-
-        column_set_arrow_button_sensitivity(clp1);
-
-        /* free strings read from the TreeModel */
-        g_free(title1);
-        g_free(format1);
-        g_free(title2);
-        g_free(format2);
+    gtk_tree_model_get(model, &iter, 0, &title, 1, &format, 2, &clp, -1);
+    if (clp) {
+      prefs.col_list = g_list_remove_link(prefs.col_list, clp);
+      new_col_list = g_list_concat(new_col_list, clp);
     }
-    cfile.cinfo.columns_changed = TRUE;
-}
-
-void
-column_set_arrow_button_sensitivity(GList *clp) {
-    gint up_sens = FALSE, dn_sens = FALSE;
-
-    if (clp != g_list_first(prefs.col_list))
-        up_sens = TRUE;
-    if (clp != g_list_last(prefs.col_list))
-        dn_sens = TRUE;
-
-    gtk_widget_set_sensitive(up_bt, up_sens);
-    gtk_widget_set_sensitive(dn_bt, dn_sens);
+  }
+  if (prefs.col_list) {
+    g_warning("column_dnd_row_deleted_cb: prefs.col_list has %d leftover data",
+      g_list_length(prefs.col_list));
+    g_list_free(prefs.col_list);
+  }
+  prefs.col_list = new_col_list;
+  cfile.cinfo.columns_changed = TRUE;
 }
 
 void
