@@ -1,7 +1,7 @@
 /* prefs_dlg.c
  * Routines for handling preferences
  *
- * $Id: prefs_dlg.c 25949 2008-08-07 12:31:43Z wmeier $
+ * $Id: prefs_dlg.c 27800 2009-03-19 17:49:11Z wmeier $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -171,15 +171,15 @@ pref_show(pref_t *pref, gpointer user_data)
     switch (pref->info.base) {
 
     case 10:
-      g_snprintf(uint_str, 10+1, "%u", pref->saved_val.uint);
+      g_snprintf(uint_str, sizeof(uint_str), "%u", pref->saved_val.uint);
       break;
 
     case 8:
-      g_snprintf(uint_str, 10+1, "%o", pref->saved_val.uint);
+      g_snprintf(uint_str, sizeof(uint_str), "%o", pref->saved_val.uint);
       break;
 
     case 16:
-      g_snprintf(uint_str, 10+1, "%x", pref->saved_val.uint);
+      g_snprintf(uint_str, sizeof(uint_str), "%x", pref->saved_val.uint);
       break;
     }
     pref->control = create_preference_entry(main_tb, pref->ordinal,
@@ -212,8 +212,7 @@ pref_show(pref_t *pref, gpointer user_data)
     break;
 
   case PREF_STRING:
-    if (pref->saved_val.string != NULL)
-      g_free(pref->saved_val.string);
+    g_free(pref->saved_val.string);
     pref->saved_val.string = g_strdup(*pref->varp.string);
     pref->control = create_preference_entry(main_tb, pref->ordinal,
 					    label_string, pref->description,
@@ -224,8 +223,7 @@ pref_show(pref_t *pref, gpointer user_data)
   {
     char *range_string;
 
-    if (pref->saved_val.range != NULL)
-      g_free(pref->saved_val.range);
+    g_free(pref->saved_val.range);
     pref->saved_val.range = range_copy(*pref->varp.range);
     range_string = range_convert_range(*pref->varp.range);
     pref->control = create_preference_entry(main_tb, pref->ordinal,
@@ -382,9 +380,7 @@ module_prefs_show(module_t *module, gpointer user_data)
 /* add a page to the tree */
 static prefs_tree_iter
 prefs_tree_page_add(const gchar *title, gint page_nr,
-                    gpointer store, prefs_tree_iter *parent_iter,
-                    gboolean has_child
-                    _U_)
+                    gpointer store, prefs_tree_iter *parent_iter)
 {
   prefs_tree_iter   iter;
 
@@ -507,26 +503,26 @@ prefs_cb(GtkWidget *w _U_, gpointer dummy _U_)
   /* GUI prefs */
   g_strlcpy(label_str, "User Interface", MAX_TREE_NODE_NAME_LEN);
   prefs_nb_page_add(prefs_nb, label_str, gui_prefs_show(), E_GUI_PAGE_KEY);
-  gui_iter = prefs_tree_page_add(label_str, cts.page, store, NULL, TRUE);
+  gui_iter = prefs_tree_page_add(label_str, cts.page, store, NULL);
   cts.page++;
 
   /* GUI layout prefs */
   g_strlcpy(label_str, "Layout", MAX_TREE_NODE_NAME_LEN);
   prefs_nb_page_add(prefs_nb, label_str, layout_prefs_show(), E_GUI_LAYOUT_PAGE_KEY);
-  prefs_tree_page_add(label_str, cts.page, store, &gui_iter, FALSE);
+  prefs_tree_page_add(label_str, cts.page, store, &gui_iter);
   cts.page++;
 
   /* GUI Column prefs */
   g_strlcpy(label_str, "Columns", MAX_TREE_NODE_NAME_LEN);
-  prefs_nb_page_add(prefs_nb, label_str, column_prefs_show(), E_GUI_COLUMN_PAGE_KEY);
-  prefs_tree_page_add(label_str, cts.page, store, &gui_iter, FALSE);
+  prefs_nb_page_add(prefs_nb, label_str, column_prefs_show(prefs_w), E_GUI_COLUMN_PAGE_KEY);
+  prefs_tree_page_add(label_str, cts.page, store, &gui_iter);
   cts.page++;
 
   /* GUI Font prefs */
   g_strlcpy(label_str, "Font", MAX_TREE_NODE_NAME_LEN);
   gui_font_pg = gui_font_prefs_show();
   prefs_nb_page_add(prefs_nb, label_str, gui_font_pg, E_GUI_FONT_PAGE_KEY);
-  prefs_tree_page_add(label_str, cts.page, store, &gui_iter, FALSE);
+  prefs_tree_page_add(label_str, cts.page, store, &gui_iter);
   cts.page++;
 
   gtk_container_set_border_width( GTK_CONTAINER(gui_font_pg), 5 );
@@ -544,7 +540,7 @@ prefs_cb(GtkWidget *w _U_, gpointer dummy _U_)
   /* GUI Colors prefs */
   g_strlcpy(label_str, "Colors", MAX_TREE_NODE_NAME_LEN);
   prefs_nb_page_add(prefs_nb, label_str, stream_prefs_show(), E_GUI_COLORS_PAGE_KEY);
-  prefs_tree_page_add(label_str, cts.page, store, &gui_iter, FALSE);
+  prefs_tree_page_add(label_str, cts.page, store, &gui_iter);
   cts.page++;
 
   /* select the main GUI page as the default page and expand it's children */
@@ -560,7 +556,7 @@ prefs_cb(GtkWidget *w _U_, gpointer dummy _U_)
   /* capture prefs */
   g_strlcpy(label_str, "Capture", MAX_TREE_NODE_NAME_LEN);
   prefs_nb_page_add(prefs_nb, label_str, capture_prefs_show(), E_CAPTURE_PAGE_KEY);
-  prefs_tree_page_add(label_str, cts.page, store, NULL, FALSE);
+  prefs_tree_page_add(label_str, cts.page, store, NULL);
   cts.page++;
 #ifdef _WIN32
   }
@@ -570,19 +566,19 @@ prefs_cb(GtkWidget *w _U_, gpointer dummy _U_)
   /* Printing prefs */
   g_strlcpy(label_str, "Printing", MAX_TREE_NODE_NAME_LEN);
   prefs_nb_page_add(prefs_nb, label_str, printer_prefs_show(), E_PRINT_PAGE_KEY);
-  prefs_tree_page_add(label_str, cts.page, store, NULL, FALSE);
+  prefs_tree_page_add(label_str, cts.page, store, NULL);
   cts.page++;
 
   /* Name resolution prefs */
   g_strlcpy(label_str, "Name Resolution", MAX_TREE_NODE_NAME_LEN);
   prefs_nb_page_add(prefs_nb, label_str, nameres_prefs_show(), E_NAMERES_PAGE_KEY);
-  prefs_tree_page_add(label_str, cts.page, store, NULL, FALSE);
+  prefs_tree_page_add(label_str, cts.page, store, NULL);
   cts.page++;
 
   /* TAPS player prefs */
   g_strlcpy(label_str, "Statistics", MAX_TREE_NODE_NAME_LEN);
   prefs_nb_page_add(prefs_nb, label_str, stats_prefs_show(), E_TAPS_PAGE_KEY);
-  prefs_tree_page_add(label_str, cts.page, store, NULL, FALSE);
+  prefs_tree_page_add(label_str, cts.page, store, NULL);
   cts.page++;
 
   /* Registered prefs */
@@ -641,6 +637,7 @@ set_option_label(GtkWidget *main_tb, int table_position,
 	gtk_widget_show(label);
 
 	event_box = gtk_event_box_new();
+	gtk_event_box_set_visible_window (GTK_EVENT_BOX(event_box), FALSE);
 	gtk_table_attach_defaults(GTK_TABLE(main_tb), event_box, 0, 1,
 	    table_position, table_position + 1);
 	if (tooltip_text != NULL && tooltips != NULL)
@@ -706,6 +703,7 @@ create_preference_radio_buttons(GtkWidget *main_tb, int table_position,
 	gtk_widget_show(radio_button_hbox);
 
 	event_box = gtk_event_box_new();
+	gtk_event_box_set_visible_window (GTK_EVENT_BOX(event_box), FALSE);
 	gtk_container_add(GTK_CONTAINER(event_box), radio_button_hbox);
 	gtk_table_attach_defaults(GTK_TABLE(main_tb), event_box, 1, 2,
 	    table_position, table_position+1);
@@ -814,6 +812,7 @@ create_preference_option_menu(GtkWidget *main_tb, int table_position,
 	gtk_box_pack_start(GTK_BOX(menu_box), option_menu, FALSE, FALSE, 0);
 
 	event_box = gtk_event_box_new();
+	gtk_event_box_set_visible_window (GTK_EVENT_BOX(event_box), FALSE);
 	gtk_table_attach_defaults(GTK_TABLE(main_tb), event_box,
 	    1, 2, table_position, table_position + 1);
 	if (tooltip_text != NULL && tooltips != NULL)
@@ -1339,7 +1338,7 @@ prefs_main_destroy_all(GtkWidget *dlg)
 }
 
 
-static void
+void
 prefs_main_write(void)
 {
   int err;
