@@ -1,7 +1,7 @@
 /* gui_prefs.c
  * Dialog box for GUI preferences
  *
- * $Id: prefs_gui.c 27070 2008-12-20 01:39:16Z wmeier $
+ * $Id: prefs_gui.c 27800 2009-03-19 17:49:11Z wmeier $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -270,7 +270,7 @@ gui_prefs_show(void)
 	/* File Open dialog preview timeout */
 	fileopen_preview_te = create_preference_entry(main_tb, pos++,
 	    "\"File Open\" preview timeout:", "Timeout, until preview gives up scanning the capture file content.", open_file_preview_str);
-	g_snprintf(current_val_str, 128, "%d", prefs.gui_fileopen_preview);
+	g_snprintf(current_val_str, sizeof(current_val_str), "%d", prefs.gui_fileopen_preview);
 	gtk_entry_set_text(GTK_ENTRY(fileopen_preview_te), current_val_str);
 	gtk_tooltips_set_tip(tooltips, fileopen_preview_te, 
         "Reading preview data in the \"File Open\" dialog will be stopped after given seconds.", NULL);
@@ -280,7 +280,7 @@ gui_prefs_show(void)
 	/* Number of recent entries in the display filter list ... */
 	recent_df_entries_max_te = create_preference_entry(main_tb, pos++,
 	    "Filter display max. list entries:", "Maximum number of recent entries in filter display list", recent_df_entries_max_str);
-	g_snprintf(current_val_str, 128, "%d", prefs.gui_recent_df_entries_max);
+	g_snprintf(current_val_str, sizeof(current_val_str), "%d", prefs.gui_recent_df_entries_max);
 	gtk_entry_set_text(GTK_ENTRY(recent_df_entries_max_te), current_val_str);
 	g_object_set_data(G_OBJECT(main_vb), GUI_RECENT_DF_ENTRIES_KEY, recent_df_entries_max_te);
 	g_signal_connect(recent_df_entries_max_te, "focus_out_event", G_CALLBACK(recent_df_entries_changed_cb), main_vb);
@@ -288,7 +288,7 @@ gui_prefs_show(void)
 	/* Number of entries in the recent_files list ... */
 	recent_files_count_max_te = create_preference_entry(main_tb, pos++,
 	    "\"Open Recent\" max. list entries:", "Maximum number of recent files", recent_files_count_max_str);
-	g_snprintf(current_val_str, 128, "%d", prefs.gui_recent_files_count_max);
+	g_snprintf(current_val_str, sizeof(current_val_str), "%d", prefs.gui_recent_files_count_max);
 	gtk_entry_set_text(GTK_ENTRY(recent_files_count_max_te), current_val_str);
 	gtk_tooltips_set_tip(tooltips, recent_files_count_max_te, 
         "Maximum number of entries in the \"File/Open Recent\" list.", NULL);
@@ -415,8 +415,7 @@ gui_prefs_fetch(GtkWidget *w)
 	prefs.gui_fileopen_style = fetch_preference_radio_buttons_val(
 	    g_object_get_data(G_OBJECT(w), GUI_FILEOPEN_KEY), gui_fileopen_vals);
 	
-	if (prefs.gui_fileopen_dir != NULL)
-		g_free(prefs.gui_fileopen_dir);
+	g_free(prefs.gui_fileopen_dir);
 	prefs.gui_fileopen_dir = g_strdup(gtk_entry_get_text(
                                               GTK_ENTRY(g_object_get_data(G_OBJECT(w), GUI_FILEOPEN_DIR_KEY))));
 
@@ -446,8 +445,7 @@ gui_prefs_fetch(GtkWidget *w)
 	if (font_fetch()) {
 		if (strcmp(new_font_name, prefs.gui_font_name) != 0) {
 			font_changed = TRUE;
-			if (prefs.gui_font_name != NULL)
-				g_free(prefs.gui_font_name);
+			g_free(prefs.gui_font_name);
 			prefs.gui_font_name = g_strdup(new_font_name);
 		}
 	}
