@@ -1,7 +1,7 @@
 /* print_dlg.c
  * Dialog boxes for printing and exporting to text files
  *
- * $Id: print_dlg.c 25634 2008-06-29 15:51:43Z wmeier $
+ * $Id: print_dlg.c 27686 2009-03-09 23:38:44Z wmeier $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -38,7 +38,6 @@
 #include "../print.h"
 #include "../alert_box.h"
 #include "../simple_dialog.h"
-#include "../tempfile.h"
 #include "../util.h"
 #include <wsutil/file_util.h>
 
@@ -58,6 +57,7 @@
 #include <windows.h>
 #include "gtk/file_dlg_win32.h"
 #include "gtk/print_win32.h"
+#include "../tempfile.h"
 #endif
 
 /* dialog output action */
@@ -908,7 +908,7 @@ print_ok_cb(GtkWidget *ok_bt, gpointer parent_w)
 #ifdef _WIN32
   gboolean          win_printer = FALSE;
   int               tmp_fd;
-  char              tmp_namebuf[128+1];    /* XXX - length was used elsewhere too, why? */
+  char              tmp_namebuf[128+1];  /* XX: see create_tmpfile which says [128+1]; why ? */
   char              *tmp_oldfile;
 #endif
   cf_print_status_t status;
@@ -945,13 +945,13 @@ print_ok_cb(GtkWidget *ok_bt, gpointer parent_w)
 #ifdef _WIN32
     win_printer = TRUE;
     /* We currently don't have a function in util.h to create just a tempfile */
-    /* name, so simply create a tempfile using the "official" function, */
-    /* then delete this file again. After this, the name MUST be available. */
+    /* name, so simply create a tempfile using the "official" function,       */
+    /* then delete this file again. After this, the name MUST be available.   */
     /* */
-    /* Don't use tmpnam() or such, as this will fail under some ACL */
-    /* circumstances: http://bugs.wireshark.org/bugzilla/show_bug.cgi?id=358 */
-
-    tmp_fd = create_tempfile(tmp_namebuf, sizeof(tmp_namebuf), "ethprint");
+    /* Don't use tmpnam() or such, as this will fail under some ACL           */
+    /* circumstances: http://bugs.wireshark.org/bugzilla/show_bug.cgi?id=358  */
+    /* Also: tmpnam is "insecure" and should not be used.                     */
+    tmp_fd = create_tempfile(tmp_namebuf, sizeof(tmp_namebuf), "wshprint");
     if(tmp_fd == -1) {
         simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
             "Couldn't create a temporary file for printing.");
@@ -1095,6 +1095,7 @@ print_ok_cb(GtkWidget *ok_bt, gpointer parent_w)
 
     /* trash temp file */
     ws_remove(args->file);
+    g_free(args->file);
 
     /* restore old settings */
     args->to_file = FALSE;
