@@ -1,7 +1,7 @@
 /* rtp_stream_dlg.c
  * RTP streams summary addition for Wireshark
  *
- * $Id: rtp_stream_dlg.c 25673 2008-07-08 02:43:40Z wmeier $
+ * $Id: rtp_stream_dlg.c 27800 2009-03-19 17:49:11Z wmeier $
  *
  * Copyright 2003, Alcatel Business Systems
  * By Lars Ruoff <lars.ruoff@gmx.net>
@@ -141,10 +141,10 @@ static void add_to_clist(rtp_stream_info_t* strinfo)
 	gtk_clist_set_row_data(GTK_CLIST(clist), added_row, strinfo);
 
 	/* Update the top label with the number of detected streams */
-	g_snprintf(label_text, 256,
+	g_snprintf(label_text, sizeof(label_text),
 	        "Detected %d RTP streams. Choose one for forward and reverse direction for analysis",
 	        ++streams_nb);
-	gtk_label_set(GTK_LABEL(top_label), label_text);
+	 gtk_label_set_text(GTK_LABEL(top_label), label_text);
 }
 
 /****************************************************************************/
@@ -156,14 +156,14 @@ static void save_stream_destroy_cb(GtkWidget *win _U_, gpointer user_data _U_)
 
 /****************************************************************************/
 /* save in a file */
-static void save_stream_ok_cb(GtkWidget *ok_bt _U_, gpointer user_data _U_)
+static void save_stream_ok_cb(GtkWidget *ok_bt _U_, gpointer fs _U_)
 {
 	gchar *g_dest;
 
 	if (!selected_stream_fwd)
 		return;
 
-	g_dest = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION (rtpstream_save_dlg)));
+	g_dest = g_strdup(gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(fs)));
 
 	/* Perhaps the user specified a directory instead of a file.
 	Check whether they did. */
@@ -306,10 +306,9 @@ static void
 rtpstream_on_save                      (GtkButton       *button _U_,
                                         gpointer         data _U_)
 {
+/* XX - not neded?
 	rtpstream_tapinfo_t* tapinfo = data;
-
-	GtkWidget *vertb;
-	GtkWidget *ok_bt;
+*/
 
 	if (!selected_stream_fwd)
 		return;
@@ -320,21 +319,10 @@ rtpstream_on_save                      (GtkButton       *button _U_,
 		return;
 	}
 
-	/* XXX - use file_selection from dlg_utils instead! */
-	rtpstream_save_dlg = gtk_file_selection_new("Wireshark: Save selected stream in rtpdump ('-F dump') format");
-
-	/* Container for each row of widgets */
-	vertb = gtk_vbox_new(FALSE, 0);
-	gtk_container_set_border_width(GTK_CONTAINER(vertb), 5);
-	gtk_box_pack_start(GTK_BOX(GTK_FILE_SELECTION(rtpstream_save_dlg)->action_area),
-		vertb, FALSE, FALSE, 0);
-	gtk_widget_show (vertb);
-
-	ok_bt = GTK_FILE_SELECTION(rtpstream_save_dlg)->ok_button;
-	g_signal_connect(ok_bt, "clicked", G_CALLBACK(save_stream_ok_cb), tapinfo);
-
-	window_set_cancel_button(rtpstream_save_dlg,
-	    GTK_FILE_SELECTION(rtpstream_save_dlg)->cancel_button, window_cancel_button_cb);
+	rtpstream_save_dlg = gtk_file_chooser_dialog_new("Wireshark: Save selected stream in rtpdump ('-F dump') format", GTK_WINDOW(rtp_stream_dlg), GTK_FILE_CHOOSER_ACTION_SAVE,
+		GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
+		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+		NULL);
 
 	g_signal_connect(rtpstream_save_dlg, "delete_event", G_CALLBACK(window_delete_event_cb), NULL);
 	g_signal_connect(rtpstream_save_dlg, "destroy", G_CALLBACK(save_stream_destroy_cb),
@@ -342,6 +330,13 @@ rtpstream_on_save                      (GtkButton       *button _U_,
 
 	gtk_widget_show(rtpstream_save_dlg);
 	window_present(rtpstream_save_dlg);
+
+	if (gtk_dialog_run(GTK_DIALOG(rtpstream_save_dlg)) == GTK_RESPONSE_ACCEPT){
+		save_stream_ok_cb(rtpstream_save_dlg, rtpstream_save_dlg);
+	}else{
+		window_destroy(rtpstream_save_dlg);
+	}
+
 }
 
 
@@ -372,7 +367,7 @@ rtpstream_on_filter                    (GtkButton       *button _U_,
 	if (selected_stream_fwd)
 	{
 		if (selected_stream_fwd->src_addr.type==AT_IPv6){
-			g_strlcpy(ip_version,"v6",3);
+			g_strlcpy(ip_version,"v6",sizeof(ip_version));
 		}		
 		else{
 			ip_version[0] = '\0';
@@ -392,7 +387,7 @@ rtpstream_on_filter                    (GtkButton       *button _U_,
 	if (selected_stream_rev)
 	{
 		if (selected_stream_fwd->src_addr.type==AT_IPv6){
-			g_strlcpy(ip_version,"v6",3);
+			g_strlcpy(ip_version,"v6",sizeof(ip_version));
 		}		
 		else{
 			ip_version[0] = '\0';
@@ -532,7 +527,7 @@ rtpstream_on_select_row(GtkCList *clist,
 	/* update the labels */
 	if (event==NULL || event->state & GDK_SHIFT_MASK) {
 		selected_stream_rev = gtk_clist_get_row_data(GTK_CLIST(clist), row);
-		g_snprintf(label_text, 80, "Reverse: %s:%u -> %s:%u, SSRC=0x%X",
+		g_snprintf(label_text, sizeof(label_text), "Reverse: %s:%u -> %s:%u, SSRC=0x%X",
 			get_addr_name(&(selected_stream_rev->src_addr)),
 			selected_stream_rev->src_port,
 			get_addr_name(&(selected_stream_rev->dest_addr)),
@@ -543,7 +538,7 @@ rtpstream_on_select_row(GtkCList *clist,
 	}
 	else {
 		selected_stream_fwd = gtk_clist_get_row_data(GTK_CLIST(clist), row);
-		g_snprintf(label_text, 80, "Forward: %s:%u -> %s:%u, SSRC=0x%X",
+		g_snprintf(label_text, sizeof(label_text), "Forward: %s:%u -> %s:%u, SSRC=0x%X",
 			get_addr_name(&(selected_stream_fwd->src_addr)),
 			selected_stream_fwd->src_port,
 			get_addr_name(&(selected_stream_fwd->dest_addr)),
@@ -779,7 +774,7 @@ static void rtpstream_dlg_create (void)
     bt_copy = gtk_button_new_from_stock(GTK_STOCK_COPY);
     gtk_container_add (GTK_CONTAINER (hbuttonbox), bt_copy);
     gtk_tooltips_set_tip(tooltips, bt_copy, 
-        "Copy all statistical values of this page to the clipboard in CSV (Comma Seperated Values) format.", NULL);
+        "Copy all statistical values of this page to the clipboard in CSV (Comma Separated Values) format.", NULL);
 
     bt_analyze = gtk_button_new_with_label ("Analyze");
     gtk_container_add (GTK_CONTAINER (hbuttonbox), bt_analyze);
