@@ -1,7 +1,7 @@
 /*
  * Copyright 2004, Irene Ruengeler <i.ruengeler [AT] fh-muenster.de>
  *
- * $Id: sctp_graph_dlg.c 24513 2008-03-01 10:23:57Z tuexen $
+ * $Id: sctp_graph_dlg.c 27800 2009-03-19 17:49:11Z wmeier $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -25,25 +25,26 @@
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
-
-
-#include <gtk/gtk.h>
-#include <gdk/gdk.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
 #include <string.h>
-#include "globals.h"
+
+#include <gtk/gtk.h>
+
 #include "epan/filesystem.h"
-#include "../color.h"
-#include "dlg_utils.h"
-#include "ui_util.h"
-#include "main.h"
-#include "compat_macros.h"
-#include "simple_dialog.h"
-#include "sctp_stat.h"
 #include <epan/strutil.h>
 
+#include "../globals.h"
+#include "../color.h"
+#include "../ui_util.h"
+#include "../simple_dialog.h"
+
+#include "gtk/dlg_utils.h"
+#include "gtk/main.h"
+#include "gtk/sctp_stat.h"
+
+
 #define DEFAULT_PIXELS_PER_TICK 2
 #define MAX_PIXELS_PER_TICK 4
 #define AUTO_MAX_YSCALE 0
@@ -131,35 +132,12 @@ static void draw_sack_graph(struct sctp_udata *u_data)
 	struct sack_chunk_header *sack_header;
 	struct gaps *gap;
 	guint32 max_num, diff;
-#if GTK_MAJOR_VERSION < 2
-	GdkColormap *colormap;
-#endif
 
 	red_gc = gdk_gc_new(u_data->io->draw_area->window);
-#if GTK_MAJOR_VERSION < 2
-		colormap = gtk_widget_get_colormap (u_data->io->draw_area);
-		if (!gdk_color_alloc (colormap, &red_color))
-		{
-			g_warning ("Couldn't allocate color");
-		}
-
-		gdk_gc_set_foreground(red_gc, &red_color);
-#else
-		gdk_gc_set_rgb_fg_color(red_gc, &red_color);
-#endif
+	gdk_gc_set_rgb_fg_color(red_gc, &red_color);
 
 	green_gc = gdk_gc_new(u_data->io->draw_area->window);
-#if GTK_MAJOR_VERSION < 2
-		colormap = gtk_widget_get_colormap (u_data->io->draw_area);
-		if (!gdk_color_alloc (colormap, &green_color))
-		{
-			g_warning ("Couldn't allocate color");
-		}
-
-		gdk_gc_set_foreground(green_gc, &green_color);
-#else
-		gdk_gc_set_rgb_fg_color(green_gc, &green_color);
-#endif
+	gdk_gc_set_rgb_fg_color(green_gc, &green_color);
 
 	if (u_data->dir==2)
 	{
@@ -261,10 +239,8 @@ static void draw_sack_graph(struct sctp_udata *u_data)
 		}
 		list = g_list_previous(list);
 	}
-#if GTK_MAJOR_VERSION >= 2
-		g_object_unref(G_OBJECT(red_gc));
-		g_object_unref(G_OBJECT(green_gc));
-#endif
+	g_object_unref(G_OBJECT(red_gc));
+	g_object_unref(G_OBJECT(green_gc));
 }
 
 
@@ -347,12 +323,7 @@ static void sctp_graph_draw(struct sctp_udata *u_data)
 	char label_string[15];
 	gfloat dis;
 	gboolean write_label = FALSE;
-
-#if GTK_MAJOR_VERSION < 2
-	GdkFont *font;
-#else
 	PangoLayout  *layout;
-#endif
 
 	if (u_data->io->x1_tmp_sec==0 && u_data->io->x1_tmp_usec==0)
 		u_data->io->offset=0;
@@ -444,21 +415,11 @@ static void sctp_graph_draw(struct sctp_udata *u_data)
 	else
 		distance=5;
 
-#if GTK_MAJOR_VERSION < 2
-	font = u_data->io->draw_area->style->font;
-#endif
-
-#if GTK_MAJOR_VERSION < 2
-	label_width=gdk_string_width(font, label_string);
-	label_height=gdk_string_height(font, label_string);
-#else
-	g_snprintf(label_string, 15, "%d", 0);
-	memcpy(label_string,(gchar *)g_locale_to_utf8(label_string, -1 , NULL, NULL, NULL), 15);
+	g_snprintf(label_string, sizeof(label_string), "%d", 0);
+	memcpy(label_string,(gchar *)g_locale_to_utf8(label_string, -1 , NULL, NULL, NULL), sizeof(label_string));
 	layout = gtk_widget_create_pango_layout(u_data->io->draw_area, label_string);
 	pango_layout_get_pixel_size(layout, &label_width, &label_height);
 
-#endif
-
 	if (u_data->io->x1_tmp_usec==0)
 		sec=u_data->io->x1_tmp_sec;
 	else
@@ -467,16 +428,9 @@ static void sctp_graph_draw(struct sctp_udata *u_data)
 
 	if (u_data->io->offset!=0)
 	{
-		g_snprintf(label_string, 15, "%u", u_data->io->x1_tmp_sec);
+		g_snprintf(label_string, sizeof(label_string), "%u", u_data->io->x1_tmp_sec);
 
-#if GTK_MAJOR_VERSION < 2
-		lwidth=gdk_string_width(font, label_string);
-		gdk_draw_string(u_data->io->pixmap,font,u_data->io->draw_area->style->black_gc,
-		                LEFT_BORDER-25,
-		                u_data->io->pixmap_height-BOTTOM_BORDER+20,
-		                label_string);
-#else
-		memcpy(label_string,(gchar *)g_locale_to_utf8(label_string, -1 , NULL, NULL, NULL), 15);
+		memcpy(label_string,(gchar *)g_locale_to_utf8(label_string, -1 , NULL, NULL, NULL), sizeof(label_string));
 		pango_layout_set_text(layout, label_string, -1);
 		pango_layout_get_pixel_size(layout, &lwidth, NULL);
 
@@ -484,7 +438,6 @@ static void sctp_graph_draw(struct sctp_udata *u_data)
 		                LEFT_BORDER-25,
 		                u_data->io->pixmap_height-BOTTOM_BORDER+20,
 		                layout);
-#endif
 	}
 
 	w=(guint32)(500/(guint32)(distance*u_data->io->x_interval)); /*there will be a label for every w_th tic*/
@@ -537,26 +490,17 @@ static void sctp_graph_draw(struct sctp_udata *u_data)
 		if (i>=u_data->io->min_x && i%1000000!=0)
 		{
 			length=5;
-			g_snprintf(label_string, 15, "%d", i%1000000);
+			g_snprintf(label_string, sizeof(label_string), "%d", i%1000000);
 			if (j%w==0)
 			{
 				length=10;
-
-				#if GTK_MAJOR_VERSION < 2
-					lwidth=gdk_string_width(font, label_string);
-					gdk_draw_string(u_data->io->pixmap,font,u_data->io->draw_area->style->black_gc,
-						(guint32)(LEFT_BORDER+u_data->io->offset+(i-u_data->io->min_x)*u_data->io->x_interval-lwidth/2),
-						u_data->io->pixmap_height-BOTTOM_BORDER+10,
-						label_string);
-				#else
-					memcpy(label_string,(gchar *)g_locale_to_utf8(label_string, -1 , NULL, NULL, NULL), 15);
-					pango_layout_set_text(layout, label_string, -1);
-					pango_layout_get_pixel_size(layout, &lwidth, NULL);
-					gdk_draw_layout(u_data->io->pixmap,u_data->io->draw_area->style->black_gc,
-						(guint32)(LEFT_BORDER+u_data->io->offset+(i-u_data->io->min_x)*u_data->io->x_interval-lwidth/2),
-						u_data->io->pixmap_height-BOTTOM_BORDER+10,
-						layout);
-				#endif
+				memcpy(label_string,(gchar *)g_locale_to_utf8(label_string, -1 , NULL, NULL, NULL), sizeof(label_string));
+				pango_layout_set_text(layout, label_string, -1);
+				pango_layout_get_pixel_size(layout, &lwidth, NULL);
+				gdk_draw_layout(u_data->io->pixmap,u_data->io->draw_area->style->black_gc,
+					(guint32)(LEFT_BORDER+u_data->io->offset+(i-u_data->io->min_x)*u_data->io->x_interval-lwidth/2),
+					u_data->io->pixmap_height-BOTTOM_BORDER+10,
+					layout);
 			}
 			gdk_draw_line(u_data->io->pixmap,u_data->io->draw_area->style->black_gc,
 				(guint32)(LEFT_BORDER+u_data->io->offset+(i-u_data->io->min_x)*u_data->io->x_interval),
@@ -584,45 +528,28 @@ static void sctp_graph_draw(struct sctp_udata *u_data)
 		if (write_label)
 		{
 			gdk_draw_line(u_data->io->pixmap,u_data->io->draw_area->style->black_gc,
-			(guint32)(LEFT_BORDER+u_data->io->offset+(i-u_data->io->min_x)*u_data->io->x_interval),
-			u_data->io->pixmap_height-BOTTOM_BORDER,
-			(guint32)(LEFT_BORDER+u_data->io->offset+(i-u_data->io->min_x)*u_data->io->x_interval),
-			u_data->io->pixmap_height-BOTTOM_BORDER+10);
+			              (guint32)(LEFT_BORDER+u_data->io->offset+(i-u_data->io->min_x)*u_data->io->x_interval),
+			              u_data->io->pixmap_height-BOTTOM_BORDER,
+			              (guint32)(LEFT_BORDER+u_data->io->offset+(i-u_data->io->min_x)*u_data->io->x_interval),
+			              u_data->io->pixmap_height-BOTTOM_BORDER+10);
 
-			g_snprintf(label_string, 15, "%d", sec);
-			#if GTK_MAJOR_VERSION < 2
-				lwidth=gdk_string_width(font, label_string);
-				gdk_draw_string(u_data->io->pixmap,font,u_data->io->draw_area->style->black_gc,
-					(guint32)(LEFT_BORDER+u_data->io->offset+(i-u_data->io->min_x)*u_data->io->x_interval-10),
-					u_data->io->pixmap_height-BOTTOM_BORDER+20,
-					label_string);
-			#else
-				memcpy(label_string,(gchar *)g_locale_to_utf8(label_string, -1 , NULL, NULL, NULL), 15);
-				pango_layout_set_text(layout, label_string, -1);
-				pango_layout_get_pixel_size(layout, &lwidth, NULL);
+			g_snprintf(label_string, sizeof(label_string), "%d", sec);
+			memcpy(label_string,(gchar *)g_locale_to_utf8(label_string, -1 , NULL, NULL, NULL), sizeof(label_string));
+			pango_layout_set_text(layout, label_string, -1);
+			pango_layout_get_pixel_size(layout, &lwidth, NULL);
 
-				gdk_draw_layout(u_data->io->pixmap,u_data->io->draw_area->style->black_gc,
-				(guint32)(LEFT_BORDER+u_data->io->offset+(i-u_data->io->min_x)*u_data->io->x_interval-10),
-					u_data->io->pixmap_height-BOTTOM_BORDER+20,
-					layout);
-			#endif
+			gdk_draw_layout(u_data->io->pixmap,u_data->io->draw_area->style->black_gc,
+			                (guint32)(LEFT_BORDER+u_data->io->offset+(i-u_data->io->min_x)*u_data->io->x_interval-10),
+			                u_data->io->pixmap_height-BOTTOM_BORDER+20,
+			                layout);
 			write_label = FALSE;
 		}
 		
 	}
 
-	g_strlcpy(label_string, "sec", 15);
+	g_strlcpy(label_string, "sec", sizeof(label_string));
 
-#if GTK_MAJOR_VERSION < 2
-	lwidth=gdk_string_width(font, label_string);
-	gdk_draw_string(u_data->io->pixmap,
-	                font,
-	                u_data->io->draw_area->style->black_gc,
-	                u_data->io->pixmap_width-RIGHT_BORDER-10,
-	                u_data->io->pixmap_height-BOTTOM_BORDER+30,
-	                label_string);
-#else
-	memcpy(label_string,(gchar *)g_locale_to_utf8(label_string, -1 , NULL, NULL, NULL), 15);
+	memcpy(label_string,(gchar *)g_locale_to_utf8(label_string, -1 , NULL, NULL, NULL), sizeof(label_string));
 	pango_layout_set_text(layout, label_string, -1);
 	pango_layout_get_pixel_size(layout, &lwidth, NULL);
 	gdk_draw_layout(u_data->io->pixmap,
@@ -630,7 +557,6 @@ static void sctp_graph_draw(struct sctp_udata *u_data)
 	                u_data->io->pixmap_width-RIGHT_BORDER-10,
 	                u_data->io->pixmap_height-BOTTOM_BORDER+30,
 	                layout);
-#endif
 
 	distance=5;
 
@@ -664,26 +590,18 @@ static void sctp_graph_draw(struct sctp_udata *u_data)
 			if (i>=u_data->io->min_y)
 			{
 				length=5;
-				g_snprintf(label_string, 15, "%d", i);
+				g_snprintf(label_string, sizeof(label_string), "%d", i);
 				if (i%distance==0 || (distance<=5 && u_data->io->y_interval>10))
 				{
 					length=10;
 
-#if GTK_MAJOR_VERSION < 2
-						lwidth=gdk_string_width(font, label_string);
-						gdk_draw_string(u_data->io->pixmap,font,u_data->io->draw_area->style->black_gc,
-						                LEFT_BORDER-length-lwidth-5,
-						                (guint32)(u_data->io->pixmap_height-BOTTOM_BORDER-u_data->io->offset-(i-u_data->io->min_y)*u_data->io->y_interval-POINT_SIZE),
-						                label_string);
-#else
-						memcpy(label_string,(gchar *)g_locale_to_utf8(label_string, -1 , NULL, NULL, NULL), 15);
-						pango_layout_set_text(layout, label_string, -1);
-						pango_layout_get_pixel_size(layout, &lwidth, NULL);
-						gdk_draw_layout(u_data->io->pixmap,u_data->io->draw_area->style->black_gc,
-						                LEFT_BORDER-length-lwidth-5,
-						                (guint32)(u_data->io->pixmap_height-BOTTOM_BORDER-u_data->io->offset-(i-u_data->io->min_y)*u_data->io->y_interval-POINT_SIZE),
-						                layout);
-#endif
+					memcpy(label_string,(gchar *)g_locale_to_utf8(label_string, -1 , NULL, NULL, NULL), sizeof(label_string));
+					pango_layout_set_text(layout, label_string, -1);
+					pango_layout_get_pixel_size(layout, &lwidth, NULL);
+					gdk_draw_layout(u_data->io->pixmap,u_data->io->draw_area->style->black_gc,
+					                LEFT_BORDER-length-lwidth-5,
+					                (guint32)(u_data->io->pixmap_height-BOTTOM_BORDER-u_data->io->offset-(i-u_data->io->min_y)*u_data->io->y_interval-POINT_SIZE),
+					                layout);
 				}
 				gdk_draw_line(u_data->io->pixmap,u_data->io->draw_area->style->black_gc,LEFT_BORDER-length,
 				              (guint32)(u_data->io->pixmap_height-BOTTOM_BORDER-u_data->io->offset-(i-u_data->io->min_y)*u_data->io->y_interval),
@@ -717,7 +635,7 @@ sctp_graph_t *ios;
 			draw_sack_graph(u_data);
 			break;
 	}
-	ios=(sctp_graph_t *)OBJECT_GET_DATA(u_data->io->draw_area, "sctp_graph_t");
+	ios=(sctp_graph_t *)g_object_get_data(G_OBJECT(u_data->io->draw_area), "sctp_graph_t");
 
 	if(!ios){
 		exit(10);
@@ -803,7 +721,7 @@ expose_event(GtkWidget *widget, GdkEventExpose *event)
 {
 	sctp_graph_t *ios;
 
-	ios=(sctp_graph_t *)OBJECT_GET_DATA(widget, "sctp_graph_t");
+	ios=(sctp_graph_t *)g_object_get_data(G_OBJECT(widget), "sctp_graph_t");
 	if(!ios){
 		exit(10);
 	}
@@ -954,7 +872,7 @@ on_button_press (GtkWidget *widget _U_, GdkEventButton *event, struct sctp_udata
 		                   (gint)floor(MIN(u_data->io->y_old,u_data->io->y_new)),
 		                   (gint)floor(abs((long)(u_data->io->x_new-u_data->io->x_old))),
 		                   (gint)floor(abs((long)(u_data->io->y_new-u_data->io->y_old))));
-		ios=(sctp_graph_t *)OBJECT_GET_DATA(u_data->io->draw_area, "sctp_graph_t");
+		ios=(sctp_graph_t *)g_object_get_data(G_OBJECT(u_data->io->draw_area), "sctp_graph_t");
 
 		if(!ios){
 			exit(10);
@@ -994,27 +912,12 @@ on_button_release (GtkWidget *widget _U_, GdkEventButton *event, struct sctp_uda
 	GPtrArray *tsnlist = NULL, *sacklist=NULL;
 	struct tsn_sort *tsn, *sack=NULL;
 	gboolean sack_found = FALSE;
+	PangoLayout  *layout;
 
-	#if GTK_MAJOR_VERSION < 2
-		GdkFont *font;
-#else
-		PangoLayout  *layout;
-#endif
-
-#if GTK_MAJOR_VERSION < 2
-		font = u_data->io->draw_area->style->font;
-#endif
-
-#if GTK_MAJOR_VERSION < 2
-		label_width=gdk_string_width(font, label_string);
-		label_height=gdk_string_height(font, label_string);
-#else
-		g_snprintf(label_string, 15, "%d", 0);
-		memcpy(label_string,(gchar *)g_locale_to_utf8(label_string, -1 , NULL, NULL, NULL), 15);
-		layout = gtk_widget_create_pango_layout(u_data->io->draw_area, label_string);
-		pango_layout_get_pixel_size(layout, &label_width, &label_height);
-
-#endif
+	g_snprintf(label_string, 15, "%d", 0);
+	memcpy(label_string,(gchar *)g_locale_to_utf8(label_string, -1 , NULL, NULL, NULL), 15);
+	layout = gtk_widget_create_pango_layout(u_data->io->draw_area, label_string);
+	pango_layout_get_pixel_size(layout, &label_width, &label_height);
 
 	if (event->y>u_data->io->pixmap_height-BOTTOM_BORDER-u_data->io->offset)
 		event->y = u_data->io->pixmap_height-BOTTOM_BORDER-u_data->io->offset;
@@ -1031,7 +934,7 @@ on_button_release (GtkWidget *widget _U_, GdkEventButton *event, struct sctp_uda
 		                   u_data->io->rect_x_min, u_data->io->rect_y_min,
 		                   u_data->io->rect_x_max - u_data->io->rect_x_min,
 				   u_data->io->rect_y_max - u_data->io->rect_y_min);
-		ios=(sctp_graph_t *)OBJECT_GET_DATA(u_data->io->draw_area, "sctp_graph_t");
+		ios=(sctp_graph_t *)g_object_get_data(G_OBJECT(u_data->io->draw_area), "sctp_graph_t");
 
 		if(!ios){
 			exit(10);
@@ -1167,7 +1070,7 @@ on_button_release (GtkWidget *widget _U_, GdkEventButton *event, struct sctp_uda
 				}
 			}
 
-			g_snprintf(label_string, 30, "(%.6lf, %u)", x_value, y_value);
+			g_snprintf(label_string, sizeof(label_string), "(%.6lf, %u)", x_value, y_value);
 
 			label_set = TRUE;
 
@@ -1179,35 +1082,27 @@ on_button_release (GtkWidget *widget _U_, GdkEventButton *event, struct sctp_uda
 				position = event->x + 5;
 
 
-#if GTK_MAJOR_VERSION < 2
-			lwidth=gdk_string_width(font, label_string);
-		                            gdk_draw_string(u_data->io->pixmap,font,text_color,
-					    (gint)position,
-					    (gint)(event->y-10),
-		                            label_string);
-#else
 			memcpy(label_string,(gchar *)g_locale_to_utf8(label_string, -1 , NULL, NULL, NULL), 15);
 			pango_layout_set_text(layout, label_string, -1);
 			pango_layout_get_pixel_size(layout, &lwidth, NULL);
 
 			gdk_draw_layout(u_data->io->pixmap,text_color,
-					    (gint)position,
-					    (gint)(event->y-10),
-					    layout);
-	#endif
+			                (gint)position,
+			                (gint)(event->y-10),
+			                layout);
 
-			ios=(sctp_graph_t *)OBJECT_GET_DATA(u_data->io->draw_area, "sctp_graph_t");
+			ios=(sctp_graph_t *)g_object_get_data(G_OBJECT(u_data->io->draw_area), "sctp_graph_t");
 
 			if(!ios){
 				exit(10);
 			}
 			gdk_draw_pixmap(u_data->io->draw_area->window,
-		                    u_data->io->draw_area->style->fg_gc[GTK_WIDGET_STATE(u_data->io->draw_area)],
-		                    ios->pixmap,
-		                    0, 0,
-		                    0, 0,
-		                    u_data->io->draw_area->allocation.width,
-		                    u_data->io->draw_area->allocation.height);
+			                u_data->io->draw_area->style->fg_gc[GTK_WIDGET_STATE(u_data->io->draw_area)],
+			                ios->pixmap,
+			                0, 0,
+			                0, 0,
+			                u_data->io->draw_area->allocation.width,
+			                u_data->io->draw_area->allocation.height);
 		}
 	}
 	return TRUE;
@@ -1239,7 +1134,7 @@ static void init_sctp_graph_window(struct sctp_udata *u_data)
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 	gtk_container_set_border_width(GTK_CONTAINER(hbox), 10);
 	gtk_button_box_set_layout(GTK_BUTTON_BOX (hbox), GTK_BUTTONBOX_SPREAD);
-	gtk_button_box_set_spacing(GTK_BUTTON_BOX (hbox), 0);
+	gtk_box_set_spacing(GTK_BOX (hbox), 0);
 	gtk_box_set_child_packing(GTK_BOX(vbox), hbox, FALSE, FALSE, 0, GTK_PACK_START);
 	gtk_widget_show(hbox);
 
@@ -1247,40 +1142,40 @@ static void init_sctp_graph_window(struct sctp_udata *u_data)
 	gtk_box_pack_start(GTK_BOX(hbox), sack_bt, FALSE, FALSE, 0);
 	gtk_widget_show(sack_bt);
 
-	gtk_signal_connect(GTK_OBJECT(sack_bt), "clicked", (GtkSignalFunc)on_sack_bt, u_data);
+	g_signal_connect(sack_bt, "clicked", G_CALLBACK(on_sack_bt), u_data);
 
 	tsn_bt = gtk_button_new_with_label ("Show TSNs");
 	gtk_box_pack_start(GTK_BOX(hbox), tsn_bt, FALSE, FALSE, 0);
 	gtk_widget_show(tsn_bt);
-	SIGNAL_CONNECT(tsn_bt, "clicked", on_tsn_bt, u_data);
+	g_signal_connect(tsn_bt, "clicked", G_CALLBACK(on_tsn_bt), u_data);
 
 	both_bt = gtk_button_new_with_label ("Show both");
 	gtk_box_pack_start(GTK_BOX(hbox), both_bt, FALSE, FALSE, 0);
 	gtk_widget_show(both_bt);
-	SIGNAL_CONNECT(both_bt, "clicked", on_both_bt, u_data);
+	g_signal_connect(both_bt, "clicked", G_CALLBACK(on_both_bt), u_data);
 
 	zoomin_bt = gtk_button_new_with_label ("Zoom in");
 	gtk_box_pack_start(GTK_BOX(hbox), zoomin_bt, FALSE, FALSE, 0);
 	gtk_widget_show(zoomin_bt);
-	SIGNAL_CONNECT(zoomin_bt, "clicked", on_zoomin_bt, u_data);
+	g_signal_connect(zoomin_bt, "clicked", G_CALLBACK(on_zoomin_bt), u_data);
 	tooltip_in = gtk_tooltips_new();
 	gtk_tooltips_set_tip(tooltip_in, zoomin_bt, "Zoom in the area you have selected", NULL);
 
 	zoomout_bt = gtk_button_new_with_label ("Zoom out");
 	gtk_box_pack_start(GTK_BOX(hbox), zoomout_bt, FALSE, FALSE, 0);
 	gtk_widget_show(zoomout_bt);
-	SIGNAL_CONNECT(zoomout_bt, "clicked", on_zoomout_bt, u_data);
+	g_signal_connect(zoomout_bt, "clicked", G_CALLBACK(on_zoomout_bt), u_data);
 	tooltip_out = gtk_tooltips_new();
 	gtk_tooltips_set_tip(tooltip_out, zoomout_bt, "Zoom out one step", NULL);
 	gtk_widget_set_sensitive(zoomout_bt, FALSE);
 
-	bt_close = BUTTON_NEW_FROM_STOCK(GTK_STOCK_CLOSE);
+	bt_close = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
 	gtk_box_pack_start(GTK_BOX(hbox), bt_close, FALSE, FALSE, 0);
 	gtk_widget_show(bt_close);
-	SIGNAL_CONNECT(bt_close, "clicked", sctp_graph_close_cb, u_data);
+	g_signal_connect(bt_close, "clicked", G_CALLBACK(sctp_graph_close_cb), u_data);
 
-	gtk_signal_connect(GTK_OBJECT(u_data->io->draw_area),"button_press_event",(GtkSignalFunc)on_button_press, u_data);
-	gtk_signal_connect(GTK_OBJECT(u_data->io->draw_area),"button_release_event",(GtkSignalFunc)on_button_release, u_data);
+	g_signal_connect(u_data->io->draw_area,"button_press_event",G_CALLBACK(on_button_press), u_data);
+	g_signal_connect(u_data->io->draw_area,"button_release_event",G_CALLBACK(on_button_release), u_data);
 	gtk_widget_set_events(u_data->io->draw_area, GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_EXPOSURE_MASK);
 
 	gtk_widget_show(u_data->io->window);
@@ -1366,14 +1261,14 @@ static void create_draw_area(GtkWidget *box, struct sctp_udata *u_data)
 {
 
 	u_data->io->draw_area=gtk_drawing_area_new();
-	SIGNAL_CONNECT(u_data->io->draw_area, "destroy", quit, u_data);
-	OBJECT_SET_DATA(u_data->io->draw_area, "sctp_graph_t", u_data->io);
+	g_signal_connect(u_data->io->draw_area, "destroy", G_CALLBACK(quit), u_data);
+	g_object_set_data(G_OBJECT(u_data->io->draw_area), "sctp_graph_t", u_data->io);
 
-	WIDGET_SET_SIZE(u_data->io->draw_area, u_data->io->pixmap_width, u_data->io->pixmap_height);
+	gtk_widget_set_size_request(u_data->io->draw_area, u_data->io->pixmap_width, u_data->io->pixmap_height);
 
 	/* signals needed to handle backing pixmap */
-	SIGNAL_CONNECT(u_data->io->draw_area, "expose_event", expose_event, NULL);
-	SIGNAL_CONNECT(u_data->io->draw_area, "configure_event", configure_event, u_data);
+	g_signal_connect(u_data->io->draw_area, "expose_event", G_CALLBACK(expose_event), NULL);
+	g_signal_connect(u_data->io->draw_area, "configure_event", G_CALLBACK(configure_event), u_data);
 
 	gtk_widget_show(u_data->io->draw_area);
 	gtk_box_pack_start(GTK_BOX(box), u_data->io->draw_area, TRUE, TRUE, 0);
