@@ -1,7 +1,7 @@
 /* sip_stat.c
  * sip_stat   2004 Martin Mathieson
  *
- * $Id: sip_stat.c 26641 2008-10-31 09:53:56Z stig $
+ * $Id: sip_stat.c 27556 2009-02-26 21:11:06Z etxrab $
  * Copied from http_stat.c
  *
  * Wireshark - Network traffic analyzer
@@ -56,8 +56,14 @@ typedef struct _sip_stats_t {
     GHashTable  *hash_requests;
     guint32	    packets;        /* number of sip packets, including continuations */
     guint32     resent_packets;
+	guint32		average_setup_time;
+	guint32		max_setup_time;
+	guint32		min_setup_time;
+	guint32		no_of_completed_calls;
+	guint64		total_setup_time;
     GtkWidget   *packets_label;
     GtkWidget   *resent_label;
+    GtkWidget   *average_setup_time_label;
 
     GtkWidget   *request_box;		/* container for INVITE, ... */
 
@@ -221,7 +227,7 @@ sip_draw_hash_requests(gchar *key _U_ , sip_request_method_t *data, gchar * unus
     else
     {
         /* Update existing label */
-        gtk_label_set(GTK_LABEL(data->widget), string_buff);
+         gtk_label_set_text(GTK_LABEL(data->widget), string_buff);
     }
 }
 
@@ -305,7 +311,7 @@ sip_draw_hash_responses(gint * key _U_ , sip_response_code_t *data, gchar * unus
     {
         /* Just update the existing label string */
         g_snprintf(string_buff, sizeof(string_buff), "%9d", data->packets);
-        gtk_label_set(GTK_LABEL(data->widget), string_buff);
+         gtk_label_set_text(GTK_LABEL(data->widget), string_buff);
     }
 }
 
@@ -338,6 +344,11 @@ sipstat_reset(void *psp)
     {
     	sp->packets = 0;
         sp->resent_packets = 0;
+		sp->average_setup_time = 0;
+		sp->max_setup_time = 0;
+		sp->max_setup_time = 0;
+		sp->no_of_completed_calls = 0;
+		sp->total_setup_time = 0;
         g_hash_table_foreach(sp->hash_responses, (GHFunc)sip_reset_hash_responses, NULL);
         g_hash_table_foreach(sp->hash_requests, (GHFunc)sip_reset_hash_requests, NULL);
     }
@@ -359,6 +370,27 @@ sipstat_packet(void *psp, packet_info *pinfo _U_, epan_dissect_t *edt _U_, const
         sp->resent_packets++;
     }
 
+	/* Calculate average setup time */
+	if (value->setup_time){
+		sp->no_of_completed_calls++;
+		/* Check if it's the first value */
+		if ( sp->total_setup_time == 0 ){
+			sp->average_setup_time = value->setup_time;
+			sp->total_setup_time = value->setup_time;
+			sp->max_setup_time = value->setup_time;
+			sp->min_setup_time = value->setup_time;
+		}else{
+			sp->total_setup_time = sp->total_setup_time + value->setup_time;
+			if (sp->max_setup_time < value->setup_time){
+				sp->max_setup_time = value->setup_time;
+			}
+			if (sp->min_setup_time > value->setup_time){
+				sp->min_setup_time = value->setup_time;
+			}
+			/* Calculate average */
+			sp->average_setup_time = (guint32)(sp->total_setup_time / sp->no_of_completed_calls); 
+		}
+	}
 
     /* Looking at both requests and responses */
     if (value->response_code != 0)
@@ -460,18 +492,23 @@ sipstat_draw(void *psp)
     /* Set summary label */
     g_snprintf(string_buff, sizeof(string_buff),
                 "SIP stats (%d packets)", sp->packets);
-    gtk_label_set(GTK_LABEL(sp->packets_label), string_buff);
+     gtk_label_set_text(GTK_LABEL(sp->packets_label), string_buff);
 
     /* Set resend count label */
     g_snprintf(string_buff, sizeof(string_buff),
                 "(%d resent packets)", sp->resent_packets);
-    gtk_label_set(GTK_LABEL(sp->resent_label), string_buff);
+     gtk_label_set_text(GTK_LABEL(sp->resent_label), string_buff);
 
     /* Draw responses and requests from their tables */
     g_hash_table_foreach(sp->hash_responses, (GHFunc)sip_draw_hash_responses, NULL);
     g_hash_table_foreach(sp->hash_requests,  (GHFunc)sip_draw_hash_requests, NULL);
 
-    gtk_widget_show_all(sp->win);
+    /* Set resend count label */
+    g_snprintf(string_buff, sizeof(string_buff),
+                "Average setup time %d ms\n Min %d ms\n Max %d ms", sp->average_setup_time, sp->min_setup_time, sp->max_setup_time);
+     gtk_label_set_text(GTK_LABEL(sp->average_setup_time_label), string_buff);
+
+	gtk_widget_show_all(sp->win);
 }
 
 
@@ -619,6 +656,13 @@ gtk_sipstat_init(const char *optarg, void *userdata _U_)
     sp->request_box = gtk_vbox_new(FALSE, 10);
     gtk_container_add(GTK_CONTAINER(request_fr), sp->request_box);
 
+    sp->average_setup_time = 0;
+	sp->max_setup_time =0;
+	sp->min_setup_time =0;
+    sp->average_setup_time_label = gtk_label_new("(Not calculated)");
+    gtk_container_add(GTK_CONTAINER(main_vb), sp->average_setup_time_label);
+    gtk_widget_show(sp->average_setup_time_label);
+
 
     /* Register this tap listener now */
     error_string = register_tap_listener("sip",
