@@ -1,7 +1,7 @@
 /* voip_calls_dlg.c
  * VoIP calls summary addition for Wireshark
  *
- * $Id: voip_calls_dlg.c 26839 2008-11-25 06:41:15Z jake $
+ * $Id: voip_calls_dlg.c 27696 2009-03-11 06:43:46Z etxrab $
  *
  * Copyright 2004, Ericsson , Spain
  * By Francisco Alcoba <francisco.alcoba@ericsson.com>
@@ -187,7 +187,7 @@ static void add_to_clist(voip_calls_info_t* strinfo)
 		plurality(calls_nb, "Call", "Calls"),
 		calls_ns,
 		plurality(calls_ns, "Call", "Calls"));
-	gtk_label_set(GTK_LABEL(top_label), label_text);
+	 gtk_label_set_text(GTK_LABEL(top_label), label_text);
 
 	/* Update the status label with the number of total messages */
         g_snprintf(label_text, 256,
@@ -196,7 +196,7 @@ static void add_to_clist(voip_calls_info_t* strinfo)
 			voip_calls_get_info()->start_packets, 
 			voip_calls_get_info()->completed_calls,
 			voip_calls_get_info()->rejected_calls);
-        gtk_label_set(GTK_LABEL(status_label), label_text);
+         gtk_label_set_text(GTK_LABEL(status_label), label_text);
 }
 
 
@@ -223,6 +223,8 @@ static void voip_calls_remove_tap_listener(void)
 	remove_tap_listener_mgcp_calls();
 	remove_tap_listener_actrace_calls();
 	remove_tap_listener_t38();
+	remove_tap_listener_skinny_calls();
+	remove_tap_listener_iax2_calls();
 }
 
 /****************************************************************************/
@@ -375,6 +377,8 @@ voip_calls_on_filter                    (GtkButton       *button _U_,
 			case TEL_BSSMAP:
 			case TEL_RANAP:
 			case VOIP_UNISTIM:
+			case VOIP_SKINNY:
+			case VOIP_IAX2:
 			case VOIP_COMMON:
 				/* XXX - not supported */
 				break;
@@ -485,7 +489,7 @@ voip_calls_on_select_row(GtkCList *clist,
             plurality(calls_nb, "Call", "Calls"),
 			calls_ns,
 			plurality(calls_ns, "Call", "Calls"));
-	gtk_label_set(GTK_LABEL(top_label), label_text);
+	 gtk_label_set_text(GTK_LABEL(top_label), label_text);
 
 
 	if 	(calls_ns > 0) {
@@ -543,7 +547,7 @@ voip_calls_on_unselect_row(GtkCList *clist,
             plurality(calls_nb, "Call", "Calls"),
 			calls_ns,
 			plurality(calls_ns, "Call", "Calls"));
-	gtk_label_set(GTK_LABEL(top_label), label_text);
+	 gtk_label_set_text(GTK_LABEL(top_label), label_text);
 
 	if 	(calls_ns > 0) {
 		gtk_widget_set_sensitive(bt_filter, TRUE);
@@ -835,7 +839,7 @@ void voip_calls_dlg_update(GList *list)
 			voip_calls_get_info()->start_packets, 
 			voip_calls_get_info()->completed_calls,
 			voip_calls_get_info()->rejected_calls);
-		gtk_label_set(GTK_LABEL(status_label), label_text);
+		 gtk_label_set_text(GTK_LABEL(status_label), label_text);
 
 		gtk_clist_freeze(GTK_CLIST(clist));
 		gtk_clist_clear(GTK_CLIST(clist));
@@ -853,7 +857,7 @@ void voip_calls_dlg_update(GList *list)
 			plurality(calls_nb, "Call", "Calls"),
 			calls_ns,
 			plurality(calls_ns, "Call", "Calls"));
-		gtk_label_set(GTK_LABEL(top_label), label_text);
+		 gtk_label_set_text(GTK_LABEL(top_label), label_text);
 	}
 
 	last_list = list;
@@ -924,6 +928,8 @@ voip_calls_init_tap(const char *dummy _U_, void* userdata _U_)
 	mgcp_calls_init_tap();
 	actrace_calls_init_tap();
 	t38_init_tap();
+	skinny_calls_init_tap();
+	iax2_calls_init_tap();
 
 	/* create dialog box if necessary */
 	if (voip_calls_dlg == NULL) {
