@@ -1,6 +1,6 @@
 /* Combine two dump files, either by appending or by merging by timestamp
  *
- * $Id: mergecap.c 24179 2008-01-24 19:09:30Z stig $
+ * $Id: mergecap.c 25354 2008-05-22 15:46:27Z morriss $
  *
  * Written by Scott Renfro <scott@renfro.org> based on
  * editcap by Richard Sharpe and Guy Harris
@@ -33,7 +33,7 @@
 
 #include "svnversion.h"
 #include "merge.h"
-#include "file_util.h"
+#include "wsutil/file_util.h"
 
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
@@ -308,19 +308,19 @@ main(int argc, char *argv[])
   }
 
   /* open the outfile */
-  if (strncmp(out_filename, "-", 2) == 0) {  
+  if (strncmp(out_filename, "-", 2) == 0) {
     /* use stdout as the outfile */
     out_fd = 1 /*stdout*/;
   } else {
     /* open the outfile */
-    out_fd = eth_open(out_filename, O_WRONLY | O_CREAT | O_TRUNC | O_BINARY, 0644);
+    out_fd = ws_open(out_filename, O_WRONLY | O_CREAT | O_TRUNC | O_BINARY, 0644);
     if (out_fd == -1) {
       fprintf(stderr, "mergecap: Couldn't open output file %s: %s\n",
               out_filename, strerror(errno));
       exit(1);
     }
-  }  
-    
+  }
+
   /* prepare the outfile */
   pdh = wtap_dump_fdopen(out_fd, file_type, frame_type, snaplen, FALSE /* compressed */, &open_err);
   if (pdh == NULL) {
