@@ -5,7 +5,7 @@
  *
  * Author: Lu Pan <lu.pan@intel.com>
  *
- * $Id: mac_hd_generic_decoder.c 24180 2008-01-24 19:30:43Z stig $
+ * $Id: mac_hd_generic_decoder.c 26103 2008-08-29 01:40:32Z wmeier $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -46,7 +46,6 @@
 #include <string.h>
 #include <glib.h>
 #include <epan/packet.h>
-#include <epan/prefs.h>
 #include <epan/address.h>
 #include <epan/reassemble.h>
 #include "crc.h"
@@ -1812,9 +1811,9 @@ void dissect_mac_header_generic_decoder(tvbuff_t *tvb, packet_info *pinfo, proto
 			/* defragment first if it is fragmented */
 			if (frag_type == NO_FRAG)
 			{	/* not fragmented payload */
-				payload_tvb =  tvb_new_subset(tvb, offset, length, length);
-				payload_length = length;
-				new_payload_len = length;
+				payload_tvb =  tvb_new_subset(tvb, offset, frag_len, frag_len);
+				payload_length = frag_len;
+				new_payload_len = frag_len;
 			}
 			else	/* fragmented payload */
 			{	/* add the frag */
