@@ -1,6 +1,6 @@
 /* rawshark.c
  *
- * $Id: rawshark.c 26113 2008-09-02 11:17:03Z wmeier $
+ * $Id: rawshark.c 27456 2009-02-15 21:47:57Z guy $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -87,7 +87,6 @@
 #include "clopts_common.h"
 #include "cmdarg_err.h"
 #include "version_info.h"
-#include <epan/conversation.h>
 #include <epan/plugins.h>
 #include "register.h"
 #include "conditions.h"
@@ -155,6 +154,7 @@ static void open_failure_message(const char *filename, int err,
     gboolean for_writing);
 static void failure_message(const char *msg_format, va_list ap);
 static void read_failure_message(const char *filename, int err);
+static void write_failure_message(const char *filename, int err);
 static void protocolinfo_init(char *field);
 static gboolean parse_field_string_format(char *format);
 
@@ -246,6 +246,7 @@ raw_pipe_open(const char *pipe_name)
   struct stat pipe_stat;
 #else
   char *pncopy, *pos;
+  DWORD err;
   wchar_t *err_str;
   HANDLE hPipe = NULL;
 #endif
@@ -268,9 +269,8 @@ raw_pipe_open(const char *pipe_name)
   } else {
 #ifndef _WIN32
     if (ws_stat(pipe_name, &pipe_stat) < 0) {
-      g_log(LOG_DOMAIN_CAPTURE_CHILD, G_LOG_LEVEL_DEBUG,
-        "The capture session could not be initiated "
-        "due to error on pipe: %s", strerror(errno));
+      fprintf(stderr, "rawshark: The pipe %s could not be checked: %s\n",
+              pipe_name, strerror(errno));
       return -1;
     }
     if (! S_ISFIFO(pipe_stat.st_mode)) {
@@ -281,17 +281,15 @@ raw_pipe_open(const char *pipe_name)
          */
       } else
       {
-        g_log(LOG_DOMAIN_CAPTURE_CHILD, G_LOG_LEVEL_DEBUG,
-            "The capture session could not be initiated because\n"
-            "\"%s\" is neither an interface nor a pipe", pipe_name);
+        fprintf(stderr, "rawshark: \"%s\" is neither an interface nor a pipe\n",
+	        pipe_name);
       }
       return -1;
     }
     rfd = ws_open(pipe_name, O_RDONLY | O_NONBLOCK, 0000 /* no creation so don't matter */);
     if (rfd == -1) {
-        g_log(LOG_DOMAIN_CAPTURE_CHILD, G_LOG_LEVEL_DEBUG,
-          "The capture session could not be initiated "
-          "due to error on pipe open: %s", strerror(errno));
+        fprintf(stderr, "rawshark: \"%s\" could not be opened: %s\n",
+                pipe_name, strerror(errno));
       return -1;
     }
 #else /* _WIN32 */
@@ -309,9 +307,8 @@ raw_pipe_open(const char *pipe_name)
     g_free(pncopy);
 
     if (!pos) {
-      g_log(LOG_DOMAIN_CAPTURE_CHILD, G_LOG_LEVEL_DEBUG,
-          "The capture session could not be initiated because\n"
-          "\"%s\" is neither an interface nor a pipe", pipe_name);
+      fprintf(stderr, "rawshark: \"%s\" is neither an interface nor a pipe\n",
+              pipe_name);
       return -1;
     }
 
@@ -323,24 +320,22 @@ raw_pipe_open(const char *pipe_name)
       if (hPipe != INVALID_HANDLE_VALUE)
         break;
 
-      if (GetLastError() != ERROR_PIPE_BUSY) {
+      err = GetLastError();
+      if (err != ERROR_PIPE_BUSY) {
         FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_ALLOCATE_BUFFER,
-          NULL, GetLastError(), 0, (LPTSTR) &err_str, 0, NULL);
-          g_log(LOG_DOMAIN_CAPTURE_CHILD, G_LOG_LEVEL_DEBUG,
-            "The capture session on \"%s\" could not be initiated "
-            "due to error on pipe open: pipe busy: %s (error %d)",
-	    pipe_name, utf_16to8(err_str), GetLastError());
+          NULL, err, 0, (LPTSTR) &err_str, 0, NULL);
+        fprintf(stderr, "rawshark: \"%s\" could not be opened: %s (error %d)\n",
+	    pipe_name, utf_16to8(err_str), err);
         LocalFree(err_str);
         return -1;
       }
 
       if (!WaitNamedPipe(utf_8to16(pipe_name), 30 * 1000)) {
+      	err = GetLastError();
         FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_ALLOCATE_BUFFER,
-          NULL, GetLastError(), 0, (LPTSTR) &err_str, 0, NULL);
-          g_log(LOG_DOMAIN_CAPTURE_CHILD, G_LOG_LEVEL_DEBUG,
-            "The capture session could not be initiated "
-            "due to error on pipe open: %s (error %d)",
-	    utf_16to8(err_str), GetLastError());
+          NULL, err, 0, (LPTSTR) &err_str, 0, NULL);
+        fprintf(stderr, "rawshark: \"%s\" could not be waited for: %s (error %d)\n",
+	    pipe_name, utf_16to8(err_str), err);
         LocalFree(err_str);
         return -1;
       }
@@ -348,9 +343,8 @@ raw_pipe_open(const char *pipe_name)
 
     rfd = _open_osfhandle((long) hPipe, _O_RDONLY);
     if (rfd == -1) {
-      g_log(LOG_DOMAIN_CAPTURE_CHILD, G_LOG_LEVEL_DEBUG,
-          "The capture session could not be initiated "
-          "due to error on pipe open: %s", strerror(errno));
+      fprintf(stderr, "rawshark: \"%s\" could not be opened: %s\n",
+              pipe_name, strerror(errno));
       return -1;
     }
 #endif /* _WIN32 */
@@ -368,7 +362,9 @@ raw_pipe_open(const char *pipe_name)
 static gboolean
 set_link_type(const char *lt_arg) {
   char *spec_ptr = strchr(lt_arg, ':');
+  char *p;
   int dlt_val;
+  long val;
   dissector_handle_t dhandle;
   GString *pref_str;
 
@@ -378,21 +374,20 @@ set_link_type(const char *lt_arg) {
   spec_ptr++;
 
   if (strncmp(lt_arg, "encap:", strlen("encap:")) == 0) {
-#ifdef HAVE_PCAP_DATALINK_NAME_TO_VAL
     dlt_val = linktype_name_to_val(spec_ptr);
-    if (dlt_val >= 0) {
-      encap = dlt_val;
-      return TRUE;
-    }
-#endif
-    dlt_val = strtol(spec_ptr, NULL, 10);
-    if (errno != EINVAL && dlt_val >= 0) {
-      encap = wtap_pcap_encap_to_wtap_encap(dlt_val);
-      if (encap == WTAP_ENCAP_UNKNOWN) {
+    if (dlt_val == -1) {
+      errno = 0;
+      val = strtol(spec_ptr, &p, 10);
+      if (p == spec_ptr || *p != '\0' || errno != 0 || val > INT_MAX) {
         return FALSE;
       }
-      return TRUE;
+      dlt_val = (int)val;
     }
+    encap = wtap_pcap_encap_to_wtap_encap(dlt_val);
+    if (encap == WTAP_ENCAP_UNKNOWN) {
+      return FALSE;
+    }
+    return TRUE;
   } else if (strncmp(lt_arg, "proto:", strlen("proto:")) == 0) {
     dhandle = find_dissector(spec_ptr);
     if (dhandle) {
@@ -498,7 +493,8 @@ main(int argc, char *argv[])
      dissectors, and we must do it before we read the preferences, in
      case any dissectors register preferences. */
   epan_init(register_all_protocols, register_all_protocol_handoffs, NULL, NULL,
-            failure_message, open_failure_message, read_failure_message);
+            failure_message, open_failure_message, read_failure_message,
+            write_failure_message);
 
   /* Now register the preferences for any non-dissector modules.
      We must do that before we read the preferences as well. */
@@ -800,7 +796,7 @@ main(int argc, char *argv[])
         epan_cleanup();
         exit(2);
       }
-	  n_rfcodes++;
+      n_rfcodes++;
     }
   }
 
@@ -884,7 +880,7 @@ raw_pipe_read(struct wtap_pkthdr *phdr, guchar * pd, int *err, gchar **err_info,
   struct pcap_pkthdr hdr;
   int bytes_read = 0;
   int bytes_needed = sizeof(struct pcap_pkthdr);
-  gchar err_str[1024+1];
+  static gchar err_str[100];
 
   /* Copied from capture_loop.c */
   while (bytes_needed > 0) {
@@ -916,7 +912,7 @@ raw_pipe_read(struct wtap_pkthdr *phdr, guchar * pd, int *err, gchar **err_info,
 #endif
   if (bytes_needed > WTAP_MAX_PACKET_SIZE) {
     *err = WTAP_ERR_BAD_RECORD;
-    g_snprintf(err_str, 1024, "Bad packet length: %d (%04x)", bytes_needed, bytes_needed);
+    g_snprintf(err_str, 100, "Bad packet length: %d (%04x)", bytes_needed, bytes_needed);
     *err_info = err_str;
     return FALSE;
   }
@@ -1631,6 +1627,16 @@ read_failure_message(const char *filename, int err)
           filename, strerror(err));
 }
 
+/*
+ * Write errors are reported with an console message in Rawshark.
+ */
+static void
+write_failure_message(const char *filename, int err)
+{
+  cmdarg_err("An error occurred while writing to the file \"%s\": %s.",
+          filename, strerror(err));
+}
+
 /*
  * Report an error in command-line arguments.
  */
