@@ -1,14 +1,19 @@
 #!/bin/bash
 #
-# $Id: win32-setup.sh 27226 2009-01-13 23:21:19Z etxrab $
+# $Id: win32-setup.sh 27796 2009-03-19 06:42:46Z etxrab $
 
 # This MUST be in the form
 #   http://anonsvn.wireshark.org/wireshark-win32-libs/tags/<date>/packages
+# or
+#   http://anonsvn.wireshark.org/wireshark-win64-libs/tags/<date>/packages
 # in order to provide backward compatibility with older trees (e.g. a
 # previous release or an older SVN checkout).
 # Save previous tag.
-DOWNLOAD_TAG="2009-01-13"
-DOWNLOAD_PREFIX="http://anonsvn.wireshark.org/wireshark-win32-libs/tags/$DOWNLOAD_TAG/packages/"
+DOWNLOAD_TAG=${DOWNLOAD_TAG:-"2009-03-19"}
+WIRESHARK_TARGET_PLATFORM=${WIRESHARK_TARGET_PLATFORM:-"win32"}
+
+DOWNLOAD_PREFIX="http://anonsvn.wireshark.org/wireshark-$WIRESHARK_TARGET_PLATFORM-libs/tags/$DOWNLOAD_TAG/packages/"
+
 TAG_FILE="current_tag.txt"
 
 # Set DOWNLOAD_PREFIX to /packages to test uploads before creating the tag.
@@ -76,6 +81,8 @@ case "$1" in
 	echo "Checking for required applications:"
 	which which > /dev/null 2>&1 || \
 		err_exit "Can't find 'which'.  Unable to proceed."
+
+	MISSING_APPS=        
 	for APP in $* ; do
 		APP_PATH=`cygpath --unix $APP`
 		if [ -x "$APP_PATH" -a ! -d "$APP_PATH" ] ; then
@@ -84,10 +91,17 @@ case "$1" in
 			APP_LOC=`which $APP_PATH 2> /dev/null`
 		fi
 		if [ "$APP_LOC" = "" ] ; then
-			err_exit "Can't find $APP. This is probably an optional cygwin package not yet installed. Try to install it using cygwin's setup.exe!"
+			MISSING_APPS="$MISSING_APPS $APP"
+		else
+			echo "	$APP: $APP_LOC $res"
 		fi
-		echo "	$APP: $APP_LOC $res"
 	done
+
+	if [ -n "$MISSING_APPS" ]; then
+		echo 
+		echo "Can't find: $MISSING_APPS"
+		err_exit "These are probably optional cygwin packages not yet installed. Try to install them using cygwin's setup.exe!"
+	fi
 	;;
 --libverify)
 	if [ -z "$2" -o -z "$3" -o -z "$4" ] ; then
