@@ -1,6 +1,6 @@
 /* file_access.c
  *
- * $Id: file_access.c 27176 2009-01-07 07:21:31Z jake $
+ * $Id: file_access.c 27683 2009-03-09 21:18:55Z gerald $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
@@ -79,6 +79,7 @@
 #include "btsnoop.h"
 #include "tnef.h"
 #include "dct3trace.h"
+#include "packetlogger.h"
 
 
 /* The open_file_* routines should return:
@@ -120,9 +121,11 @@ static wtap_open_routine_t open_routines_base[] = {
 	k12_open,
 	catapult_dct2000_open,
 	ber_open,
-	mpeg_open,
 	pcapng_open,
 	btsnoop_open,
+	packetlogger_open, /* This type does not have a magic number, but its
+			    * files are sometimes grabbed by mpeg_open. */
+	mpeg_open,
 	tnef_open,
 	dct3trace_open,
 	/* Files that don't have magic bytes at a fixed location,
@@ -995,11 +998,11 @@ static FILE *wtap_dump_file_fdopen(wtap_dumper *wdh _U_, int fd)
 #endif
 
 /* internally writing raw bytes (compressed or not) */
-size_t wtap_dump_file_write(wtap_dumper *wdh, const void *buf, unsigned bufsize)
+size_t wtap_dump_file_write(wtap_dumper *wdh, const void *buf, size_t bufsize)
 {
 #ifdef HAVE_LIBZ
 	if(wdh->compressed) {
-		return gzwrite(wdh->fh, buf, bufsize);
+		return gzwrite(wdh->fh, buf, (unsigned) bufsize);
 	} else
 #endif
 	{
