@@ -1,6 +1,6 @@
 /* netmon.c
  *
- * $Id: netmon.c 15534 2005-08-25 21:29:54Z ulfl $
+ * $Id: netmon.c 27683 2009-03-09 21:18:55Z gerald $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
@@ -105,8 +105,8 @@ struct netmon_atm_hdr {
 };
 
 static gboolean netmon_read(wtap *wth, int *err, gchar **err_info,
-    long *data_offset);
-static gboolean netmon_seek_read(wtap *wth, long seek_off,
+    gint64 *data_offset);
+static gboolean netmon_seek_read(wtap *wth, gint64 seek_off,
     union wtap_pseudo_header *pseudo_header, guchar *pd, int length,
     int *err, gchar **err_info);
 static gboolean netmon_read_atm_pseudoheader(FILE_T fh,
@@ -306,7 +306,7 @@ int netmon_open(wtap *wth, int *err, gchar **err_info)
 
 /* Read the next packet */
 static gboolean netmon_read(wtap *wth, int *err, gchar **err_info,
-    long *data_offset)
+    gint64 *data_offset)
 {
 	netmon_t *netmon = wth->capture.netmon;
 	guint32	packet_size = 0;
@@ -471,7 +471,7 @@ static gboolean netmon_read(wtap *wth, int *err, gchar **err_info,
 }
 
 static gboolean
-netmon_seek_read(wtap *wth, long seek_off,
+netmon_seek_read(wtap *wth, gint64 seek_off,
     union wtap_pseudo_header *pseudo_header, guchar *pd, int length,
     int *err, gchar **err_info _U_)
 {
@@ -778,7 +778,7 @@ static gboolean netmon_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
 	netmon->frame_table[netmon->frame_table_index] =
 	    htolel(netmon->frame_table_offset);
 	netmon->frame_table_index++;
-	netmon->frame_table_offset += hdr_size + phdr->caplen + atm_hdrsize;
+	netmon->frame_table_offset += (int) hdr_size + phdr->caplen + atm_hdrsize;
 
 	return TRUE;
 }
@@ -817,7 +817,7 @@ static gboolean netmon_dump_close(wtap_dumper *wdh, int *err)
 	case WTAP_FILE_NETMON_1_x:
 		magicp = netmon_1_x_magic;
 		magic_size = sizeof netmon_1_x_magic;
-		/* current NetMon version, for 1.x, is 1.1 */
+		/* NetMon file version, for 1.x, is 1.1 */
 		file_hdr.ver_major = 1;
 		file_hdr.ver_minor = 1;
 		break;
@@ -825,7 +825,10 @@ static gboolean netmon_dump_close(wtap_dumper *wdh, int *err)
 	case WTAP_FILE_NETMON_2_x:
 		magicp = netmon_2_x_magic;
 		magic_size = sizeof netmon_2_x_magic;
-		/* current NetMon version, for 2.x, is 2.0 */
+		/*
+		 * NetMon file version, for 2.x, is 2.0;
+		 * for 3.0, it's 2.1.
+		 */
 		file_hdr.ver_major = 2;
 		file_hdr.ver_minor = 0;
 		break;
