@@ -1,6 +1,6 @@
 /* ngsniffer.c
  *
- * $Id: ngsniffer.c 24307 2008-02-12 00:24:49Z stig $
+ * $Id: ngsniffer.c 27690 2009-03-10 17:38:24Z gerald $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
@@ -485,7 +485,7 @@ static gboolean ngsniffer_read_frame6(wtap *wth, gboolean is_random,
 static void set_pseudo_header_frame6(wtap *wth,
     union wtap_pseudo_header *pseudo_header, struct frame6_rec *frame6);
 static gboolean ngsniffer_read_rec_data(wtap *wth, gboolean is_random,
-    guchar *pd, int length, int *err);
+    guchar *pd, size_t length, int *err);
 static int infer_pkt_encap(const guint8 *pd, int len);
 static int fix_pseudo_header(int encap, const guint8 *pd, int len,
     union wtap_pseudo_header *pseudo_header);
@@ -496,7 +496,7 @@ static gboolean ngsniffer_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
 static gboolean ngsniffer_dump_close(wtap_dumper *wdh, int *err);
 static int SnifferDecompress( unsigned char * inbuf, size_t inlen,
         unsigned char * outbuf, size_t outlen, int *err );
-static int ng_file_read(void *buffer, size_t elementsize, size_t numelements,
+static gint64 ng_file_read(void *buffer, size_t elementsize, size_t numelements,
     wtap *wth, gboolean is_random, int *err);
 static int read_blob(FILE_T infile, ngsniffer_comp_stream_t *comp_stream,
     int *err);
@@ -1287,7 +1287,7 @@ static gboolean ngsniffer_seek_read(wtap *wth, gint64 seek_off,
 static int ngsniffer_read_rec_header(wtap *wth, gboolean is_random,
     guint16 *typep, guint16 *lengthp, int *err)
 {
-	int	bytes_read;
+	gint64	bytes_read;
 	char	record_type[2];
 	char	record_length[4]; /* only 1st 2 bytes are length */
 
@@ -1319,7 +1319,7 @@ static int ngsniffer_read_rec_header(wtap *wth, gboolean is_random,
 static gboolean ngsniffer_read_frame2(wtap *wth, gboolean is_random,
     struct frame2_rec *frame2, int *err)
 {
-	int bytes_read;
+	gint64 bytes_read;
 
 	/* Read the f_frame2_struct */
 	bytes_read = ng_file_read(frame2, 1, sizeof *frame2, wth, is_random,
@@ -1427,7 +1427,7 @@ static void set_pseudo_header_frame2(wtap *wth,
 static gboolean ngsniffer_read_frame4(wtap *wth, gboolean is_random,
     struct frame4_rec *frame4, int *err)
 {
-	int bytes_read;
+	gint64 bytes_read;
 
 	/* Read the f_frame4_struct */
 	bytes_read = ng_file_read(frame4, 1, sizeof *frame4, wth, is_random,
@@ -1696,7 +1696,7 @@ static void set_pseudo_header_frame4(union wtap_pseudo_header *pseudo_header,
 static gboolean ngsniffer_read_frame6(wtap *wth, gboolean is_random,
     struct frame6_rec *frame6, int *err)
 {
-	int bytes_read;
+	gint64 bytes_read;
 
 	/* Read the f_frame6_struct */
 	bytes_read = ng_file_read(frame6, 1, sizeof *frame6, wth, is_random,
@@ -1725,13 +1725,13 @@ static void set_pseudo_header_frame6(wtap *wth,
 }
 
 static gboolean ngsniffer_read_rec_data(wtap *wth, gboolean is_random,
-    guchar *pd, int length, int *err)
+    guchar *pd, size_t length, int *err)
 {
-	int	bytes_read;
+	gint64	bytes_read;
 
 	bytes_read = ng_file_read(pd, 1, length, wth, is_random, err);
 
-	if (bytes_read != length) {
+	if (bytes_read != (gint64) length) {
 		if (*err == 0)
 			*err = WTAP_ERR_SHORT_READ;
 		return FALSE;
@@ -2198,7 +2198,7 @@ static gboolean ngsniffer_dump_close(wtap_dumper *wdh, int *err)
    Parameters
       inbuf - buffer of compressed bytes from file, not including
          the preceding length word
-      inlen - length of inbuf in bytes
+      inlen - length of inbuf in bytes (max 64k)
       outbuf - decompressed contents, could contain a partial Sniffer
          record at the end.
       outlen - length of outbuf.
@@ -2220,6 +2220,10 @@ SnifferDecompress( unsigned char * inbuf, size_t inlen,
    int length;             /* length of RLE sequence or repeated string */
    int offset;             /* offset of string to repeat */
 
+   if (inlen > G_MAXUINT16) {
+      return ( -1 );
+   }
+
    bit_mask  = 0;  /* don't have any bits yet */
    while (1)
    {
@@ -2380,7 +2384,7 @@ SnifferDecompress( unsigned char * inbuf, size_t inlen,
          break;
    }
 
-   return ( pout - outbuf );  /* return length of expanded text */
+   return (int) ( pout - outbuf );  /* return length of expanded text */
 }
 
 /*
@@ -2397,18 +2401,18 @@ typedef struct {
 	gint64	blob_uncomp_offset;
 } blob_info_t;
 
-static int
+static gint64
 ng_file_read(void *buffer, size_t elementsize, size_t numelements, wtap *wth,
     gboolean is_random, int *err)
 {
     FILE_T infile;
     ngsniffer_comp_stream_t *comp_stream;
-    int copybytes = elementsize * numelements; /* bytes left to be copied */
-    int copied_bytes = 0; /* bytes already copied */
+    size_t copybytes = elementsize * numelements; /* bytes left to be copied */
+    gint64 copied_bytes = 0; /* bytes already copied */
     unsigned char *outbuffer = buffer; /* where to write next decompressed data */
     blob_info_t *blob;
-    int bytes_to_copy;
-    int bytes_left;
+    size_t bytes_to_copy;
+    size_t bytes_left;
 
     if (is_random) {
 	infile = wth->random_fh;
@@ -2421,7 +2425,7 @@ ng_file_read(void *buffer, size_t elementsize, size_t numelements, wtap *wth,
     if (wth->file_type == WTAP_FILE_NGSNIFFER_UNCOMPRESSED) {
 	errno = WTAP_ERR_CANT_READ;
 	copied_bytes = file_read(buffer, 1, copybytes, infile);
-	if (copied_bytes != copybytes)
+	if ((size_t) copied_bytes != copybytes)
 	    *err = file_error(infile);
 	return copied_bytes;
     }
@@ -2497,7 +2501,7 @@ ng_file_read(void *buffer, size_t elementsize, size_t numelements, wtap *wth,
 	copybytes -= bytes_to_copy;
 	copied_bytes += bytes_to_copy;
 	outbuffer += bytes_to_copy;
-	comp_stream->nextout += bytes_to_copy;
+	comp_stream->nextout += (int) bytes_to_copy;
 	comp_stream->uncomp_offset += bytes_to_copy;
     }
     return copied_bytes;
@@ -2508,7 +2512,7 @@ ng_file_read(void *buffer, size_t elementsize, size_t numelements, wtap *wth,
 static int
 read_blob(FILE_T infile, ngsniffer_comp_stream_t *comp_stream, int *err)
 {
-    size_t in_len;
+    int in_len;
     size_t read_len;
     unsigned short blob_len;
     gint16 blob_len_host;
@@ -2539,7 +2543,7 @@ read_blob(FILE_T infile, ngsniffer_comp_stream_t *comp_stream, int *err)
     /* Read the blob */
     errno = WTAP_ERR_CANT_READ;
     read_len = file_read(file_inbuf, 1, in_len, infile);
-    if (in_len != read_len) {
+    if ((size_t) in_len != read_len) {
 	*err = file_error(infile);
 	return -1;
     }
