@@ -1,6 +1,6 @@
 /* vms.c
  *
- * $Id: vms.c 25254 2008-05-08 05:16:33Z etxrab $
+ * $Id: vms.c 27685 2009-03-09 22:37:55Z gerald $
  *
  * Wiretap Library
  * Copyright (c) 2001 by Marc Milgram <ethereal@mmilgram.NOSPAMmail.net>
@@ -222,7 +222,7 @@ static gboolean vms_check_file_type(wtap *wth, int *err)
     }
     if (file_gets(buf, VMS_LINE_LENGTH, wth->fh) != NULL) {
       
-      reclen = strlen(buf);
+      reclen = (guint) strlen(buf);
       if (reclen < strlen(VMS_HDR_MAGIC_STR1) ||
 	  reclen < strlen(VMS_HDR_MAGIC_STR2) || 
 	  reclen < strlen(VMS_HDR_MAGIC_STR3)) {
@@ -435,7 +435,7 @@ parse_vms_rec_hdr(wtap *wth, FILE_T fh, int *err, gchar **err_info)
     if (wth) {
         p = strstr(months, mon);
         if (p)
-            tm.tm_mon = (p - months) / 3;
+            tm.tm_mon = (int) (p - months) / 3;
         tm.tm_year -= 1900;
 
 	tm.tm_isdst = -1;
