@@ -1,6 +1,6 @@
 /* wtap.h
  *
- * $Id: wtap.h 27176 2009-01-07 07:21:31Z jake $
+ * $Id: wtap.h 27463 2009-02-16 07:24:04Z sfisher $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
@@ -207,6 +207,7 @@ extern "C" {
 #define WTAP_ENCAP_TNEF                         114
 #define WTAP_ENCAP_USB_LINUX_MMAPPED            115
 #define WTAP_ENCAP_GSM_UM                       116
+#define WTAP_ENCAP_DPNSS                        117
 
 #define WTAP_NUM_ENCAP_TYPES                    wtap_get_num_encap_types()
 
@@ -268,6 +269,7 @@ extern "C" {
 #define WTAP_FILE_X2E_XORAYA                    52
 #define WTAP_FILE_TNEF                          53
 #define WTAP_FILE_DCT3TRACE                     54
+#define WTAP_FILE_PACKETLOGGER                  55
 
 #define WTAP_NUM_FILE_TYPES                     wtap_get_num_file_types()
 
@@ -426,9 +428,14 @@ struct ascend_phdr {
 	guint32	task;			/* Task number */
 };
 
+/* Also defined in epan/packet_info.h */
+#define P2P_DIR_UNKNOWN	-1
+#define P2P_DIR_SENT	0
+#define P2P_DIR_RECV	1
+
 /* Packet "pseudo-header" for point-to-point links with direction flags. */
 struct p2p_phdr {
-	gboolean	sent; /* TRUE=sent, FALSE=received */
+	int	sent; /* TRUE=sent, FALSE=received, -1=unknown*/
 };
 
 /*
